/* finger_conf.c 
 * tool to create map file for Active Badge system uids
 * (c) 1998 Dariusz Zbik <zbik@ernie.icslab.agh.edu.pl>
 */

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>
#include <fcntl.h>
#include <db.h>

#include <stdlib.h>
#include <string.h>

#include "../consts.h"

void usage(char *progname);
void fill_base(FILE *input, DB *map_db);

int main(int argc, char *argv[])
{
  DB *map_db = NULL;
  FILE *input;

  if (argc<2)
  {
    usage(argv[0]);
    exit(1);
  }
      
  if ( !(input=fopen(argv[1], "r")) )
  {
    perror("Unable to open input file");
    exit(2);
  }

  if ( !(map_db = dbopen(MAPFILENAME, O_CREAT | O_TRUNC | O_RDWR,
			S_IROTH | S_IRGRP | S_IWGRP | S_IRUSR | S_IWUSR ,
			DB_HASH, NULL)) )
  {
    perror("Unable to create database");
    exit(3);
  }

  fill_base(input, map_db);
  
  /*  map_db->sync(map_db, 0); */
  map_db->close(map_db);
  return 0;
}

void usage(char *progname)
{
  printf("%s: sourcefilename\n", progname);
}

void fill_base(FILE *input, DB *map_db)
{
  DBT key, data;
  char *line;
  char buff[1025];
  buff[sizeof(buff)-1]=0; 
  
  while( (line = fgets(buff, sizeof(buff)-1, input)) )
  {
    printf ("%s", line);
    if (line[0]=='#') /* ignore all comments */
      continue;
    while( *line && isspace(*line) )
      line++;
    if (! *line )        /* this was empty line */
      continue;
    key.data = line; 
    if ( !(line=strchr(line,':')) ) /* can not find seperator */
      continue;
    *line=0;
    line++;

    while( *line && isspace(*line) )
      line++;
    if (! *line )        /* no dest ABng uid (string) */
      continue;

    data.data=line;

    while( *line && !isspace(*line) )
      line++;
    if (! *line )        /* it is immposible but if happen */
      continue;
    *line=0;

    key.size = strlen((char*)key.data)+1;
    data.size = strlen((char*)data.data)+1;

    printf ("putin alias: %s %s\n",(char *)key.data, (char *)data.data);
    map_db->put(map_db, &key, &data, 0);
  }
}

