/* abng_tools.c 
 * Tools for comunicate with Active Badge system
 * (c) 1998 Dariusz Zbik <zbik@ernie.icslab.agh.edu.pl>
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <limits.h>
#include <fcntl.h>
#include <db.h>

#include "abng_tools.h"
#include "finger.h"
#include "whlib.h"

DB *map_db = NULL;
whlib_handle whlib = NULL;

int abng_constructor(int argc, char *argv[], const char *mapfile)
{
  map_db = dbopen(mapfile, O_RDONLY, O_RDONLY, DB_HASH, NULL);
  whlib = whereis_init(argc, argv);
  whereis_connect(whlib, "viewroom");
  return 1;
}

void abng_destructor(void)
{
  whereis_disconnect(whlib);
  whereis_destroy(whlib); 
}

const char *map_name(const char *loginname)
{
  DBT key, data;
  static char buff[1025];

  if ( !map_db )
    return loginname;

  key.data=(void*)loginname;
  key.size=strlen(loginname)+1;
  data.data=(void*)buff;
  data.size=sizeof(buff);
  
  if ( !map_db || map_db->get(map_db, &key, &data, 0) )
    return loginname;

  strcpy(buff,data.data);
  return buff;
}

void abng_fill_it(PERSON *pn, const char *unixlogin)
{
  const char *abng_uid;
  const char *strptr;
  whlib_result whres = NULL;

  abng_uid=map_name(unixlogin);

  if ( !strcmp(abng_uid, "null") || !(whres=whereis_findUserFL(whlib, abng_uid, WH_FU_AWAY)) )
  {
    pn->abng_uid = NULL;
    pn->abng_where = NULL;
    pn->abng_message = NULL;
    return;
  }

  pn->abng_uid = strdup(abng_uid);
  strptr = whres ? get_RoomName(whres) : NULL;
  pn->abng_where = strptr ? strdup(strptr) : NULL;

  strptr = whres ? get_Away(whres) : NULL;
  pn->abng_message = strptr ? strdup(strptr) : NULL;

  if (whres)
    whereis_free_result(whres); 
}

void abng_show_message(PERSON *pn, const char *header)
{
  if ( pn->abng_message )
    printf ("%s%s\n", header, pn->abng_message);
}

