/*
 * (c) 1998 Dariusz Zbik <zbik@dwd.interkom.pl>
 */


#include <stdlib.h>
#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif

  typedef void* whlib_handle;
  typedef void* whlib_result;


#if    ( !defined(__cplusplus) || defined(__finger__))
#define WH_FU_NULL     0
#define WH_FU_AWAY     1
#define WH_FU_NEIGHB   2
#define WH_FU_VERBOSE  4
#endif

  whlib_handle whereis_init(int argc, char *argv[]);
  void whereis_destroy(whlib_handle wh);
  int whereis_connect(whlib_handle wh, const char *viewname); 
  void whereis_disconnect(whlib_handle wh);

  whlib_result whereis_findUser(whlib_handle wh,
				const char *userid);
  whlib_result whereis_findUserFL(whlib_handle wh,
				  const char *userid,
				  int flags);

  void whereis_free_result(whlib_result wr);

  const char *get_Name(whlib_result wr);
  const char *get_ForeName(whlib_result wr);
  const char *get_SurName(whlib_result wr);
  const char *get_Away(whlib_result wr);
  const char *get_LastSeenStr(whlib_result wr);
  const char *get_RoomName(whlib_result wr);
  const char *get_RoomNum(whlib_result wr);

#ifdef __cplusplus
} /* extern "C" */
#endif 

#if    ( defined(__cplusplus) && (!defined(__finger__)))

#include <iostream.h>

#include "abng_core_management.hh"
#include "bootstrap_n.hh"

#define TIME_PERIOD 240
//#define TIME_PERIOD 24000000 
#define PROP_NUMBER "number"
#define PROP_DESCR  "description"

extern char *defaultview;


class whereis_result;
class whereis_tools;
class whereis_observer;

class whereis_result {
friend class whereis_tools;
friend class whereis_observer;
protected:
  whereis_result();
public:
  ~whereis_result ();

  Domain_Repository::Repository_Item_Id getID()
  { return loct_descr_item_id; };
  Domain_Repository::Repository_Item_Id getLocID()
  { return current_location; };
  const char *getView()     const { return viewname; };

  const char *getName()     const { return username; };
  const char *getForeName() const { return forename; };
  const char *getSurName()  const { return surname;  };
  const char *getPosition() const { return position; };

  const char *getAway()     const { return away;     };

  const char *getLastSeenStr() const { return lastseenStr; };
  time_t      getLastSeen()    const { return lastseen;    };
  const char *getLocType()  const { return location; };
  const char *getRoomName() const { return roomname; };
  const char *getRoomNum()  const { return roomnum;  };
  short       getQuality()  const { return quality;  };

  short       getRoomPeople() const { return roompeople; };
  const whereis_result *getRoomPeople(short idx) const
  {
    if (idx<0 && idx >= roompeople)
      return NULL;
    return roomlist[idx];
  }
  int getTrace() { return trace; };

protected:
  void setView(const char *view);
  void setName(const char *name);
  void setForeName(const char *fore);
  void setSurName(const char *sur);
  void setPosition(const char *pos);
  void setAway(const char *aw);

  void setLocType(const char *ltype);
  void setRoomNum(const char *num);
  void setRoomName(const char *name);
  void setQuality(short q);
  void setLastSeen(const time_t seen);

  void setRoomPeople(short num);
  void setRoomPeople(short idx, class whereis_result *res);

  void freeroomlist();
protected:
  class whereis_observer *whobs;
  ABng_Core::View_ptr view;
  Domain_Repository::Repository_Item_Id loct_descr_item_id;
  Domain_Repository::Repository_Item_Id current_location;
  char *viewname;
  char *username;
  char *forename;
  char *surname;
  char *position;
  char *lastseenStr;
  time_t lastseen;
  char *location;
  char *roomnum;
  char *roomname;
  char *away;
  short quality;
  short roompeople;
  whereis_result **roomlist;
  int trace;
  whereis_tools *wht;
};

const int WH_ST_TRACE   = 2;
const int WH_ST_CONNECT = 1;
const int WH_ST_NULL    = 0;

const int WH_FU_NULL    = 0;
const int WH_FU_AWAY    = 1;
const int WH_FU_NEIGHB  = 2;
const int WH_FU_VERBOSE = 4;

class whereis_tools
{
  friend class whereis_result;
  friend class whereis_observer;
public:
  whereis_tools(int argc, char *argv[]);
  ~whereis_tools();
  int connect(const char *viewname);
  void disconnect();

  class whereis_result *traceUser(const char *userid,
				  void (*notify)(whereis_result *)=NULL,
				  int flags=WH_FU_NULL);

  class whereis_result *findUser(const char *userid, int flags=WH_FU_NULL)
  {   
    return findUser(0, userid, flags);
  };
protected:
  void stop_trace();
  int attach_it(whereis_result *res,
		void (*notify)(whereis_result *),
		int flags);
  void whereis_tools::detach_it(whereis_result *res);
protected:
  class whereis_result *findUser(
    Domain_Repository::Repository_Item_Id usernr,
    const char *userid,
    int flags,
    whereis_result *ret=NULL);
protected:
  char *to_string(CORBA::Any &any_value);

protected:
  CORBA::ORB_ptr orb;
  CORBA::BOA_ptr boa;
  ABng_Core::Away_Rep_var away_rep;
  ABng_Core::View_var view;
  int state;
};
 
class whereis_observer : public BOA_IMPL_BASE(ABng_Core,Dynamic_Data_Observer) {
public:
  whereis_observer(whereis_result *obj_arg,
		   void (*notify_arg)(whereis_result *),
		   int verbose_arg) {
    obj = obj_arg;
    notify = notify_arg;
    verbose = verbose_arg;
  };
  ~whereis_observer() { };
  
  void receive_sighting( const ABng_Core::Sighting_Description &sd 
			 ENV_DECL( IT_env) )
    { 
//      if (sd.my_location != obj->current_location)
//	cerr << "istotnie rozne changed" << endl;

      if (verbose || (sd.my_location != obj->current_location) )
      {
//	cerr << "location changed" << endl;
	obj->wht->findUser(obj->loct_descr_item_id, NULL, WH_FU_NULL, obj);
	obj->current_location;
	if (notify)
	  notify(obj);
      }
    };

  void receive_sightings( const ABng_Core::Sighting_Descriptions &sds
			  ENV_DECL( IT_env) )
    { 
      for (int i=0; i<sds.length(); i++)
	receive_sighting( sds[i], IT_env);
    };
  

  void abort( ENV_DECL1( IT_env) ) { };
  private:

  int verbose;
  whereis_result *obj;
  void (*notify)(whereis_result *);
};



#endif /* __cplusplus */
