package jabba;

import com.sun.java.swing.JPanel;
import com.sun.java.swing.JTable;
import com.sun.java.swing.table.TableColumn;
import com.sun.java.swing.table.TableColumnModel;

import java.awt.Component;
import java.awt.BorderLayout;

import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;

class ViewerLocationsTab extends JPanel implements ViewerTab {
  JTable table;
  ViewerLocationsTableDataModel dataModel;
  String name;

  Jabba jabba;
  Viewer viewer;

  ViewerLocationsTab( Jabba jabba, Viewer viewer ) {
    this( jabba,viewer,"Locations" );
  }

  ViewerLocationsTab( Jabba jabba, Viewer viewer, String name ) {
    super();
    this.jabba = jabba;
    this.viewer = viewer;
    this.name = name;
    jabba.console.addWriter( this, "Viewer locations tab" );

    setLayout( new BorderLayout() );
  }


  // implementacja ViewerTab
  public void regenerate() {
    jabba.console.messageFrom( this, "regenerate() called." );

    // usuwamy stara tabele
    removeAll();

    // tworzymy model danych
    dataModel = new ViewerLocationsTableDataModel(jabba);

    jabba.receiver.registerDataModel( dataModel,Receiver.LOCATION_MESSAGES );

    // tworzymy tabele
    TableSorter ts = new TableSorter(dataModel);
    table = new JTable(ts,null,ts.getSelectionModel());
    // table.setFont( Viewer.defaultFont );
    ts.addMouseListenerToHeaderInTable(table);
    // table.setPreferredScrollableViewportSize( new Dimension(500,100) );
    table.addMouseListener( new MouseAdapter() {
      public void mouseClicked( MouseEvent ev ) {
	if( ev.getClickCount()==2 ) addNewSingleLocationTab();
      }
    } );

    ViewerTabHelper.initColumnSizes( table,dataModel );

    this.add( JTable.createScrollPaneForTable(table),BorderLayout.CENTER );
  }    

  public void addNewSingleLocationTab() {
    jabba.console.messageFrom( this, "addNewSingleLocationTab() called." );
    int row = table.getSelectedRow();
    int columnNumber = table.convertColumnIndexToView(ViewerLocationsTableDataModel.COLUMN_NUMBER);
    int columnDescr = table.convertColumnIndexToView(ViewerLocationsTableDataModel.COLUMN_DESCRIPTION);

    Object number = table.getValueAt( row,columnNumber );
    Object descr = table.getValueAt( row,columnDescr );
    String tabName = "("+number+") "+descr;

    int loc_index = ((Integer)table.getValueAt( row,ViewerLocationsTableDataModel.ATTRIBUTE_INDEX)).intValue();

    DismissableViewerTab vt = viewer.createSingleLocationTab( tabName,loc_index );
    vt.setParentTab( this );
    viewer.addTab( vt );
    viewer.selectTab( vt );
  }

  public void update() {
    jabba.console.messageFrom( this, "update() called." );

    if( dataModel==null ) regenerate();
    else jabba.receiver.updateDataModel( dataModel, Receiver.LOCATION_MESSAGES );
  }
  
  public JabbaDataModel getJabbaDataModel() {
    return dataModel;
  }

  public JPanel getPanel() {
    return this;
  }

  public String getName() {
    return name;
  }
}

