package jabba;

import com.sun.java.swing.table.AbstractTableModel;
import java.awt.Color;
import com.sun.java.swing.SwingUtilities;

/** 
 * Receives messages from Receiver object and directly accesses it.
 */
class ViewerMapTableDataModel extends AbstractJabbaTableDataModel {

  Jabba jabba;
  Receiver receiver;
  ViewerMapTab viewerMapTab;

	Runnable people = new Runnable() {
		public void run() {
		  viewerMapTab.svr.visR.removeUsers();
		  insertPeople(viewerMapTab.svr.visR);
		}
	};

	Runnable equipment = new Runnable() {
		public void run() {
		  viewerMapTab.svr.visR.removeEquipment();
		  insertEquipment(viewerMapTab.svr.visR);
		}
	};

  ViewerMapTableDataModel( Jabba jabba, ViewerMapTab viewerMapTab ) {
    super();
    this.jabba = jabba;
    receiver = jabba.receiver;
    this.viewerMapTab = viewerMapTab;
    jabba.console.addWriter( this, "1st equipment table model" );
  }

  public Rooms insertPeople(Rooms rooms)  {
    int rowCount = getRowCount(Receiver.USER_MESSAGES);
    Human hum;
    Room room;
    for (int i=0; i<rowCount; i++)  {
      hum = new Human(receiver.getUserName(i), receiver.getUserSurname(i), 
		receiver.getUserSightingTime(i));
      hum.setMask(receiver.getUserMask(i));
      room = rooms.getUserLocation(receiver.getUserLocationNumber(i));
      if (room == null) continue;
      else room.addH(hum);
    }
    return rooms;
  }

  public Rooms insertEquipment(Rooms rooms)  {
    int rowCount = getRowCount(Receiver.EQUIPMENT_MESSAGES);
    Device dev;
    Room room;
    for (int i=0; i<rowCount; i++)  {
      String type = receiver.getEquipmentType(i);
      dev = new Device(receiver.getEquipmentName(i), type);
      dev.setMask(receiver.getEquipmentMask(i));
      room = rooms.getEquipmentLocation(receiver.getEquipmentLocationNumber(i));
      if (room == null) continue;
      else room.addD(dev);
    }
    return rooms;
  }

  public Rooms makeRooms() {	

    int rowCount = getRowCount(Receiver.LOCATION_MESSAGES);
    Room room;
    Rooms rs = new Rooms();
    int sizeR_x, sizeR_y;


    for (int i=0; i<rowCount; i++)  {
	sizeR_x= receiver.getLocationSizeX(i).intValue();
	sizeR_y= receiver.getLocationSizeY(i).intValue();
      room = new Room(new Color(receiver.getLocationRGBColor(i).intValue()),
		"a", receiver.getLocationNumber(i),
		receiver.getLocationOriginX(i).intValue(),
		receiver.getLocationOriginY(i).intValue(),
		sizeR_x, sizeR_y, 0xfb);

	int[] tab=receiver.getLocationDoors(i);
	int doorCount = tab.length/3;
	int where, point, size;
	int x, y, size_x, size_y;
	Door door;

	for (int j=0; j<doorCount; j++)
	{
	where=tab[3*j];
	point=tab[1+3*j];
	size=tab[2+3*j];

	switch (where) {
	case 0:	x=0;
		y=sizeR_y-point-size/2;
		break;
	case 1:	x=point-size/2;
		y=0;
		break;
	case 2:	x=sizeR_x;
		y=point-size/2;
		break;
	case 3:	x=sizeR_x-point-size/2;
		y=sizeR_y;
		break;
	default:x=0;
		y=0;
		System.out.println("Wrong door data");
		break;
	}

	if ((where==0) || (where==2))
		{size_x=4; size_y=size;}
	else
		{size_x=size; size_y=4;}

	door = new Door(x, y, size_x, size_y);
	room.addDoor(door);
	}
      	rs.addRoom(room);
    }
  	rs.currentRoom=rs.getRoomAt(0);
        return rs;
  }

  // implementacja JabbaDataModel
  public void user_nameChanged(int pos) {
    jabba.console.messageFrom( this, "nameChanged("+pos+") called." );
  }

  public void user_surnameChanged(int pos) {
    jabba.console.messageFrom( this, "surnameChanged("+pos+") called." );
  }

  public void user_locationChanged(int pos) {
    jabba.console.messageFrom( this, "locationChanged1("+pos+") called." );
	SwingUtilities.invokeLater(people);
	SwingUtilities.invokeLater(viewerMapTab.svr.visR.currentRoom.doTextPane);
	SwingUtilities.invokeLater(viewerMapTab.svr.doredrawALL);
    jabba.console.messageFrom( this, "locationChanged2("+pos+") called." );
  }

  public void user_phoneChanged(int pos) {
    jabba.console.messageFrom( this, "phoneChanged("+pos+") called." );
  }

  public void user_qualityChanged(int pos) { 
    jabba.console.messageFrom( this, "qualityChanged("+pos+") called." );
  }

  public void user_peopleNumberChanged(int pos) {
    jabba.console.messageFrom( this, "peopleNumberChanged("+pos+") called." );
  }
  
  public void user_timeChanged(int pos) {
    jabba.console.messageFrom(this, "timeChanged("+pos+") called." );
    Room curRoom = viewerMapTab.svr.visR.currentRoom;
    Rooms rooms = viewerMapTab.svr.visR;
    if (curRoom == rooms.getUserLocation(receiver.getUserLocationNumber(pos)))
{
	SwingUtilities.invokeLater(curRoom.doTextPane);
}  }

  public void maskChanged() {
	SwingUtilities.invokeLater(people);
	SwingUtilities.invokeLater(equipment);
	SwingUtilities.invokeLater(viewerMapTab.svr.visR.currentRoom.doTextPane);
	SwingUtilities.invokeLater(viewerMapTab.svr.doredrawALL);
  }
 
  public void equipment_locationChanged(int pos) {
    jabba.console.messageFrom( this, "locationChanged("+pos+") called." );
	SwingUtilities.invokeLater(equipment);
	SwingUtilities.invokeLater(viewerMapTab.svr.visR.currentRoom.doTextPane);
	SwingUtilities.invokeLater(viewerMapTab.svr.doredrawALL);
  }

  public void equipment_timeChanged(int pos) {
    jabba.console.messageFrom(this, "timeChanged("+pos+") called." );
//    Room curRoom = viewerMapTab.svr.visR.currentRoom;
//    Rooms rooms = viewerMapTab.svr.visR;

//    if (curRoom == rooms.getEquipmentLocation(receiver.getEquipmentLocationNumber(pos)))
//{
//	SwingUtilities.invokeLater(curRoom.doTextPane);
//  }
  }

/////// potrzebne do Tabeli - u nas nie

  // pomocnicza w okresleniu szerokosci kolumn
  public Object getLongValue( int clmn ) {
    return "q"; 
  }

  public int getColumnCount() {
    return 0;
  }

  public int getRowCount() {
    return 0;
  }

  public int getRowCount(int msg) {
    return receiver.getObservedObjectsCount(msg);
  }

  public Class getColumnClass(int col) {
    Object o = getValueAt(0,col);
    if( o!=null ) return o.getClass();
    else return java.lang.String.class;
  }

  public Object getValueAt( int row, int clmn ) {
      return new Integer(row);
  }

  public String getColumnName( int clmn ) {
    return "q";
  }
}
