//
// $Id: abng_guarded.hh,v 1.2 1998/07/29 13:14:44 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/abng_guarded.hh,v $
//
// $Log: abng_guarded.hh,v $
// Revision 1.2  1998/07/29 13:14:44  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/01/13 13:15:48  mencnar
// Initial revision
//
//

#ifndef __abng_guarded_hh__
#define __abng_guarded_hh__

#include "omnithread.h"
#include <assert.h>

//-----------------------------------------------------------ABng_S_Guarded
//
class ABng_S_Guarded { // not reentrant
  omni_mutex guard;

  void lock()		{ guard.lock(); }
  void unlock()		{ guard.unlock(); }
public:
  omni_mutex & mutex() { return guard; }
  
  class auto_lock {
    ABng_S_Guarded &guard;
  public:
    auto_lock( ABng_S_Guarded &_guard) : guard( _guard) {
      guard.lock();
    }

    ~auto_lock() {
      guard.unlock();
    }
  };

  friend class auto_lock;
};

class ABng_Guarded { // reentrant
  omni_mutex critical_sec;
  int thread_id;
  int threads_count;
  omni_condition critical_cond;

public:
  ABng_Guarded() : 
    thread_id( 0),
    threads_count( 0),
    critical_cond( &critical_sec) {}

private:
  void lock() {
    critical_sec.lock();
    while (threads_count != 0 && thread_id != omni_thread::self()->id())
      critical_cond.wait();
    thread_id= omni_thread::self()->id();
    threads_count++;
    critical_cond.signal(); // wake up other waiting threads
    critical_sec.unlock();
  }

  void unlock() {
    critical_sec.lock();
    assert( thread_id == omni_thread::self()->id());
    assert( threads_count>0);
    if (--threads_count==0) critical_cond.signal();
    critical_sec.unlock();
  }

public:
  class auto_lock {
    ABng_Guarded &guard;
  public:
    auto_lock( ABng_Guarded &_guard) : guard( _guard) {
      guard.lock();
    }

    ~auto_lock() {
      guard.unlock();
    }
  };

  friend class auto_lock;
};
    
#endif // __abng_guarded_hh__

//

