//
//  File:         abng_string.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 28.01.98
//
// General Description:
//   Implementation file for the ABng_String class
//

#define EXCEPTIONS
#include "abng_string.h"

char*
   ABng_String::copy_string(
	const char* data
   )
{
#ifdef ObjectStore
	int len = strlen(data) + 1;
	char* string_field = 
		new (os_database::of(this), ::char_type, len) char[len];
	strcpy(string_field, data);
	return string_field;
#else
#ifdef ORBIX
	return CORBA::string_dupl(data);
#else
	return CORBA::string_dup(data);
#endif // ORBIX
#endif // ObjectStore
}

void
   ABng_String::free_string(
	char* string_field
   )
{
#ifdef ObjectStore
	delete [] string_field;
#else
	CORBA::string_free(string_field);
#endif
}

int 
   ABng_String::match_string (
	const char* _string_pattern,
	const char* _string_field
   )
{
	// check matching of string field
	RWCRegexp my_string_pattern(_string_pattern);
	RWCString my_pattern(_string_pattern);
	RWCString my_string_field(_string_field);
	size_t match_start, match_length;

	if ( my_pattern == "")
	  return my_pattern == my_string_field;

	// find a match in the string
	match_start = my_string_field.index(my_string_pattern, &match_length);

	// check if match found
	if (match_start == RW_NPOS)
		return 0;

	// check if it begins 
	if (match_start != 0)
		return 0;

	if (match_length != my_string_field.length())
		return 0;

	// matches
	return 1;
}
#ifdef FilePersistency

char*
   ABng_String::read_string(
	istream &iostr
   )
{
	// read string from stream
	char buffer[256];
	iostr >> buffer;
	
	if (!iostr) {
		cerr << "Error: reading from file";
		exit(1);
	}

#ifdef ORBIX
	return CORBA::string_dupl(buffer);
#else
	return CORBA::string_dup(buffer);
#endif // ORBIX
}

#endif /* FilePersistency */

