// $Id: basic_observed_i.cc,v 1.2 1998/07/29 13:14:46 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/basic_observed_i.cc,v $
//
// $Log: basic_observed_i.cc,v $
// Revision 1.2  1998/07/29 13:14:46  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
// File:          basic_observed.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  02.02.98
//
// General Description:
//  Definition of the template Basic_Observed_I class which implements
//  operations common for all observed ABng Repositories.

#include "basic_observed_i.h"
#include <string.h>
#include <unistd.h>

template <class Observer_Types>
Basic_Observed_I<Observer_Types>::Basic_Observed_I(
#ifdef FilePersistency
  const char *repository_name_p
#endif
) :
  ABng_Guarded(),
  observers_to_item_ids_list(hash_observer),
  proxy_table(::hash_proxy),
  save_observers_needed( 0),
  save_observers_latency( 5),
  save_observers_cond( & save_observers_guard.mutex()),
  item_ids_to_observers_list(hash_item_id) {

#ifdef FilePersistency
    static const char *obs_postfix= ".observers";
    persistent_observers_rep= new char[ strlen(repository_name_p)+
				        strlen(obs_postfix)+1];
    strcpy( persistent_observers_rep, repository_name_p);
    strcat( persistent_observers_rep, obs_postfix);
    (new omni_thread( static_gc_worker, this))->start();
    (new omni_thread( static_save_observers_worker, this))->start();
#endif
}

template <class Observer_Types>
Basic_Observed_I<Observer_Types>::~Basic_Observed_I() {

  // Abort all observers.
  // Construct iterator.
  Observer_Item_Dictionary_Iterator iterator(observers_to_item_ids_list);

  // Notify all observers that the observed quits.
  while (iterator()) {
    try {
      iterator.key()->abort();
    }
    catch (...) {}
  }
  delete []persistent_observers_rep;
}

template <class Observer_Types>
Basic_Observed_I<Observer_Types>::Observer_Entries * 
  Basic_Observed_I<Observer_Types>::observer_list (
  ENV_DEF1(env)) 
     throw (CORBA::SystemException) {

  auto_lock local_lock(*this);

  // Initialize the sequence for collected observers
  Observed::Observer_Entries* collected_observers = 
    new Observed::Observer_Entries(
      observers_to_item_ids_list.entries());

  collected_observers->length(observers_to_item_ids_list.entries());

  // Create an iterator to browse through the observer hash table.
  Observer_Item_Dictionary_Iterator iterator(observers_to_item_ids_list);

  // Browse through the hash table to collect observer data.
  for (int i = 0; iterator(); i++){
    // Copy observer reference
    (*collected_observers)[i].obs = Observer::_duplicate(iterator.key());
    // Set length of the interest seqeance
    (*collected_observers)[i].interest.length(iterator.value()->entries());
    // Create an iterator to browse through the item ids sequence
    Item_Id_List_Iterator iterator1(*(iterator.value()));
    // Copy observer interest 
    for (int j = 0; iterator1(); j++) {
      (*collected_observers)[i].interest[j] = iterator1.key();
    }
  }
  return collected_observers;
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::static_save_observers_worker( void *args) {
  Basic_Observed_I<Observer_Types> &impl=
    *((Basic_Observed_I<Observer_Types>*) args);

  ABng_S_Guarded::auto_lock local_lock( impl.save_observers_guard);

  while (impl.save_observers_needed) {
    impl.save_observers_guard.mutex().unlock();
    impl.do_save_observers();
    impl.save_observers_guard.mutex().lock();
  }

  unsigned long sec,nsec,last_sec, rel_sec, rel_nsec;
  omni_thread::get_time( &last_sec, &nsec, 0,0);
  while (1) {
    int wait_ret;
     
    impl.save_observers_cond.wait();
    // waken up ?
    omni_thread::get_time( &sec, &nsec, 0,0);
    while (sec-last_sec < impl.save_observers_latency) {
      omni_thread::get_time( &rel_sec, &rel_nsec, 
			     impl.save_observers_latency-(sec-last_sec), 0);
#ifdef OMNIORB
      impl.save_observers_cond.timedwait( rel_sec, rel_nsec);
#else
      impl.save_observers_cond.timed_wait( rel_sec, rel_nsec);
#endif
      omni_thread::get_time( &sec, &nsec, 0,0);
    }
    while (impl.save_observers_needed) {
      impl.save_observers_guard.mutex().unlock();
      impl.do_save_observers();
      impl.save_observers_guard.mutex().lock();
    }
    omni_thread::get_time( &last_sec, &nsec, 0,0);
  }
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::static_gc_worker( void *args) {
  sleep(60); // sleep for a minute
  while (1) {
    ((Basic_Observed_I<Observer_Types>*) args)->gc_worker();
    sleep(3600); // sleep for an hour
  }
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::gc_worker() {
  RWTValDlist<Observer_ptr> dead_observers;
  RWTValDlist<Observer_ptr> c_observers;

  // it's important to do not hold a lock while pinging observers,
  // as it could lead to a deadlock in observer's process
  //

  {
    auto_lock local_lock(*this);

    for (Observer_Item_Dictionary_Iterator iterator(observers_to_item_ids_list);
	 iterator(); )
      c_observers.insert( Observer::_duplicate( iterator.key()));
  }

  // Browse through the hash table
  for (RWTValDlistIterator<Observer_ptr> iterator( c_observers); 
       iterator(); ) {

    int dead= 0;
    try {
      dead= iterator.key()->_non_existent();
    }
    catch (...) {
      dead= 1;
    }
    if (dead) dead_observers.insert( iterator.key());
  }

  {
    auto_lock local_lock(*this);

    for (RWTValDlistIterator<Observer_ptr> diter( dead_observers);
	 diter(); ) {
      try {
	private_detach_add_observer( diter.key());
      } catch (...) {}
      try {
	private_detach_observer( diter.key(),
				 get_item_ids_list( diter.key()));
      } catch(...) {}
    }
  }
  if (!loading_observers && !dead_observers.isEmpty()) save_observers();

  for (RWTValDlistIterator<Observer_ptr> deliter( c_observers);
       deliter(); )
    CORBA::release( deliter.key());
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::save_observers_to_stream( ostream &ostr) {

  auto_lock local_lock(*this);

  ostr << observers_to_item_ids_list.entries() << endl;

  // Create an iterator to browse through the observer hash table.
  Observer_Item_Dictionary_Iterator iterator(observers_to_item_ids_list);

  // Browse through the hash table
  for (int i = 0; iterator(); i++){
    // Copy observer reference
    CORBA::String_var tmpstr=
      Bootstrap_base::orb->object_to_string( iterator.key());
    ostr << ((const char *) tmpstr) << ' '
	 << iterator.value()->entries() << ' ';

    // Create an iterator to browse through the item ids sequence
    Item_Id_List_Iterator iterator1(*(iterator.value()));
    for (int j = 0; iterator1(); j++)
      ostr << iterator1.key() << ' ';
    ostr << endl;
  }
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::save_observers() {
  ABng_S_Guarded::auto_lock local_lock(save_observers_guard);

#ifdef DEBUG
  cerr << "save_observers " << persistent_observers_rep << endl;
#endif
  save_observers_needed= 1;
  save_observers_cond.signal();
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::do_save_observers() {
  {
    auto_lock local_lock(*this);

#ifdef DEBUG
    cerr << "do_save_observers " << persistent_observers_rep << endl;
#endif
    save_observers_needed= 0;
    FileTransaction ft( persistent_observers_rep,
			0 /* don't create backup copy */);

    ostream &ostr= *(ft.prepare());
  
    save_observers_to_stream( ostr);
    ft.commit();
  }
  cerr << "do_save_observers done " << persistent_observers_rep << endl;
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::load_observers_from_stream( istream &ifs) {

  int obs_items;
  ifs >> obs_items;
  if (!ifs) return;
  for (int i=0; i<obs_items; i++) {
    RWCString tmpstr;
    ifs >> ws >> tmpstr;
    try {
      CORBA::Object_var tmpobj=
	Bootstrap_base::orb->string_to_object( (char*) (const char *) tmpstr);
      Observer_ptr obsobj=
	Observer::_narrow( (CORBA::Object_ptr) tmpobj);
      if ( CORBA::is_nil( obsobj)) continue;
      
      int rids_len;
      ifs >> rids_len;
      if (!ifs) return;
      Domain_Repository::Repository_Item_Ids rids( 1);
      rids.length( 1);
      for (int k=0; k<rids_len; k++) {
	try {
	  int tmpi;
	  ifs >> tmpi;
	  rids[0]= tmpi;
	  attach_observer( obsobj,
			   rids);
	}
	catch (Domain_Repository::Unknown_Repository_Item_Ids) {}
      }
      CORBA::release( obsobj);
    }
    catch (...) {
      return;
    }
  }
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::load_observers() {
  loading_observers= 1;

#ifdef DEBUG
  cerr << "Loading observers from " << persistent_observers_rep << endl;
#endif
  ifstream ifs( persistent_observers_rep);
  if (!ifs) {
#ifdef DEBUG
    cerr << "Could not find file " << persistent_observers_rep << endl;
#endif
    loading_observers= 0;
    return;
  }
  load_observers_from_stream( ifs);
#ifdef DEBUG
  cerr << observers_to_item_ids_list.entries() << " observers restored" << endl;
#endif
  loading_observers= 0;
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::attach_observer (
  Observer_ptr obs,  
  const Domain_Repository::Repository_Item_Ids& observed_id_list
  ENV_DEF(env))
     throw (CORBA::SystemException, 
	    Domain_Repository::Unknown_Repository_Item_Ids) {

  auto_lock local_lock(*this);
  private_attach_observer(obs, observed_id_list USE_ENV(env));
  if (!loading_observers) save_observers();
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::detach_observer (
  Observer_ptr obs, 
  const Domain_Repository::Repository_Item_Ids& observed_id_list
  ENV_DEF(env)) 
     throw (CORBA::SystemException, 
	    Domain_Repository::Unknown_Repository_Item_Ids, 
	    Domain_Repository::Unknown_Observer,
	    Domain_Repository::Unmatched_Observer_Item_Ids) {

  auto_lock local_lock( *this);
  private_detach_observer(obs, observed_id_list USE_ENV(env));
  if (!loading_observers) save_observers();
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::private_attach_observer (
  Observer_ptr obs,  
  const Domain_Repository::Repository_Item_Ids& observed_id_list
  ENV_DEF(env))
     throw (CORBA::SystemException, 
	    Domain_Repository::Unknown_Repository_Item_Ids) {

  if ( CORBA::is_nil( obs)) {
    return;
  }
  add_proxy(obs);

  // Initialize the sequence for unknown item ids
  Domain_Repository::Repository_Item_Ids unknown_item_ids(
    observed_id_list.length());

  // Check if all item ids are known.
#ifdef DEBUG
  	cerr << "item_ids_to_observers_list entries ";
	cerr << item_ids_to_observers_list.entries() << endl;
#endif

  int observed_id_list_len = observed_id_list.length();
  for (int i = 0; i < observed_id_list_len; i++) {
    if (!item_ids_to_observers_list.contains(observed_id_list[i])){
      // There is no item with observed_id_list[i] id, thus put
      // this id into the list of unknown ids.
      unknown_item_ids.length( unknown_item_ids.length()+1);
      unknown_item_ids[unknown_item_ids.length()-1] = observed_id_list[i];
    }
  }

  // If there is any unknown id throw an exception
  if (unknown_item_ids.length()) {
    throw Domain_Repository::Unknown_Repository_Item_Ids(unknown_item_ids);
  }

  // Update observers_to_item_ids_list
  if (!observers_to_item_ids_list.contains(obs)){
    // Copy item ids list
    Item_Id_List *item_id_list_p = new Item_Id_List(hash_item_id);
    for (int i = 0; i < observed_id_list_len; i++) {
      item_id_list_p->insert(observed_id_list[i]);
    }
    observers_to_item_ids_list.insertKeyAndValue(obs, item_id_list_p);
  }
  else {
    // Copy item ids list
    Item_Id_List *item_id_list_p = 0;
    observers_to_item_ids_list.findValue(obs, item_id_list_p);

    for (int i = 0; i < observed_id_list_len; i++) {
      if (!item_id_list_p->contains(observed_id_list[i])) {
	// not duplicated
	item_id_list_p->insert(observed_id_list[i]);
      }
    }
  }
	
  // Update item_ids_to_observers_list
  for (int j = 0; j < observed_id_list_len; j++) {
    Observer_List *observer_list_p = 
      get_item_observer_list(observed_id_list[j]);
    if (!observer_list_p->contains(obs)) {
      // not duplicated
      observer_list_p->insert(obs);
    }
  }
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::private_detach_observer (
  Observer_ptr obs, 
  const Domain_Repository::Repository_Item_Ids& observed_id_list
  ENV_DEF(env)) 
     throw (CORBA::SystemException, 
	    Domain_Repository::Unknown_Repository_Item_Ids, 
	    Domain_Repository::Unknown_Observer,
	    Domain_Repository::Unmatched_Observer_Item_Ids) {

  if ( CORBA::is_nil(obs)) { 
    throw Domain_Repository::Unknown_Observer();
  }
  add_proxy(obs);

  // Check if the observer is in the observer list
  if (!observers_to_item_ids_list.contains(obs)) {
    // Unknown observer
    throw Domain_Repository::Unknown_Observer();
  }
		
  // Initialize the sequence for unknown item ids
  Domain_Repository::Repository_Item_Ids 
    unknown_item_ids( observed_id_list.length());

  // initialize the sequence for unmatched item ids
  Domain_Repository::Repository_Item_Ids 
    unmatched_item_ids(observed_id_list.length());

  // Check if all item ids from observed_id_list are known and are 
  // observed by obs
  int observed_id_list_len = observed_id_list.length();
  for (int i = 0; i < observed_id_list_len; i++) {
    if (!item_ids_to_observers_list.contains(observed_id_list[i])) {
      // Unknown item id
      // !!! The below line causes memory realocation each time.

      unknown_item_ids.length(unknown_item_ids.length()+1);
      unknown_item_ids[unknown_item_ids.length()-1] = observed_id_list[i];
    }
    else {
      // Checking matching observer with this item id

      Observer_List *observer_list_p = 0;
      item_ids_to_observers_list.findValue(observed_id_list[i], 
        observer_list_p);

      if(!observer_list_p->contains(obs)) {
	// do not exist
	unmatched_item_ids.length( unmatched_item_ids.length()+1);
	unmatched_item_ids[unmatched_item_ids.length()-1] = 
	  observed_id_list[i];
      }
    }
  }

  // if there is any unknown id throw an exception
  if (unknown_item_ids.length()) {
    throw Domain_Repository::Unknown_Repository_Item_Ids(unknown_item_ids);
  }

  // if there is any unmatch id throw exception
  if (unmatched_item_ids.length()) {
    throw Domain_Repository::Unmatched_Observer_Item_Ids(unmatched_item_ids);
  }

  // Update item_ids_to_observers_list
  for (int j = 0; j < observed_id_list_len; j++) {
    Observer_List *observer_list_p = 0;
    item_ids_to_observers_list.findValue(observed_id_list[j], observer_list_p);
    // Check if the list is consistent
    if (observer_list_p->contains(obs)) {
      // Remove observer from the list of this id
      observer_list_p->remove(obs);
    }
  }

  // Update observers_to_item_ids_list
  Item_Id_List *item_id_list_p = 0;
  observers_to_item_ids_list.findValue(obs, item_id_list_p);

  for (int k = 0; k < observed_id_list_len; k++)
    //Ccheck if the list is consistent
    if (item_id_list_p->contains(observed_id_list[k]))
      // remove id from list
      item_id_list_p->remove(observed_id_list[k]);

  // Check if this observer still observes some item ids
  if (!item_id_list_p->entries()){
    // clean this observer data
    /*    Observer_ptr*
      obs_p = observers_to_item_ids_list.find(&obs);*/
    observers_to_item_ids_list.remove(obs);
    /*    delete obs_p;
    delete item_id_list_p;
    CORBA::release(obs);*/
  }
  remove_proxy(obs);
}

template <class Observer_Types>
Domain_Repository::Repository_Item_Ids 
  Basic_Observed_I<Observer_Types>::get_item_ids_list(
  Observer_ptr obs) {

  Domain_Repository::Repository_Item_Ids ret;

  Item_Id_List * item_id_list_p = 0;
  observers_to_item_ids_list.findValue(obs, item_id_list_p);
  if (!item_id_list_p) {
    return ret;
  }

  // Allocate memory
  ret.length(item_id_list_p->entries());

  ret.length(0);
  Item_Id_List_Iterator iterator( *item_id_list_p);

  while (iterator()) {
    ret.length(ret.length()+1);
    ret[ ret.length()-1]= iterator.key();
  }
  return ret;
}

template <class Observer_Types>
Basic_Observed_I<Observer_Types>::Observers* 
  Basic_Observed_I<Observer_Types>::
  get_item_observers (
    Domain_Repository::Repository_Item_Id item_id_p
    ENV_DEF(env)) 
  throw (CORBA::SystemException) {

  auto_lock local_lock(*this);

  Observer_List *observer_list_p = 0;
  item_ids_to_observers_list.findValue(item_id_p, observer_list_p);
  int observers_number = observer_list_p->entries();
  Observer_List_Iterator iterator(*observer_list_p); 
  // Initialize the sequence for collected observer
  Observers* collected_observers = new Observers(observers_number);
  collected_observers->length(observers_number);
 
  for (int i = 0; iterator(); i++){
    // Current observer reference 
    const Observer_ptr obs_p = iterator.key();
 
    // Copy observer reference
    (*collected_observers)[i] = 
      Observer::_duplicate(obs_p);
  }
 
  return collected_observers;
}

template <class Observer_Types>
Basic_Observed_I<Observer_Types>::Observer_List* 
  Basic_Observed_I<Observer_Types>::
  get_item_observer_list (Domain_Repository::Repository_Item_Id item_id_p) {

  static Observer_List null_obs_list(hash_observer);

  Observer_List *observer_list_p = 0;
  item_ids_to_observers_list.findValue(item_id_p, observer_list_p);
  if (observer_list_p) return observer_list_p;
  else return &null_obs_list;
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::insert_new_item(
  Domain_Repository::Repository_Item_Id id_p) {

  item_ids_to_observers_list.insertKeyAndValue(id_p, 
    new Observer_List(hash_observer));
}

template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::detach_item_from_observers(
  Domain_Repository::Repository_Item_Id item_id_p) {

  Observer_List *observer_list_p = 0;
  item_ids_to_observers_list.findValue(item_id_p, observer_list_p);
  Observer_List_Iterator iterator1(*observer_list_p);

  while (iterator1()){
    Item_Id_List *id_list_p = 0;
    observers_to_item_ids_list.findValue(iterator1.key(), id_list_p);
    if (id_list_p->contains(item_id_p)) {
      id_list_p->remove(item_id_p);
    }      
  }
  item_ids_to_observers_list.remove(item_id_p);
  if (!loading_observers) save_observers();
}

// Calculate the hash value of the item id.
template <class Observer_Types>
unsigned Basic_Observed_I<Observer_Types>::hash_item_id(
  const Domain_Repository::Repository_Item_Id& item_id_p) {

  return item_id_p;
}

// Calculate the hash value of the observer pointer.
template <class Observer_Types>
unsigned Basic_Observed_I<Observer_Types>::hash_observer(
  const Observer_ptr& observer_p) {

  return (unsigned)observer_p;
}


template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::add_proxy(
  Basic_Observed_I<Observer_Types>::Observer_ptr &obs) {

  if(!proxy_table.contains(SDO_ptr(obs))) {
#ifdef DEBUG
	cerr << "Welcome, the new observer" << endl;
#endif
    SDO_ptr new_sdo( obs);
    new_sdo.update_ic( get_new_ic());
    proxy_table.insert( new_sdo);
  }
  else {
    SDO_ptr sdo;
    proxy_table.find(SDO_ptr(obs), sdo);
    obs = sdo.my_object;
    sdo.update_ic( get_new_ic());
  }
}
 
template <class Observer_Types>
void Basic_Observed_I<Observer_Types>::remove_proxy(
  Basic_Observed_I<Observer_Types>::Observer_ptr obs) {

  if(observers_to_item_ids_list.contains(obs)) return;

  proxy_table.remove(SDO_ptr(obs));

#ifdef DEBUG
	cerr << "Bye bye, the dead observer" << endl;
#endif
}
