// $Id: db_utils.hh,v 1.3 1998/07/29 13:14:52 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/db_utils.hh,v $
//
// $Log: db_utils.hh,v $
// Revision 1.3  1998/07/29 13:14:52  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.2  1998/06/16 12:56:04  mencnar
// *** empty log message ***
//
// Revision 1.1  1998/01/15 14:24:30  mencnar
// Initial revision
//
//

#ifndef __db_utils_hh__
#define __db_utils_hh__

#include <iostream.h>
#include <rw/cstring.h>
#include <fstream.h>
#include <netinet/in.h>

struct in_addr get_host_address();

class DB_Header {
  RWCString rep_name;
  RWCString ver_name;
  RWCString record_description;
  int dont_include_default_description;

public:
  DB_Header() {}

  DB_Header( const char *_rep_name,
	     const char *_ver_name,
	     const char *_record_description,
	     int _dont_include_default_description= 0);

  friend ostream & operator << ( ostream &ostr, const DB_Header &db);
  friend istream & operator >> ( istream &istr, DB_Header &db);
};

class FileTransaction {
  RWCString file_name;
  int make_backup;
  ostream *ofs;
  char *tmp_file_name;
public:
  FileTransaction( const char *_file_name,
		   int _make_backup) :
    file_name( _file_name),
    make_backup( _make_backup),
    ofs( NULL),
    tmp_file_name( NULL) {}


  ostream *prepare();
  void commit();
  ~FileTransaction();
};

class TransactionLOG {
  ofstream ofs;
  char *log_file_name;
  size_t min_log_size;
  size_t max_log_size;
  int active;
  int stream_opened;
public:
  TransactionLOG( const char *db_file_name,
                  size_t _min_log_size= 10000,
                  size_t _max_log_size= 15000,
		  int active= 1);
  ~TransactionLOG();

  ostream & ostr() { return ofs; }

  class LOGItem {
    int committed;
    TransactionLOG &tlog;
  public:
    LOGItem( TransactionLOG &_tlog) : committed(0), tlog( _tlog) {
       tlog.start_transaction();
    }
    void commit() { committed= 1; }
    ~LOGItem() {
      if (committed) tlog.stop_transaction();
      else tlog.fail_transaction();
    }
  };
      
  void start_transaction();
  void stop_transaction();
  void fail_transaction();
protected:
  void check_and_reorganize();
};



ostream & operator << ( ostream &ostr, const DB_Header &db);
istream & operator >> ( istream &istr, DB_Header &db);

ostream & operator << ( ostream &str, const Domain_Repository::Octets &seq);
istream & operator >> ( istream &str, Domain_Repository::Octets &seq);

ostream & operator << ( ostream &str, const CORBA::TypeCode_ptr tc);
istream & operator >> ( istream &str, CORBA::TypeCode_ptr tc);

void save_string( ostream &ostr, const char *str);
void restore_string( istream &istr, char *&str);

#endif //

//

