

#include <stream.h>
#include "domain_rep_management.hh"
#include "abng_server_manager.hh"
#include "domain_rep_types.h"



Domain_Repository::Location_Type::Pattern 
 Location_Type_Types::init_pattern() {

  Domain_Repository::Location_Type::Pattern pattern;
  pattern.is_any = 1;
  pattern.item_id_pattern.is_any = 1;
  pattern.name_pattern.is_any = 1;
  pattern.property_description_list_pattern.is_any = 1;
  pattern.property_description_list_pattern.constraints =
    Domain_Repository::UNRESTRICTED;

  return pattern;
}

ostream& operator << (
  ostream &str, 
  Domain_Repository::Location_Type::Value_Description& value
) {
  str << value.item_id << " " 
      << value.name << " " 
      << value.property_description_list.length() << " ";
  for(int i = 0; i < value.property_description_list.length(); i ++)
     str  << value.property_description_list[i].name << " "
          << value.property_description_list[i].type << " ";
  return str;
}

Domain_Repository::Location_Description::Pattern 
 Location_Description_Types::init_pattern() {

  Domain_Repository::Location_Description::Pattern pattern;
  pattern.is_any = 1;
  pattern.item_id_pattern.is_any = 1;
  pattern.type_name_pattern.is_any = 1;
  pattern.property_list_pattern.is_any = 1;
  pattern.property_list_pattern.constraints =
    Domain_Repository::UNRESTRICTED;

  return pattern;
}

ostream& operator << (
  ostream& str, 
  Domain_Repository::Location_Description::Value_Description& value
) {
  str << value.item_id << " " 
      << value.type_name << " " 
      << value.property_list.length() << " ";
  for(int i = 0; i < value.property_list.length(); i ++)
    str << value.property_list[i].name << " " 
        << value.property_list[i].value << " ";
  return str;
}

Domain_Repository::Equipment::Pattern
 Equipment_Types::init_pattern() {

  Domain_Repository::Equipment::Pattern pattern;

  pattern.is_any = 1;
  pattern.item_id_pattern.is_any = 1;
  pattern.meta_data_pattern.is_any = 1;
  pattern.meta_data_pattern.my_control_info_pattern.my_control_data_pattern.
    default_field_pattern( Unsigned_Long_Pattern());
    //    Domain_Repository::Meta_Equipment::Control_Info_Pattern::IRCI_Pattern(
    //      Domain_Repository::Meta_Equipment::ANYTHING_ELSE
    //    );
  pattern.meta_data_pattern.specific_property_description_list_pattern.constraints = 
    Domain_Repository::UNRESTRICTED;
  pattern.name_pattern.is_any = 1;
  pattern.last_command_pattern.is_any = 1;
  pattern.property_list_pattern.is_any = 1;
  pattern.property_list_pattern.constraints = 
    Domain_Repository::UNRESTRICTED;

  return pattern;
}

ostream& operator << (
      ostream& str, 
      Domain_Repository::Equipment::Value_Description& descr) {

  str << descr.item_id << " ( " << descr.meta_data << " ) " 
      << descr.name << " " 
      << descr.last_command << " "
      << descr.property_list.length() << " ";
  for(int i = 0; i < descr.property_list.length(); i ++)
    str << descr.property_list[i].name << " " 
        << descr.property_list[i].value << " ";

  return str;
}

Domain_Repository::Meta_Equipment::Pattern
 Meta_Equipment_Types::init_pattern() {

  Domain_Repository::Meta_Equipment::Pattern pattern;
    pattern.is_any = 1;
  pattern.item_id_pattern.is_any = 1;
  pattern.type_pattern.is_any = 1;
  pattern.vendor_pattern.is_any = 1;
  pattern.model_pattern.is_any = 1;
  pattern.my_dimensions_pattern.is_any = 1;
  pattern.my_control_info_pattern.is_any = 1;
  pattern.my_control_info_pattern.my_control_data_pattern.
    default_field_pattern( Unsigned_Long_Pattern());

    //    Domain_Repository::Meta_Equipment::Control_Info_Pattern::IRCI_Pattern(
    //      Domain_Repository::Meta_Equipment::ANYTHING_ELSE
    //    );
  pattern.my_icon_pattern.is_any = 1;
  pattern.specific_property_description_list_pattern.is_any = 1;
  pattern.specific_property_description_list_pattern.constraints = 
    Domain_Repository::UNRESTRICTED;
  return pattern;
}

ostream& operator << (
      ostream& str, 
      Domain_Repository::Meta_Equipment::Value_Description& descr) {

  str << descr.item_id << " "
      << descr.type << " " 
      << descr.vendor << " " 
      << descr.model << " " 
      << descr.my_dimensions.height << " "
      << descr.my_dimensions.width << " " 
      << descr.my_dimensions.depth << " ";

  if(descr.my_control_info._d() == 
     Domain_Repository::Meta_Equipment::INFRA_RED
  ) {
    str << "INFRA_RED ( " 
        << descr.my_control_info.my_IR_control_data().commands.length() 
        << " ";
    for(int i = 0; 
        i < descr.my_control_info.my_IR_control_data().commands.length(); 
        i ++
    ) str << descr.my_control_info.my_IR_control_data().commands[i] << " ";

    str << descr.my_control_info.my_IR_control_data().my_IR_data.length() 
        << " ";
    for(i = 0; 
        i < descr.my_control_info.my_IR_control_data().my_IR_data.length(); 
        i ++
    ) str << descr.my_control_info.my_IR_control_data().my_IR_data[i] << " ";
  }
  else str << "ANYTHING ELSE ";

  str << descr.my_icon.length() << " ";
  for(int i = 0; i < descr.my_icon.length(); i ++)
    str << descr.my_icon[i] << " ";
  for(int j = 0; j < descr.specific_property_description_list.length(); j++)
     str  << descr.specific_property_description_list[j].name << " "
          << descr.specific_property_description_list[j].type << " ";
  return str;
}


Domain_Repository::User::Pattern User_Types::init_pattern() {

  Domain_Repository::User::Pattern pattern;
  pattern.is_any = 1;
  pattern.item_id_pattern.is_any = 1;
  pattern.uid_pattern.is_any = 1;
  pattern.forename_pattern.is_any = 1;
  pattern.surname_pattern.is_any = 1;
  pattern.position_pattern.is_any = 1;
  pattern.email_pattern.is_any = 1;
  pattern.phone_pattern.is_any = 1;

  return pattern;
}

ostream& operator << (
  ostream &str, 
  Domain_Repository::User::Value_Description& value
) {
  str << value.item_id << " " 
      << value.uid << " "
      << value.forename << " "
      << value.surname << " "
      << value.position << " "
      << value.email << " "
      << value.phone;

  return str;
}
ostream& operator << (ostream& os, CORBA::Any& val) {
    switch(val.type()->kind()) {
    case CORBA::tk_void:
      os << "void ";
      break;
    case CORBA::tk_null:
      os << "null ";
      break;
    case CORBA::tk_short: {
      CORBA::Short sh;
      val >>= sh;
      os << "short " << sh; break;
    }
    case CORBA::tk_long : {
      CORBA::Long ln;
      val >>= ln;
      os << "long " << ln; break;
    }
    case CORBA::tk_ushort: {
      CORBA::UShort ush;
      val >>= ush;
      os << "ushort " << ush; 
      break;
    }
    case CORBA::tk_ulong: {
      CORBA::ULong uln;
      val >>= uln;
      os << "ulong " << uln;
      break;
    }
    case CORBA::tk_float: {
      CORBA::Float fl;
      val >>= fl;
      os << "float " << fl; break;
    }
    case CORBA::tk_double: {
      CORBA::Double dbl;
      val >>= dbl;
      os << "double " << dbl; break;
    }
    case CORBA::tk_char : {
      CORBA::Char chr;
      CORBA::Any::to_char tchr(chr);
      val >>= tchr;
      os << "char " << chr; break;
    }
    case CORBA::tk_octet: os << "octet " << *(unsigned char*)val.value();
      break;
    case CORBA::tk_boolean: 
      os << "boolean " 
	 << (*(unsigned char*)val.value()) ? "true" : "false"; 
      break;
    case CORBA::tk_string:
      {
#ifdef OMNIORB
	char *str= "";
	CORBA::Any::to_string ts( str, val.type()->length());
	
	val >>= ts;
	os << "string " << str;
#else
	CORBA::String_var str;
	val >>= str;
	os << "string " << (const char *) str;
#endif
      }
    break;

#ifdef OMNIORB
    case CORBA::tk_sequence:
      {
	CORBA::ULong seqlen= val.type()->length();
	
	if (val.type()->content_type()->kind() == CORBA::tk_ulong) {
	  Domain_Repository::Repository_Item_Ids *pids= NULL;
	  if (val >>= pids) ;
	  else os << "FAILED";
	  os << "@" << seqlen << " ";
	  for (int i=0; i< pids->length(); i++)
	    os << (*pids)[i] << ' ';
	  if (pids) delete pids;
	}
	else if (val.type()->content_type()->kind() == CORBA::tk_string) {
	  ABng_Server_Manager::Name *pids= NULL;
	  if (val >>=pids) ;
	  else os << "FAILED";
	  os << "@" << seqlen << " ";
	  for (int i=0; i<pids->length(); i++)
	    os << (*pids)[i] << ' ';
	  if (pids) delete pids;
	}
      }
    break;
#else
    // Orbix specific:

    case CORBA::tk_sequence: {
      if(val.type()->equal(Domain_Repository::_tc_Shorts)) {
	os << "seq short ";
	Domain_Repository::Shorts* v = 
	  (Domain_Repository::Shorts*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      if(val.type()->equal(Domain_Repository::_tc_Longs))  {
	os << "seq long ";
	Domain_Repository::Longs* v = 
	  (Domain_Repository::Longs*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      if(val.type()->equal(Domain_Repository::_tc_Ushorts)) {
	os << "seq ushort ";
	Domain_Repository::Ushorts* v = 
                   (Domain_Repository::Ushorts*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      if(val.type()->equal(Domain_Repository::_tc_Ulongs)) {
	os << "seq ulong ";
	Domain_Repository::Ulongs* v = 
	  (Domain_Repository::Ulongs*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      if(val.type()->equal(Domain_Repository::_tc_Floats)) {
	os << "seq float ";
	Domain_Repository::Floats* v = 
	  (Domain_Repository::Floats*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      if(val.type()->equal(Domain_Repository::_tc_Doubles)) {
	os << "seq double ";
	Domain_Repository::Doubles* v = 
	  (Domain_Repository::Doubles*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      if(val.type()->equal(Domain_Repository::_tc_Chars)) {
	os << "seq char ";
	Domain_Repository::Chars* v = 
	  (Domain_Repository::Chars*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      if(val.type()->equal(Domain_Repository::_tc_Octets)) {
	os << "seq octet ";
	Domain_Repository::Octets* v = 
	  (Domain_Repository::Octets*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      if(val.type()->equal(Domain_Repository::_tc_Booleans)) {
	os << "seq boolean ";
	Domain_Repository::Booleans* v = 
	  (Domain_Repository::Booleans*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      } 
      if(val.type()->equal(Domain_Repository::_tc_Strings)) {
	os << "seq string ";
	Domain_Repository::Strings* v = 
	  (Domain_Repository::Strings*)val.value();
	os << v->length() << " ";
	for(int i = 0; i < v->length(); i ++) os << (*v)[i] << " ";
	break;
      }
      os << "sequence";
      break;
    }
#endif

    case CORBA::tk_struct: os << "struct"; break;
    case CORBA::tk_union: os << "union"; break;
    case CORBA::tk_array: os << "array"; break;
    default: 
      os << "Unknown type code kind " << (int) (val.type()->kind()) << endl;
      break;
    }
    return os;
}
