// $Id$
//
// $Source$
//
// $Log$
//

#include <iostream.h>
#include "event_log_i.hh"
#include <stdlib.h>

static const CORBA::ULong initial_length= 2000;


class EventLogImpl : public BOA_IMPL_BASE(/**/,EventLog),
                     public ABng_Guarded {
  CORBA::ULong r_max_length;
  CORBA::ULong curr_pos;
  EventLog::Events buf;
  ABng_Server_Manager::Name reg_name;
  int enabled;
public:
  EventLogImpl( const ABng_Server_Manager::Name &cname) : 
    r_max_length( initial_length),
    curr_pos( 0),
    buf( r_max_length),
    enabled( 0),
    reg_name( cname) {
    
    buf.length( r_max_length);
    CONNECT_IMPL( EventLog_var,this);
  }

  void register_log();
  void add_event( const EventLog::UniqueEventID &id);


  void reset( ENV_DECL1(env));
  void start( ENV_DECL1(env));
  void stop( ENV_DECL1(env));

  CORBA::ULong max_length( ENV_DECL1(env));
  void max_length( CORBA::ULong ml ENV_DECL(env));
  EventLog::Events * get_events( CORBA::ULong pos,
				 CORBA::ULong max_seq
				 ENV_DECL(env));
} *event_log= NULL;


void init_event_log( const char *name) {
  if (event_log) return;
  ABng_Server_Manager::Name cname= Bootstrap::CTX_ABng();;
  cname= (cname+"EventLog")+name;
  (event_log= new EventLogImpl( cname))->register_log();
}

void add_event( const EventLog::UniqueEventID &id) {
  if (event_log) event_log->add_event( id);
}

void EventLogImpl::register_log() {
  try {
    Bootstrap::register_server_in_NS( reg_name, 
				      this);
  }
  catch ( CORBA::Exception &ex) {
    cerr << reg_name << endl;
    cerr << "Error while contacting server manager" << ex << endl;
    exit(1);
  }
}

void EventLogImpl::reset( ENV_DEF1(env)) {
  auto_lock local_lock( *this);
  
  curr_pos= 0;
}

void EventLogImpl::start( ENV_DEF1(env)) {
  auto_lock local_lock( *this);

  enabled= 1;
}

void EventLogImpl::stop( ENV_DEF1(env)) {
  auto_lock local_lock( *this);

  enabled= 0;
}

void EventLogImpl::max_length( CORBA::ULong ml ENV_DEF(env)) {
  auto_lock local_lock( *this);

  r_max_length= ml;
  curr_pos= curr_pos>=r_max_length ? r_max_length : curr_pos;
  buf.length( r_max_length);
}

CORBA::ULong EventLogImpl::max_length( ENV_DEF1(env)) {
  auto_lock local_lock( *this);

  return r_max_length;
}

EventLog::Events * EventLogImpl::get_events( CORBA::ULong pos,
					     CORBA::ULong max_seq
					     ENV_DEF(env)) {

  auto_lock local_lock( *this);
  pos= pos>curr_pos ? curr_pos : pos;
  max_seq= max_seq<curr_pos-pos ? max_seq : curr_pos-pos;
  EventLog::Events *ret= new EventLog::Events( max_seq);
  ret->length( max_seq);
  for (CORBA::ULong x=0; x<max_seq; x++)
    (*ret)[x]= buf[x+pos];
  return ret;
}

void EventLogImpl::add_event( const EventLog::UniqueEventID &id) {
  auto_lock local_lock( *this);

  if (!enabled) return;
  if (curr_pos>=r_max_length) return;
  
  buf[curr_pos].unique_id= id;
  ABng_Server_Manager::Time ct= Bootstrap::gettimeofday();
  buf[curr_pos].receive_ts.seconds= ct.seconds;
  buf[curr_pos].receive_ts.microseconds= ct.microseconds;
  ++curr_pos;
}



