// $Id: rep_server.cc,v 1.1 1998/06/16 12:55:00 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/rep_server.cc,v $
//
// $Log: rep_server.cc,v $
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
// File:          rep_server.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  02.02.98
//
// General Description:
//  Implementation of the Rep_Server class which is a skeleton for ABng 
//  repository servers.

#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include "rep_server.h"

Rep_Server::Rep_Server(
#ifdef ObjectStore
    const char *db_name_p
#elif FilePersistency
    const char *file_name_p,
#endif
    const char *current_directory_p
  ) : Basic_Server(), cold_start(0) {

  my_current_directory = strdup(current_directory_p);
#ifdef ObjectStore
  my_db_name = strdup(db_name_p);
#elif FilePersistency
  file_name = strdup(file_name_p);
#endif
}

Rep_Server::~Rep_Server() {

  delete [] my_current_directory;
#ifdef ObjectStore
  delete [] my_db_name;
#elif FilePersistency
  delete [] file_name;
#endif
  delete [] prog_name;
}

// for getopt()
extern char *optarg;
extern int optind;

void Rep_Server::get_options(int argc, char **argv) {

  int c;
#ifdef ObjectStore
  while (( c = getopt (argc, argv, "t:d:sh")) != -1) {
#else 
  while (( c = getopt (argc, argv, "t:d:sh")) != -1) {
#endif
    switch (c) {
      case 's': {
	cold_start++;
	break;
      }
#ifdef ORBIX
      case 't': {
	my_timeout = atol( optarg);
	break;
      }
#endif /* ORBIX */
      case 'd': {
	// set the data directory
	my_current_directory = strdup(optarg);
	break;
      }
      case 'h' : {
	print_info();
	exit(1);
      }
      default: {
	print_info();
	exit(1);
      }
    }
  }
}

void Rep_Server::check_directory(const char *dir_name_p) {
 
  cout << "Data directory is " << dir_name_p << endl;

  // want to change current directory in to the data directory.
  // first, create that dir if necessary
  if (mkdir(dir_name_p, 0770) == -1) {
    if (errno != EEXIST) {	
      cerr << "Cannot create directory: ";
      cerr << dir_name_p << endl;
      exit(1);
    }
  }
  
  if ( chdir(dir_name_p) == -1 ) {
    cerr << "Cannot read directory ";
    cerr << dir_name_p << endl;
    exit(1);
  }
}

#ifdef ObjectStore
os_database *Rep_Server::init_os(const char *db_name_p) {

  os_database *db_p;
  objectstore::initialize();
  if (coldStart) {
    db_p = os_database::create(db_name_p, 0664, 1);
  }
  else {
    db_p = os_database::open(db_name_p);
  }
  oosa->initDB(db_p);
  return db_p;
}
#endif /* ObjectStore */

char *Rep_Server::concat_path_name(
  const char *dir_name_p, 
  const char *file_name_p) {

  // create path of the persistency file for this repository
  char* path = new char[strlen(dir_name_p) + strlen(file_name_p) + 2];
  (void)strcpy(path, dir_name_p);
  if (path[strlen(path) - 1] != '/') {
    (void)strcat(path, "/");
  }
  (void)strcat(path, file_name_p);
  return path;
}

void Rep_Server::init(int argc, char **argv) {

  prog_name = strdup(argv[0]);
  init_corba(argc, argv);
  get_options(argc, argv);
#ifdef ObjectStore
  os_database *db;
  db = init_os(my_db_name);
#else
  check_directory(my_current_directory);
  char *full_name = concat_path_name(my_current_directory, file_name);
  delete [] file_name;
  file_name = full_name;
#endif
#ifdef ORBIX
  set_orbix_conf();
#endif
  create_objects();
}

void Rep_Server::final() {

  delete_objects();
}

void Rep_Server::print_info() {

  cout << "Usage:\t" << prog_name << " [options]" << endl;
  cout << "\t" << prog_name << " -s [options]" << endl << endl;
  cout << "The following options may be used:" << endl;
  cout << "-d directory\tSpecify the database directory." << endl;
  cout << "-s\t\tUsed when the server is launched for the first time (cold" << endl;
  cout << "\t\tstart)." << endl;
#ifdef ORBIX
  cout << "-t timeout\tSpecify a number of miliseconds to wait between Orbix events." << endl;
#endif ORBIX
}






