import org.omg.CORBA.Any;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.Principal;
import java.io.PrintWriter;

  class AP_Context {
    private int marg= 0;
    private int step;
    
    public AP_Context( int step) {
      this.step= step;
    }
    
    public void push()   { marg+= step; }
    public void pop()	 { marg-= step; }
    public void fill( PrintWriter pw) {
      for ( int i=0; i<marg; i++)
	pw.print(' ');
    }
  };

public class AnyPrinter {

  
  public static void print( Any any, PrintWriter pw) 
	    throws org.omg.CORBA.TypeCodePackage.BadKind,
		   org.omg.CORBA.TypeCodePackage.Bounds {
    org.omg.CORBA.portable.InputStream pis= any.create_input_stream();
    TypeCode tc= any.type();
    print( pis, tc, pw, new AP_Context( 2));
  }
  
  private static void print( 
	org.omg.CORBA.portable.InputStream pis,
	TypeCode tc,
	PrintWriter pw,
	AP_Context apc) 
	    throws org.omg.CORBA.TypeCodePackage.BadKind,
		   org.omg.CORBA.TypeCodePackage.Bounds {
    
    apc.fill( pw);
    switch ( tc.kind().value()) {
      case TCKind._tk_null:	pw.println("(tk_null)");
				break;
      case TCKind._tk_void:	pw.println("(tk_void)");
				break;
      case TCKind._tk_short:	pw.println("(tk_short) "+pis.read_short());
				break;
      case TCKind._tk_long:	pw.println("(tk_long) "+pis.read_long());
				break;
      case TCKind._tk_ushort:	pw.println("(tk_ushort) "+pis.read_ushort());
				break;
      case TCKind._tk_ulong:	pw.println("(tk_ulong) "+pis.read_ulong());
				break;
      case TCKind._tk_float:	pw.println("(tk_float) "+pis.read_float());
				break;
      case TCKind._tk_double:	pw.println("(tk_double) "+pis.read_double());
				break;
      case TCKind._tk_boolean:  pw.println("(tk_boolean) "+pis.read_boolean());
				break;
      case TCKind._tk_char:	pw.println("(tk_char) "+pis.read_char());
				break;
      case TCKind._tk_octet:	pw.println("(tk_octet) "+pis.read_octet());
				break;
      case TCKind._tk_any:	pw.println("(tk_any)");
				apc.push();
				TypeCode new_tc= pis.read_TypeCode();
				print( pis, new_tc, pw, apc);
				apc.pop();
				break;
      case TCKind._tk_TypeCode:	pw.println("(tk_TypeCode) {");
				apc.push();
				TypeCode new_tc2= pis.read_TypeCode();
				print( new_tc2, pw, apc);
				apc.pop();
				apc.fill( pw);
				pw.println("}");
				break;
      case TCKind._tk_Principal:
				Principal principal= pis.read_Principal();
				pw.print("(tk_Principal) <");
				for ( int i=0; i<principal.name().length; i++)
				  pw.print( principal.name()[i]+" ");
				pw.println(">");
				break;
      case TCKind._tk_objref:	org.omg.CORBA.Object obj= pis.read_Object();
				pw.println("(tk_objref) "+
				    org.omg.CORBA.ORB.init().object_to_string( obj));
				break;
      case TCKind._tk_struct:	pw.println("(tk_struct) "+tc.name()+" "+tc.id() +"{");
				apc.push();
				for ( int i=0; i<tc.member_count(); i++) {
				  apc.fill( pw);
				  pw.println( tc.member_label( i));
				  apc.push();
				  print( pis, tc.member_type(i), pw, apc);
				  apc.pop();
				}
				apc.pop();
				apc.fill( pw);
				pw.println("}");
				break;
      case TCKind._tk_union:	pw.println("(tk_union) "+tc.name()+" "+tc.id() +"{");
				apc.push();
				
				apc.fill( pw);
				pw.print("discriminator:");
				apc.push();
				Any dany= org.omg.CORBA.ORB.init().create_any();
				dany.type( tc.discriminator_type());
				dany.read_value( pis, dany.type());
				
				print( dany.create_input_stream(), dany.type(), pw, apc);
				apc.pop();
				
				apc.fill( pw);
				pw.print("value:");
				apc.push();
				for ( int i=0; i<tc.member_count(); i++)
				  if ( dany.equal( tc.member_label( i)))
				    print( pis, dany.type(), pw, apc);
				apc.pop();
				apc.pop();
				pw.println("}");
				break;
      case TCKind._tk_enum:	pw.println("(tk_enum) "+tc.member_name( pis.read_ulong()));
				break;
      case TCKind._tk_string:	pw.println("(tk_string) "+pis.read_string());
				break;
      case TCKind._tk_sequence:	int seq_size= pis.read_ulong();
				pw.println("(tk_sequence) <"+tc.length()+">  ["+seq_size+"] {");
				apc.push();
				for ( int i=0; i<seq_size; i++) {
				  apc.fill( pw);
				  pw.println("#"+i);
				  print( pis, tc.content_type(), pw, apc);
				}
				apc.pop();
				apc.fill( pw);
				pw.println("}");
				break;
      case TCKind._tk_array:	int arr_size= tc.length();
				pw.println("(tk_array) ["+arr_size+"] {");
				apc.push();
				for ( int i=0; i<arr_size; i++) {
				  apc.fill( pw);
				  pw.println("#"+i);
				  print( pis, tc.content_type(), pw, apc);
				}
				apc.pop();
				apc.fill( pw);
				pw.println("}");
				break;
      default:			pw.println("Unimplemented or unknown typecode kind "+tc.kind().value());
				break;
    }
  }
  
  private static void print(
	TypeCode tc,
	PrintWriter pw,
	AP_Context apc) {
	
    apc.fill( pw);
    pw.println("Unimplemented");
  }
}

