// -*- Mode: C++; -*-
//
//  File:         abng_location_descr_rep_i.h
//  Author:       Malgorzata Steinder (gosia@ics.agh.edu.pl)
//  Date Created: 17.12.97
//
// General Description:
//   Header file for Abng_Location_Descr_Rep implementation class
//

#ifndef abng_location_descr_rep_i_h
#define abng_location_descr_rep_i_h

#include <stdlib.h>
#include <fstream.h>
#include <string.h>
#include <rw/tvvector.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "abng_location_descr_i.h"
#include "bootstrap_n.hh"
#include "basic_rep_i.h"
#include "process_status.hh"


class ALD_SDO_Types {

public:

  typedef ABng_Core_System::System_Static_Data_Observed_R Observed;
  typedef ABng_Core_System::System_Static_Data_Observer Observer;
  typedef ABng_Core_System::System_Static_Data_Observer_ptr Observer_ptr;
  typedef ABng_Core_System::System_Static_Data_Observed_E
          ::System_Static_Data_Observers Observers;
};

class ALD_Types {

public:

  typedef ABng_Core_System::ABng_Location_Description Item;  
  typedef ABng_Core_System::ABng_Location_Description_ptr Item_ptr;  
  typedef ABng_Core_System::ABng_Location_Descriptions Items;  
  typedef ABng_Loc_Descr_I Item_I;  
  typedef ABng_Core_System::ABng_Location_Description::Description 
               Initial_Data;
  typedef ABng_Core_System::System_Static_Data_Observer
          ::System_Static_Data_Object Data_Object;
};

class ABng_Loc_Descr_Rep_I  : public Bootstrap_listener,
			      public Basic_Rep_I<ALD_Types, ALD_SDO_Types>,
			      public ServerStatusInterface  { 

public:

  ABng_Loc_Descr_Rep_I (
#ifdef ObjectStore
    os_database* db_p
#elif defined FilePersistency
    const char *repository_name_p
#endif
  );

  virtual ~ABng_Loc_Descr_Rep_I ();

  void init( int init_v);

  // get instance of this class 
  //  - Singleton pattern (E. Gamm & al.,"Design Paterns", p. 127)
  static ABng_Loc_Descr_Rep_I* 
    instance ();

  void remove_sensor(Domain_Repository::Repository_Item_Id);

  void public_notify_update(ABng_Loc_Descr_I *);
 
  virtual int validate_update_add_data(Item_I*, 
    const ABng_Core_System::ABng_Location_Description::Description& 
  );


  //
  // IDL operations of the System_Static_Data_Observed_R interface
  //                       _____________________________

  virtual ABng_Core_System::System_Static_Data_Observed_R::Observer_Entries*
   system_static_data_observer_list (
	ENV_DECL1( env)
   ) 
	throw (CORBA::SystemException) {
        return observer_list(USE_ENV1(env));
  }
             
 virtual 
 ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   add_system_static_data_observer_list (
	ENV_DECL1( env)
   ) 
	throw (CORBA::SystemException) {
        return add_observer_list(USE_ENV1(env));
   }
           
  virtual void 
  attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL( env)
   ) {
        attach_observer(obs, observed_id_list USE_ENV(env));
  }

  virtual void 
   detach_system_static_data_observer (
        ABng_Core_System::System_Static_Data_Observer_ptr obs, 
        const Domain_Repository::Repository_Item_Ids& observed_id_list
        ENV_DECL( env)
   ) {
        detach_observer( obs, observed_id_list USE_ENV(env));
  }



  virtual void 
   attach_add_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs 
	ENV_DECL( env)
   )
	throw (CORBA::SystemException) {
        attach_add_observer(obs USE_ENV(env));
  }

  virtual void 
   detach_add_system_static_data_observer (
        ABng_Core_System::System_Static_Data_Observer_ptr obs
        ENV_DECL( env)
   ) {
        detach_add_observer(obs USE_ENV(env));
  }

  // Suplementary operation

 virtual 
 ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   system_static_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DECL( env)
   ) {
        return get_item_observers(item_id_p USE_ENV(env));
  }


  //
  // IDL operations of the ABng_Location_Description_Rep interface
  //                       __________

  virtual ABng_Core_Management::ABng_Location_Description_Rep *
   rep(
	ENV_DECL1( env)
   );
  
  virtual 
  ABng_Core_System::ABng_Location_Descriptions* 
   abng_location_descriptions_list (
	ENV_DECL1( env)
   ) {
    return item_list(USE_ENV1(env));
  }

  virtual 
  ABng_Core_System::ABng_Location_Descriptions* find(
        const ABng_Core_System::ABng_Location_Description::Pattern& 
            data_record_pattern
	ENV_DECL( env)
  ) {   // no locking here 

    return Basic_Rep_I<ALD_Types, ALD_SDO_Types>::find(data_record_pattern USE_ENV(env));
  }
  
  virtual 
  ABng_Core_System::ABng_Location_Descriptions* private_find(
        const ABng_Core_System::ABng_Location_Description::Pattern& 
                data_record_pattern
        ENV_DECL( env)
   );

  virtual ABng_Core_System::ABng_Location_Description::Value_Descriptions* 
   find_values(const ABng_Core_System::ABng_Location_Description::Pattern& 
                     data_record_pattern
	       ENV_DECL( env)
   ) {
            return Basic_Rep_I<ALD_Types, ALD_SDO_Types>::find_values( 
                        data_record_pattern
                        USE_ENV(env)
                   );
  }

  virtual ABng_Core_System::ABng_Location_Description::Query_Results*
   complex_query(
		 const ABng_Core_System::ABng_Location_Description::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core_System::System_Static_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return Basic_Rep_I<ALD_Types, ALD_SDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual ABng_Core_System::ABng_Location_Description_ptr 
   add(const ABng_Core_System::ABng_Location_Description::Description& 
             data_record
       ENV_DECL( env)
   ) {
            return Basic_Rep_I<ALD_Types, ALD_SDO_Types>::add(data_record USE_ENV(env));
  }

  virtual void 
   remove(ABng_Core_System::ABng_Location_Description_ptr object_to_remove 
	  ENV_DECL( env)
   ) {
    
         Basic_Rep_I<ALD_Types, ALD_SDO_Types>::remove(object_to_remove USE_ENV(env));
  }

  virtual void 
   update (ABng_Core_System::ABng_Location_Description_ptr object_to_update, 
	   const ABng_Core_System::ABng_Location_Description::Description& 
              data_record_pattern
	   ENV_DECL( env)
   ) {

    Basic_Rep_I<ALD_Types, ALD_SDO_Types>::update(object_to_update, 
                                             data_record_pattern
                                             USE_ENV(env)
    );
  }

#ifdef ObjectStore
  static os_typespec* get_os_typespec();
#endif

 protected:

  // calculate the hash value of the ABng_Loc_Description
  static unsigned hash_abng_loc_descr(
    const ABng_Loc_Descr_I& abng_loc_descr_i_p); 

  // definition of structure containing parameters of 
  // match_abng_loc_descr method
  struct MatchABng_Loc_DescrApplyDataStruct {

    ABng_Core_System::ABng_Location_Descriptions * _found_abng_loc_descrs;
    ABng_Core_System::ABng_Location_Description::Pattern _data_record_pattern;
    Domain_Repository::Location_Description::Value_Descriptions*
           _loc_value_descrs;
    RWTValVector<ABng_Core_System::Sensor::Value_Descriptions*>
           _sensor_value_descrs;
    Domain_Repository::Location_Description_Rep_var _location_description_rep;
    ABng_Core_Management::Sensor_Rep_var _sensor_rep;


    MatchABng_Loc_DescrApplyDataStruct(
        ABng_Core_System::ABng_Location_Descriptions* found_abng_loc_descrs,
        const ABng_Core_System::ABng_Location_Description::Pattern& 
            data_record_pattern,
        Domain_Repository::Location_Description_Rep_var 
            location_description_rep,
        ABng_Core_Management::Sensor_Rep_var sensor_rep
    );

    ~MatchABng_Loc_DescrApplyDataStruct();

    int match_loc_descr(Domain_Repository::Location_Description::Pattern&,
                        Domain_Repository::Repository_Item_Id);
    int match_sensor(ABng_Core_System::Sensor::Pattern&,
                     Domain_Repository::Repository_Item_Ids*, int, int&
        );
    int match_sensors_pattern(ABng_Loc_Descr_I *);
  };

  // if the current ABng_Loc_Description matches the pattern 
  // puts it into the container
  
  static void match_abng_loc_descr(ABng_Loc_Descr_I*, void*);

  Domain_Repository::Location_Description_Rep_var location_description_rep;
  ABng_Core_Management::Sensor_Rep_var sensor_rep;

  void reference_resolved(const ABng_Server_Manager::Name &name,
			  CORBA::Object_var server_ref,
			  ABng_Server_Manager::uid callback_id);

  virtual int remove_permitted(ABng_Loc_Descr_I *);

  virtual void init_corba_object(ABng_Loc_Descr_I *item_p);
  virtual DB_Header *create_db_header();

  virtual void construct_data_object(
    ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& 
       data_object_p,
    ABng_Core_System::ABng_Location_Description_ptr item_p);


  virtual int validate_update_data(Item_ptr item_p, 
    const ABng_Core_System::ABng_Location_Description::Description& 
          data_record_pattern
  );

  virtual int validate_add_data(
    const ABng_Core_System::ABng_Location_Description::Description& 
          data_record_pattern
  );

  virtual void collect_values(ABng_Loc_Descr_I *lt, 
    ABng_Core_System::ABng_Location_Description::Value_Description& ld);

  virtual void create_corba_objects();
  virtual void destroy_corba_objects();

  // Attributes
  // pointer to an unique istance of the class
  static ABng_Loc_Descr_Rep_I* _instance;

  // repository
  ABng_Core_Management::ABng_Location_Description_Rep_ptr my_rep;
  ProcessControlImpl *ps_dev;
};

void restore_loc_descr(ABng_Loc_Descr_I *, void*);
void restore_sensor_list(ABng_Loc_Descr_I *, void*);

#endif /* abng_location_descr_rep_i_h */














