// $Id: away_i.hh,v 1.1.1.1 1998/07/29 14:16:23 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/away_rep/away_i.hh,v $
//
// $Log: away_i.hh,v $
// Revision 1.1.1.1  1998/07/29 14:16:23  mencnar
// initial revision
//
//
// File:          away_i.cc
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  07.07.98
//
// General Description:
//  Header file for the implementation class of the Away interface 
//

#ifndef away_i_h
#define away_i_h

#include <stdio.h>
#include <fstream.h>
#include <errno.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#ifdef FilePersistency
#include <rw/rwfile.h>
#endif

#include "abng_core_management.hh"
#include "abng_poller.hh"
#include "abng_guarded.hh"
#include "compatibility.hh"

int operator < (const Domain_Repository::Time &t1,
		const Domain_Repository::Time &t2);
int operator == ( const ABng_Poller::AB_Poller::Badge_Id &first,
                  const ABng_Poller::AB_Poller::Badge_Id &second);

class Away_I
{
 private:
  // state of the IDL Away object
  ABng_Core::Away::Description my_descr;
  Domain_Repository::Repository_Item_Id badge_holder_item_id;
  Domain_Repository::Repository_Item_Id my_badge_item_id;
  ABng_Core_System::Badge_Holder_var my_holder;
  ABng_Core_System::Badge_var my_badge;
  ABng_Poller::AB_Poller::Badge_Id badge_id;
  ABng_Guarded &my_guard;
  int pending_delete;

  // reference of CORBA object encapsulated this C++ object
  ABng_Core::Away_ptr my_corba_object;

 public:
  // ctor
   Away_I (
	const ABng_Core_Management::Away_Rep_Manager::Change_Description& descr_init
   );

  Domain_Repository::Repository_Item_Id holder_item_id() {
    return badge_holder_item_id;
  }

  Domain_Repository::Repository_Item_Id badge_item_id() {
    return my_badge_item_id;
  }

  ABng_Poller::AB_Poller::Badge_Id get_badge_id() {
    return badge_id;
  }

  ABng_Core_System::Badge_Holder_var get_badge_holder() {
    return my_holder;
  }

  int cancelled_by_sighting( Domain_Repository::Time &current_time) {
    return my_descr.cancel_by_badge && my_descr.cancel_from_time<current_time;
  }

  void mark_for_delete() {
    pending_delete= 1;
  }

  int marked_for_delete() {
    return pending_delete;
  }

  void set_active() {
    my_descr.active= (CORBA::Boolean) 1;
  }

  void reset_active() {
    my_descr.active= (CORBA::Boolean) 0;
  }

#ifdef FilePersistency
   Away_I (
	istream &iostr
   );
#endif

  virtual 
   ~Away_I ();

   operator==(	
	const Away_I&
   );

  //
  // methods enabling access to the state
  //
  void
   set_descr (
	const ABng_Core_Management::Away_Rep_Manager::Change_Description& descr_update
   );

  const ABng_Core::Away::Description &get_descr() const {
    return my_descr;
  }

  void fast_change_holder( ABng_Core_System::Badge_Holder_var new_badge_holder_var,
			   ABng_Core_System::Badge_Holder::Description &bhd,
			   ABng_Core_System::Badge_Holder::Value_Description &bhvd,
			   ABng_Core::Away::Description &new_descr);

  void change_holder( ABng_Core_System::Badge_Holder_var new_holder_var,
		      ABng_Core::Away::Description &new_descr);

  void
   set_corba_object (
	ABng_Core::Away_ptr corba_object_p
   );

  ABng_Core::Away_ptr
   get_corba_object ();

  void 
   set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   );

  //
  // IDL operations of the Away_Data_Observed_E interface
  //                       _____________________________

  virtual ABng_Core::Away_Data_Observed_E::Away_Data_Observers*
    away_data_observer_list (
	ENV_DECL1( env)
    );
                
  virtual void 
    attach_away_data_observer (
	ABng_Core::Away_Data_Observer_ptr obs
	ENV_DECL(env)
    );

  virtual void 
    detach_away_data_observer (
	ABng_Core::Away_Data_Observer_ptr obs
        ENV_DECL(env)
    );

  void resolve_badge_holder( ABng_Core_Management::Badge_Holder_Rep_var holder_rep);

  //
  // IDL operations of the Away interface
  //                       ______
 
  virtual ABng_Core::Away::Description* 
   descr (
	ENV_DECL1(env)
   );

  virtual ABng_Core::Away::Value_Description* 
   value_descr (
	ENV_DECL1(env)
   );
 
  virtual Domain_Repository::Repository_Item_Id 
   item_id (
	ENV_DECL1(env)
   );

  virtual ABng_Core::Locatable_Type_Specific_Object*
   type_specific_object(
	ENV_DECL1(env)
   );

  virtual Domain_Repository::Time
   start_time(
        ENV_DECL1(env)
   );

  virtual Domain_Repository::Time
   stop_time(
        ENV_DECL1(env)
   );

  virtual Domain_Repository::Time
   cancel_from_time(
        ENV_DECL1(env)
   );

  virtual char *
   message(
        ENV_DECL1(env)
   );

  virtual char *
   contact(
        ENV_DECL1(env)
   );

  virtual CORBA::Boolean
   cancel_by_badge(
        ENV_DECL1(env)
   );

  virtual CORBA::Boolean
   active(
        ENV_DECL1(env)
   );

#ifdef ObjectStore
  //
  // method for ObjectStore
  //

  static os_typespec* get_os_typespec();

#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void
   save (
	ostream & istr
   );

  virtual void
   restore (
	istream &ostr
   );
#endif
};   

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_Away(Away_I)
#else
DEF_TIE_ABng_Core_Away(Away_I)
#endif
#elif defined OMNIORB

#endif

#endif /* away_i_hh */



