//
//  File:         badge_holder_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 21.12.97
//
// General Description:
//   Implementation file for the Badge_Holder_I class
//

#define EXCEPTIONS
#include "badge_holder_i.h"
#include "badge_holder_rep_i.h"
#include <rw/tvdlist.h>
#include "db_utils.hh"

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

//
// Implementation of constructor, destructor and operators
//                   _____________________________________

  Badge_Holder_I::Badge_Holder_I (
	const ABng_Core_System::Badge_Holder::Description& descr_init
  ) : 
		my_guard(*(Badge_Holder_Rep_I::instance())),
		my_badge_id(0),
		current_badge_id(0),
		home_sensor_id(0),
		type_specific_object_id(0)
{
  // init my_descr
  my_descr= descr_init;

  // init my_corba_object my_badge_id
  my_corba_object = (ABng_Core_System::Badge_Holder_ptr)NULL;
  
  // init additional data - my_badge_id
  if (!CORBA::is_nil(my_descr.my_badge)) {
    try {
      my_badge_id = my_descr.my_badge->item_id();
    }
    catch (...) {}
  }
  
  // init additional data -  current_badge_id
  if (!CORBA::is_nil(my_descr.current_badge)) {
    try {
      current_badge_id = my_descr.current_badge->item_id();
    }
    catch (...) {}
  }
  
  // init additional data - home_sensor_id
  if (!CORBA::is_nil(my_descr.home_sensor)) {
    try {
      home_sensor_id = my_descr.home_sensor->item_id();
    }
    catch (...) {}
  }
  
  // init additional data - badge_holder_kind
  badge_holder_kind = my_descr.type_specific_object._d();
  
  // init additional data - type_specific_object_id
  if (badge_holder_kind == ABng_Core::LOCATABLE_USER) { 
    if (!CORBA::is_nil(my_descr.type_specific_object.user_obj())) {
      try {
	type_specific_object_id = 
	  my_descr.type_specific_object.user_obj()->item_id();
	my_descr.type_specific_object.user_obj()->attach_static_data_observer(
       		    Badge_Holder_Rep_I::instance()->observer_corba_object);

      }
      catch (...) {}
    }
  }
  else {
    if (!CORBA::is_nil(my_descr.type_specific_object.equipment_obj())) {
      try {
	type_specific_object_id =
	  my_descr.type_specific_object.equipment_obj()->item_id();
      }
      catch (...) {}
    }
  }
}

#ifdef FilePersistency
   Badge_Holder_I::Badge_Holder_I (
	istream &iostr
   ) :
		my_guard(*(Badge_Holder_Rep_I::instance()))
{
	// read description from the file
	restore(iostr);
  
	// init my_corba_object
	my_corba_object = (ABng_Core_System::Badge_Holder_ptr)NULL;

	my_descr.my_badge = (ABng_Core_System::Badge_ptr)NULL;
	my_descr.current_badge = (ABng_Core_System::Badge_ptr)NULL;
	my_descr.home_sensor = (ABng_Core_System::Sensor_ptr)NULL;
	
	if (badge_holder_kind == ABng_Core::LOCATABLE_USER) 
		my_descr.type_specific_object.user_obj((Domain_Repository::User_ptr)NULL);
	else 
		my_descr.type_specific_object.equipment_obj((Domain_Repository::Equipment_ptr)NULL);
}
#endif

Badge_Holder_I::~Badge_Holder_I (
  )
{
#ifdef DEBUG
  cerr << "Badge_Holder_I destructor " << (int) this << endl;
#endif
}

  Badge_Holder_I::operator== (
	const Badge_Holder_I& badge_holder_p
  )
{
	if (current_badge_id == badge_holder_p.current_badge_id)
		return 1;
 	else
		return 0;
}

//
// Implementation of methods enabling access to the state
//

void
  Badge_Holder_I::set_descr (
	const ABng_Core_System::Badge_Holder::Description& descr_update
  )
{
  // update my_descr
  /// NO !!!! my_descr.item_id = descr_update.item_id;
  my_descr.my_badge = descr_update.my_badge;
  my_descr.current_badge = descr_update.current_badge;
  my_descr.home_sensor = descr_update.home_sensor;
  my_descr.assigned = descr_update.assigned;
  my_descr.visitor = descr_update.visitor;
  if (descr_update.type_specific_object._d() == ABng_Core::LOCATABLE_USER) 
    my_descr.type_specific_object.
      user_obj( Domain_Repository::User::
		_duplicate( descr_update.type_specific_object.user_obj()));
  else 
    my_descr.type_specific_object.
      equipment_obj( Domain_Repository::Equipment::
	      _duplicate( descr_update.type_specific_object.equipment_obj()));

  // init additional data - my_badge_id
  if (!CORBA::is_nil(my_descr.my_badge)) {
    try {
      my_badge_id = my_descr.my_badge->item_id();
    }
    catch (...) {}
  }

  // init additional data -  current_badge_id
  if (!CORBA::is_nil(my_descr.current_badge)) {
    try {
      current_badge_id = my_descr.current_badge->item_id();
    }
    catch (...) {}
  }
  
  // init additional data - home_sensor_id
  if (!CORBA::is_nil(my_descr.home_sensor)) {
    try {
      home_sensor_id = my_descr.home_sensor->item_id();
    }
    catch (...) {}
  }
  
  // init additional data - badge_holder_kind
  badge_holder_kind = my_descr.type_specific_object._d();
  
  // init additional data - type_specific_object_id
  if (badge_holder_kind == ABng_Core::LOCATABLE_USER) {
    if (!CORBA::is_nil(my_descr.type_specific_object.user_obj())) {
      try {
	type_specific_object_id = 
	  my_descr.type_specific_object.user_obj()->item_id();
	my_descr.type_specific_object.user_obj()->attach_static_data_observer(
       		    Badge_Holder_Rep_I::instance()->observer_corba_object);
      }
      catch (...) {}
    }
  }
  else {
    if (!CORBA::is_nil(my_descr.type_specific_object.equipment_obj())) {
      try {
	type_specific_object_id =
	  my_descr.type_specific_object.equipment_obj()->item_id();
      }
      catch (...) {}
    }
  }
}

const ABng_Core_System::Badge_Holder::Description&
   Badge_Holder_I::get_descr (
   ) const
{
	return my_descr;  
}

void
   Badge_Holder_I::set_corba_object (
	ABng_Core_System::Badge_Holder_ptr corba_object_p
   )
{
	my_corba_object = corba_object_p;
}

const ABng_Core_System::Badge_Holder_ptr
   Badge_Holder_I::get_corba_object (
   ) 
{
	return my_corba_object;
}

void 
   Badge_Holder_I::set_item_id (
	const Domain_Repository::Repository_Item_Id item_id_p
   )
{
	my_descr.item_id = item_id_p;
}

CORBA::Boolean Badge_Holder_I::assigned( ENV_DEF1(env)) {
  ABng_Guarded::auto_lock local_lock(my_guard);
  return my_descr.assigned;
}

CORBA::Boolean Badge_Holder_I::visitor( ENV_DEF1(env)) {
  ABng_Guarded::auto_lock local_lock(my_guard);
  return my_descr.visitor;
}

void 
   Badge_Holder_I::resolve_my_badge( 
	ABng_Core_Management::Badge_Rep_var badge_rep
   ) 
{
	ABng_Core_System::Badge::Pattern pat;

	pat.is_any = 0;
	pat.item_id_pattern.is_any = 0;
	pat.item_id_pattern.value = my_badge_id;
	pat.id_pattern.is_any = 1;

	try {
		ABng_Core_System::Badges_var ret_val(badge_rep->find(pat));
		if (ret_val->length() == 1) 
			my_descr.my_badge = ABng_Core_System::Badge::_duplicate(ret_val[0]);
		else {
			cerr << endl << "Error in Badge_Rep";
			cerr << " current badge_holder_id = " << my_descr.item_id;
			cerr << " (my)badge id being resolved = " << my_badge_id;
			cerr << endl;
		}
	}
	catch (CORBA::COMM_FAILURE &ex) { 
		throw;
	}
	catch (...) {}
}

void 
   Badge_Holder_I::resolve_current_badge( 
	ABng_Core_Management::Badge_Rep_var badge_rep
   ) 
{
  if (current_badge_id == 0) return;

  ABng_Core_System::Badge::Pattern pat;
  
  pat.is_any = 0;
  pat.item_id_pattern.is_any = 0;
  pat.item_id_pattern.value = current_badge_id;
  pat.id_pattern.is_any = 1;
  
  try {
    ABng_Core_System::Badges_var ret_val(badge_rep->find(pat));
    if (ret_val->length() == 1) 
      my_descr.current_badge = ABng_Core_System::Badge::_duplicate(ret_val[0]);
    else {
      cerr << endl << "Error in Badge_Rep";
      cerr << " current badge_holder_id = " << my_descr.item_id;
      cerr << " (current)badge id being resolved = " << current_badge_id;
      cerr << endl;
    }
  }
  catch (CORBA::COMM_FAILURE &ex) { 
    throw;
  }
  catch (...) {}
}

void 
   Badge_Holder_I::resolve_home_sensor(
	ABng_Core_Management::Sensor_Rep_var sensor_rep
   )
{
  if (home_sensor_id == 0) return;

  ABng_Core_System::Sensor::Pattern pat;
  
  pat.is_any = 0;
  pat.item_id_pattern.is_any = 0;
  pat.item_id_pattern.value = home_sensor_id;
  pat.id_pattern.is_any = 1;
	
  try {
    ABng_Core_System::Sensors_var ret_val(sensor_rep->find(pat));
    if (ret_val->length() == 1) 
      my_descr.home_sensor = ABng_Core_System::Sensor::_duplicate(ret_val[0]);
    else {
      cerr << endl << "Error in Sensor_Rep";
      cerr << " current sensor_holder_id = " << my_descr.item_id;
      cerr << " sensor id being resolved = " << home_sensor_id;
      cerr << endl;
    }
  }
  catch (CORBA::COMM_FAILURE &ex) { 
    throw;
  }
  catch (...) {}
}

void 
   Badge_Holder_I::resolve_user(
	Domain_Repository::User_Rep_var user_rep
   )
{
  if (type_specific_object_id == 0) return;

  Domain_Repository::User::Pattern pat;
  
  pat.is_any = 0;
  pat.item_id_pattern.is_any = 0;
  pat.item_id_pattern.value = type_specific_object_id;
  pat.uid_pattern.is_any = pat.forename_pattern.is_any = pat.surname_pattern.is_any = 1;
  pat.position_pattern.is_any = pat.phone_pattern.is_any = pat.email_pattern.is_any = 1;

  try {
    Domain_Repository::Users_var ret_val(user_rep->find(pat));
    if (ret_val->length() == 1) {
      my_descr.type_specific_object.user_obj(Domain_Repository::User::_duplicate(ret_val[0]));
      ret_val[0]->attach_static_data_observer(
       		    Badge_Holder_Rep_I::instance()->observer_corba_object);
    }
    else {
      my_descr.type_specific_object.user_obj( NULL);
      cerr << endl << "Error in User_Rep";
      cerr << " current user_id = " << my_descr.item_id;
      cerr << " user id being resolved = " << type_specific_object_id;
      cerr << endl;
    }
  }
  catch (CORBA::COMM_FAILURE &ex) { 
    throw;
  }
  catch (...) {}
}

void 
   Badge_Holder_I::resolve_equipment(
	Domain_Repository::Equipment_Rep_var equipment_rep
   )
{
  if (type_specific_object_id == 0) return;

  Domain_Repository::Equipment::Pattern pat;

  pat.is_any = 0;
  pat.item_id_pattern.is_any = 0;
  pat.item_id_pattern.value = type_specific_object_id;
  pat.meta_data_pattern.is_any = pat.name_pattern.is_any = pat.last_command_pattern.is_any = pat.property_list_pattern.is_any= 1;

  pat.meta_data_pattern.my_control_info_pattern.my_control_data_pattern.
    default_field_pattern( Unsigned_Long_Pattern());
  pat.property_list_pattern.constraints = 
    Domain_Repository::UNRESTRICTED;

  try {
    Domain_Repository::Equipments_var ret_val(equipment_rep->find(pat));
    if (ret_val->length() == 1) 
      my_descr.type_specific_object.equipment_obj(
		    Domain_Repository::Equipment::_duplicate(ret_val[0]));
    else {
      cerr << endl << "Error in Equipment_Rep";
      cerr << " current equipment_id = " << my_descr.item_id;
      cerr << " equipmet id being resolved = " << type_specific_object_id;
      cerr << endl;
    }
  }
  catch (CORBA::COMM_FAILURE &ex) { 
    throw;
  }
  catch (CORBA::Exception &cex) {
    cerr << cex << endl;
  }
}
//
// Implementation of IDL operations of the System_Static_Data_Observed_E interface
//                                         _____________________________

ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   Badge_Holder_I::system_static_data_observer_list (
        ENV_DEF1(env)
   ) 
                throw (CORBA::SystemException)
{
        return Badge_Holder_Rep_I::instance()->system_static_data_observer_list(
                my_descr.item_id USE_ENV(env));
}
                
void 
   Badge_Holder_I::attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DEF(env)
   )
		throw (CORBA::SystemException)
{
	// ignore observed_id_list parameter
  	//  initialize the sequence for this badge_holder item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this badge_holder item it into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
	Badge_Holder_Rep_I::instance()->attach_system_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

void 
   Badge_Holder_I::detach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
                       Domain_Repository::Unknown_Observer)
{
	// ignore observed_id_list parameter
	//  initialize the sequence for this badge_holder item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this  item it into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
	Badge_Holder_Rep_I::instance()->detach_system_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

//
// Implementation of IDL operations of the Badge_Holder interface
//                                         ____________
 
ABng_Core_System::Badge_Holder::Description* 
  Badge_Holder_I::descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	if (my_badge_id != 0 && CORBA::is_nil(my_descr.my_badge))
		throw Domain_Repository::Destination_Unreachable("badge_rep");

	if (current_badge_id != 0 && CORBA::is_nil(my_descr.current_badge))
		throw Domain_Repository::Destination_Unreachable("badge_rep");

	if (home_sensor_id != 0 && CORBA::is_nil(my_descr.home_sensor))
		throw Domain_Repository::Destination_Unreachable("sensor_rep");

	if (badge_holder_kind == ABng_Core::LOCATABLE_USER) {
		if (type_specific_object_id!= 0 && 
		    CORBA::is_nil(my_descr.type_specific_object.user_obj())) 
			throw Domain_Repository::Destination_Unreachable("user_rep");

	}
	else {
		if (type_specific_object_id != 0 && 
		    CORBA::is_nil(my_descr.type_specific_object.equipment_obj()))
	 		throw Domain_Repository::Destination_Unreachable("equipment_rep");
	}

	// duplicate my_descr
	ABng_Core_System::Badge_Holder::Description* my_descr_duplicated =
		new ABng_Core_System::Badge_Holder::Description(my_descr);

	return my_descr_duplicated;
}
 
ABng_Core_System::Badge_Holder::Value_Description* 
  Badge_Holder_I::value_descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	return private_value_descr();
}

ABng_Core_System::Badge_Holder::Value_Description* 
  Badge_Holder_I::private_value_descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
  if (my_badge_id != 0 && CORBA::is_nil(my_descr.my_badge))
    throw Domain_Repository::Destination_Unreachable("badge_rep");
  
  if (current_badge_id != 0 && CORBA::is_nil(my_descr.current_badge))
    throw Domain_Repository::Destination_Unreachable("badge_rep");
  
  if (home_sensor_id != 0 && CORBA::is_nil(my_descr.home_sensor))
    throw Domain_Repository::Destination_Unreachable("sensor_rep");

  if (badge_holder_kind == ABng_Core::LOCATABLE_USER) {
    if (type_specific_object_id!= 0 && 
	CORBA::is_nil(my_descr.type_specific_object.user_obj())) 
      throw Domain_Repository::Destination_Unreachable("user_rep");
  }
  else {
    if (type_specific_object_id != 0 && 
	CORBA::is_nil(my_descr.type_specific_object.equipment_obj()))
      throw Domain_Repository::Destination_Unreachable("equipment_rep");
  }

  // new Value_Description
  ABng_Core_System::Badge_Holder::Value_Description* my_value_descr =
    new ABng_Core_System::Badge_Holder::Value_Description;

  my_value_descr->item_id = my_descr.item_id;
  my_value_descr->assigned = my_descr.assigned;
  my_value_descr->visitor = my_descr.visitor;

  // collect state from badge_rep
  try {
    if (!CORBA::is_nil(my_descr.my_badge)) {
      cerr << "<<value_descr for " << my_descr.item_id << endl;
      ABng_Core_System::Badge::Value_Description_var bvdesc=
	my_descr.my_badge->value_descr();
      cerr << ">>value_descr for " << my_descr.item_id << endl;
#ifdef ORBIX
      // Orbix bug workaround
      if (CORBA::is_nil(bvdesc->holder))
	bvdesc->holder= NULL;
#endif
      my_value_descr->my_badge = *(bvdesc.operator->());
    }
    else {
      cerr << endl << "Error - badge_holder item id = " << my_descr.item_id 
	   << " : my_badge reference has to be set" << endl;
      exit(1);
    }
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << " getting my_badge description.." << endl;
    throw Domain_Repository::Destination_Unreachable("badge_repository");
  }
  
  // collect state from badge_rep
  try {
    if (!CORBA::is_nil(my_descr.current_badge)) {
      ABng_Core_System::Badge::Value_Description_var bvdesc=
	my_descr.current_badge->value_descr();
#ifdef ORBIX
      // Orbix bug workaround
      if (CORBA::is_nil(bvdesc->holder))
	bvdesc->holder= NULL;
#endif
      my_value_descr->current_badge = *(bvdesc.operator->());
    }
    else {
      cerr << endl << "Error - badge_holder item id = " << my_descr.item_id 
	   << " : current_badge reference has to be set" << endl;
      exit(1);
    }
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << " getting current_badge description" << endl;
    throw Domain_Repository::Destination_Unreachable("badge_repository");
  }
  
  // collect state from sensor_rep
  try {
    if (!CORBA::is_nil(my_descr.home_sensor)) {
      ABng_Core_System::Sensor::Value_Description_var svdesc=
	my_descr.home_sensor->value_descr();
      my_value_descr->home_sensor = *(svdesc.operator->());
    }
    else {
      my_value_descr->home_sensor.item_id = 0;
      my_value_descr->home_sensor.id.sensor_num = 0;
      my_value_descr->home_sensor.id.network_num = 0;
      my_value_descr->home_sensor.id.group_name = CORBA::string_dup("");
    }
  }
  catch (...) {
    throw Domain_Repository::Destination_Unreachable("sensor_repository");
  }
  
  
  // collect state from user_rep or equipment_rep
  if (my_descr.type_specific_object._d() == ABng_Core::LOCATABLE_USER) {
    try {
      if (!CORBA::is_nil(my_descr.type_specific_object.user_obj())) {
	Domain_Repository::User::Value_Description_var uvdesc=
	  my_descr.type_specific_object.user_obj()->value_descr();
	my_value_descr->type_specific_object.
	  user_description( *(uvdesc.operator->()));
      }
      else {
	cerr << endl << "Error - badge_holder item id = " << my_descr.item_id 
	     << " : user reference has to be set" << endl;
	exit(1);
      }
    }
    catch (...) {
      throw Domain_Repository::Destination_Unreachable("user_repository");
    }
  }
  else {
    try {
      cerr << "item_id " << my_value_descr->item_id << endl;
      if (!CORBA::is_nil(my_descr.type_specific_object.equipment_obj())) {
	Domain_Repository::Equipment::Value_Description_var evdesc=
	  my_descr.type_specific_object.equipment_obj()->value_descr();
	my_value_descr->type_specific_object.
	  equipment_description( *(evdesc.operator->()));
      }
      else {
	cerr << endl << "Error - badge_holder item id = " << my_descr.item_id 
	     << " : equipment reference has to be set" << endl;
	exit(1);
      }
    }
    catch (...) {	
      throw Domain_Repository::Destination_Unreachable("equipment_repository");
    }
  }
  
  return my_value_descr;
}

Domain_Repository::Repository_Item_Id 
   Badge_Holder_I::item_id (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	// no lock here: item_id is immutable
	
	return my_descr.item_id;
}

ABng_Core_System::Badge_ptr 
   Badge_Holder_I::my_badge (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	if (my_badge_id != 0 && CORBA::is_nil(my_descr.my_badge))
		throw Domain_Repository::Destination_Unreachable("badge_rep");

	// duplicate my_badge
	return ABng_Core_System::Badge::_duplicate(my_descr.my_badge);
}

ABng_Core_System::Badge_ptr 
   Badge_Holder_I::current_badge (
	ENV_DEF1(env)
  )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	if (current_badge_id != 0 && CORBA::is_nil(my_descr.current_badge))
		throw Domain_Repository::Destination_Unreachable("badge_rep");

	// duplicate current_badge
	return ABng_Core_System::Badge::_duplicate(my_descr.current_badge);
}

ABng_Core_System::Sensor_ptr 
   Badge_Holder_I::home_sensor (
	ENV_DEF1(env)
  )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	if (home_sensor_id != 0 && CORBA::is_nil(my_descr.home_sensor))
		throw Domain_Repository::Destination_Unreachable("sensor_rep");

	// duplicate home_sensor
	return ABng_Core_System::Sensor::_duplicate(my_descr.home_sensor);
}

ABng_Core::Locatable_Type_Specific_Object* 
   Badge_Holder_I::type_specific_object (
	ENV_DEF1(env)
  )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	if (badge_holder_kind == ABng_Core::LOCATABLE_USER) {
		if (type_specific_object_id!= 0 && 
		    CORBA::is_nil(my_descr.type_specific_object.user_obj())) 
			throw Domain_Repository::Destination_Unreachable("user_rep");

	}
	else {
		if (type_specific_object_id != 0 && 
		    CORBA::is_nil(my_descr.type_specific_object.equipment_obj()))
	 		throw Domain_Repository::Destination_Unreachable("equipment_rep");
	}

	// duplicate type_specific_object
        ABng_Core::Locatable_Type_Specific_Object* type_specific_object_duplicated =
		new ABng_Core::Locatable_Type_Specific_Object(my_descr.type_specific_object);

	return type_specific_object_duplicated;
}

#ifdef FilePersistency
//
//		Store and restore state methods
//		_________________


void
   Badge_Holder_I::save (
	ostream &iostr
   )
{
	iostr << my_descr.item_id        << ' ';
	iostr << my_badge_id             << ' ';
	iostr << current_badge_id        << ' ';
	iostr << home_sensor_id          << ' ';
	iostr << (int)badge_holder_kind  << ' ';
	iostr << type_specific_object_id << ' ';
	iostr << (int) my_descr.assigned  << ' ';
	iostr << (int) my_descr.visitor;
	iostr << endl;
}

void
   Badge_Holder_I::restore (
	istream &iostr
   )
{
	iostr >> my_descr.item_id;
	iostr >> my_badge_id;
	iostr >> current_badge_id;
	iostr >> home_sensor_id;
	int badge_holder_kind_p;
	iostr >> badge_holder_kind_p;
	badge_holder_kind = (ABng_Core::Locatable_Type)badge_holder_kind_p;
	iostr >> type_specific_object_id;
	int tmp;
	iostr >> tmp;
	my_descr.assigned= (CORBA::Boolean) tmp;
	iostr >> tmp;
	my_descr.visitor= (CORBA::Boolean) tmp;
  
	if (!iostr) {
		cerr << "Badge_Holder_I error: restoring from file";
		exit(1);
	}

#ifdef DEBUG
	cerr << "Badge_Holder_I: read badge_holder item_id = ";
	cerr << my_descr.item_id << endl;
#endif
}
#endif /* FilePersistency */




