//
//  File:         location_type_rep_i.cc
//  Author:       Malgorzata Steinder (gosia@ics.agh.edu.pl)
//  Date Created: 17.12.97
//
// General Description:
//   Implementation file for the Location_Type_Rep_I class
//

#define EXCEPTIONS

#include <rw/cstring.h>

#include "location_type_rep_i.h"

#include <rw/tvdlist.h>

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_Domain_Repository_Location_Type_Rep(Location_Type_Rep_I)
#else
DEF_TIE_Domain_Repository_Location_Type_Rep(Location_Type_Rep_I)
#endif

#ifdef ObjectStore
DEF_ObjectStoreTIE_Domain_Repository_Management_Location_Type_Rep_Manager(Location_Type_Rep_I)
#else
DEF_TIE_Domain_Repository_Management_Location_Type_Rep_Manager(Location_Type_Rep_I)
#endif
#elif defined OMNIORB
  // DEF_TIE__tie_Location_Type_Rep
  // DEF_TIE__tie_Location_Type_Rep_Manager
#endif

ostream& operator << (ostream& str, Domain_Repository::Location_Type::Description& descr);

void 
 Location_Type_Rep_I::set_rep(Domain_Repository::Location_Type_Rep* rep) {
   my_rep = rep;
}

//
// 		Constructor
//              ___________

const char * 
 Location_Type_Rep_I::ref_server_names[]= { "location_description_repository", 
                                           NULL
                                         };

  Location_Type_Rep_I::Location_Type_Rep_I (
#ifdef ObjectStore
    os_database* db_p
#elif defined FilePersistency
    const char *repository_name_p
#endif
  ) : Basic_Rep_I<LT_Types, LT_SDO_Types>(hash_location_type,
#ifdef ObjectStore
    db_p
#elif defined FilePersistency
    repository_name_p
#endif
 ) {
  _instance = this;
  create_corba_objects();
}

//
// 		Destructor
//		__________

Location_Type_Rep_I::~Location_Type_Rep_I() {

  destroy_corba_objects();
  delete ps_dev;
}

Location_Type_Rep_I* 
 Location_Type_Rep_I::_instance = 0;

Location_Type_Rep_I* 
 Location_Type_Rep_I::instance() {
   return _instance;
}

void Location_Type_Rep_I::init( int init_v) {
  Basic_Rep_I<LT_Types, LT_SDO_Types>::init( init_v);
  try {
    Bootstrap::register_server_in_NS(Bootstrap::SRV_Location_Type_Repository(),
				     my_rep);
    Bootstrap::register_server_in_NS(
         Bootstrap::SRV_Location_Type_Repository_Manager(),
	 my_rep_manager
    );
    register_callback( Bootstrap::SRV_Location_Description_Repository());
    ps_dev= new ProcessControlImpl( "location_type_repository", this);
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Caught exception while contacting server manager" << ex << endl;
    exit( 1);
  }
}

void 
 Location_Type_Rep_I::reference_resolved(const ABng_Server_Manager::Name &name,
				      CORBA::Object_var server_ref,
				      ABng_Server_Manager::uid callback_id) {
  auto_lock local_lock(*this);

  if ( name == Bootstrap::SRV_Location_Description_Repository()) {
    try {
     location_description_rep = 
        Domain_Repository::Location_Description_Rep::_narrow( server_ref);
    }
    catch (...) {}
  } 
}
//
// Private functions
//



int
Location_Type_Rep_I::remove_permitted(Location_Type_I * location_type_i_p) {
    
        Domain_Repository::Location_Descriptions *
         location_descriptions;
	Domain_Repository::Location_Description::Pattern pattern;
        pattern.is_any = 0;
        pattern.item_id_pattern.is_any = 1;
        pattern.type_name_pattern.is_any = 0;
        pattern.type_name_pattern.value = location_type_i_p->get_descr().name;
        pattern.property_list_pattern.is_any = 1;
        pattern.property_list_pattern.constraints = 
	  Domain_Repository::UNRESTRICTED;

        if(CORBA::is_nil((Domain_Repository::Location_Description_Rep_ptr)
                         location_description_rep
                        )
        ) return 1;

        try {
            location_descriptions = location_description_rep->find(pattern);
        } catch (CORBA::COMM_FAILURE &ex) {
            location_description_rep = 
                Domain_Repository::Location_Description_Rep::_nil();
            return 1;
	} catch(...) {
            return 1;
        }

        int result = 1;
        if(location_descriptions->length()) result = 0;

        delete location_descriptions;
     
        return result;
}
           
int Location_Type_Rep_I::validate_update_data(Item_ptr item_p, 
  const Domain_Repository::Location_Type::Description& data_record_pattern) {

  Basic_Rep_I<LT_Types, LT_SDO_Types>::validate_update_data(
    item_p, data_record_pattern);

  CORBA::String_var name = item_p->name();
  if(strcmp(name, data_record_pattern.name)) {
    throw Domain_Repository::Invalid_Description();
  }
  return 1;
}

unsigned 
 Location_Type_Rep_I::hash_location_type(
        const Location_Type_I& location_type_i_p
) {
	Domain_Repository::Location_Type::Description 
        location_type_descr = location_type_i_p.get_descr();

        RWCString rwcString(location_type_descr.name);

	return rwcString.hash();
}

void
 Location_Type_Rep_I::match_location_type(
	Location_Type_I *current_location_type,
	void* apply_data
) {
        // drp == data_record_pattern!!!!!
	Domain_Repository::Location_Type::Pattern 
        drp = ((struct MatchLocation_TypeApplyDataStruct*)apply_data)
                     ->_data_record_pattern;

        //
	// check general patten matching rule
        //
	if (drp.is_any){
	     //
	     // any location_type matches this pattern
	     //
	     gather_item (
                 current_location_type, 
		 ((struct MatchLocation_TypeApplyDataStruct*)apply_data)
                       ->_found_location_types
             );
	     return;
	}

        //
	// check matching of item_id
        //
	if (!drp.item_id_pattern.is_any &&
	    (drp.item_id_pattern.value 
               != current_location_type->get_descr().item_id
            )
        )
	   // this location_type item_id does not match pattern 
	   return;

        //
	// check matching of name
        //
	if (!drp.name_pattern.is_any
            && strcmp(drp.name_pattern.value,
                      current_location_type->get_descr().name
               )
           ) 
	   // does not match
           return;

        //
        // check matching of property_description_list
        //
        if(!drp.property_description_list_pattern.is_any) {

	  Domain_Repository::Property_Description_Patterns * 
          pdp = & drp.property_description_list_pattern.value;
          const Domain_Repository::Property_Descriptions * 
          pd = & current_location_type->get_descr().property_description_list;

          if(drp.property_description_list_pattern.constraints == 
                Domain_Repository::RESTRICTED_ORDERING_AND_NUMBER
            || drp.property_description_list_pattern.constraints == 
                Domain_Repository::RESTRICTED_NUMBER
          ) if(pdp->length() != pd->length()) {
               cerr << "PDP and PD lengths differ!!!" << endl;
               return;
            }

          
          if(drp.property_description_list_pattern.constraints == 
                Domain_Repository::RESTRICTED_ORDERING_AND_NUMBER
            || drp.property_description_list_pattern.constraints == 
                Domain_Repository::RESTRICTED_ORDERING
	  ) {
            for(int i = 0; i < pdp->length(); i++) {
	       if(!(*pdp)[i].is_any) {

                  if(!(*pdp)[i].name_pattern.is_any && 
                     strcmp((*pdp)[i].name_pattern.value, (*pd)[i].name)
                  )
		    // this name does not match
                    {  
                         cerr << "The name of the " << i 
                              << "th pattern does not match" << endl;
                         return;
                  }

                  if(!(*pdp)[i].type_pattern.is_any && 
                     !(*pdp)[i].type_pattern.value->equal((*pd)[i].type)
                    )
		    // this type does not match
                    {                 
                         cerr << "The type of the " << i 
                              << "th pattern does not match" << endl;
                         return;
                  }
               }
	    }
	  }
          else {
            if(drp.property_description_list_pattern.constraints == 
               Domain_Repository::RESTRICTED_NUMBER) {
               int *matched_pd = new int[pd->length()];
               for(int i = 0; i < pd->length(); i ++) matched_pd[i] = 0;

               for(i = 0; i < pdp->length(); i++) {
	          if(!(*pdp)[i].is_any) {
		     for(int j = 0; j < pd->length(); j ++) {
                       if(!(*pdp)[i].name_pattern.is_any && 
                          strcmp((*pdp)[i].name_pattern.value, (*pd)[j].name)
                       )
		       // this name does not match
                       continue;

                       if(!(*pdp)[i].type_pattern.is_any && 
                          !(*pdp)[i].type_pattern.value->equal((*pd)[j].type)
                       )
		       // this type does not match
                       continue;

                       if(!matched_pd[j]) {
                          matched_pd[j] = 1;
                          break;
                       }
                    }
                    if(j == pd->length())
                    // None of the descriptions matches
                    { 
                        cerr << "None of the descriptions matches pattern " 
                             << i << endl;
                        delete matched_pd;
                        return;
                    }
                 }
               }
               delete matched_pd;
            } else {
               for(int i = 0; i < pdp->length(); i++) {
	          if(!(*pdp)[i].is_any) {
		     for(int j = 0; j < pd->length(); j ++) {
                        if(!(*pdp)[i].name_pattern.is_any && 
                           strcmp((*pdp)[i].name_pattern.value, (*pd)[j].name)
                        )
		        // this name does not match
                        continue;

                        if(!(*pdp)[i].type_pattern.is_any && 
                           !(*pdp)[i].type_pattern.value->equal((*pd)[j].type)
                        )
		        // this type does not match
                        continue;
                        break;
                     }
                     if(j == pd->length()) { 
                        cerr << "None of the descriptions matches pattern " 
                             << i << endl;
                        return;
                     }
                  }
               }
            }
          }
        }

        //
	// this location_type matches the pattern
        //
	gather_item(
                current_location_type, 
		((struct MatchLocation_TypeApplyDataStruct*)apply_data)
                          ->_found_location_types
        );
}



//
// Implementation of IDL operations of the Location_Type_Rep interface
//                                         __________

Domain_Repository::Location_Type_Rep *
 Location_Type_Rep_I::rep( ENV_DEF1(env)) 
       throw (CORBA::SystemException) {
       return Domain_Repository::Location_Type_Rep::_duplicate(my_rep);
}

Domain_Repository::Location_Types* 
   Location_Type_Rep_I::private_find (
        const Domain_Repository::Location_Type::Pattern& data_record_pattern
        ENV_DEF(env)
   )
        throw (CORBA::SystemException,
               Domain_Repository::Search_Not_Implemented
        ) {

        // create a structure consisting of a pointer to the container
        //  and a reference of the pattern
        MatchLocation_TypeApplyDataStruct 
         apply_data(new Domain_Repository::Location_Types(
									     //my_location_type_list.entries()
                            my_item_list.entries()
                        ),
                    data_record_pattern
         );

        //my_location_type_list.apply(match_location_type, (void*)&apply_data);
        my_item_list.apply(match_location_type, (void*)&apply_data);

        // pointer to found_location_types was stored inside apply_data 
        // thus return it
        return apply_data._found_location_types;
}

void Location_Type_Rep_I::collect_values(Location_Type_I *lt, 
  Domain_Repository::Location_Type::Value_Description& ld) {

  ld = lt->get_descr();
}

void Location_Type_Rep_I::init_corba_object(Location_Type_I *item_p) {

  // create the TIE (CORBA) object for the C++ object
#ifdef ORBIX
#ifdef ObjectStore
  ObjectStoreTIE_Domain_Repository_Location_Type(Location_Type_I)* 
    new_location_type_tie =
    new(my_db, ObjectStoreTIE_Domain_Repository_Location_Type(
      Location_Type_I)::get_os_typespec()) 
      ObjectStoreTIE_Domain_Repository_Location_Type(
        Location_Type_I
	) (item_p);
#else
  TIE_Domain_Repository_Location_Type(Location_Type_I) *
    new_location_type_tie = 
    new TIE_Domain_Repository_Location_Type(Location_Type_I) (item_p);
#endif
#elif defined OMNIORB
  Domain_Repository::Location_Type_ptr new_location_type_tie=
    new TIE_CLASS(Domain_Repository,Location_Type)<Location_Type_I,0>(item_p);
  CONNECT_IMPL( Location_Type::Location_Type_var, new_location_type_tie);
  
#endif
  item_p->set_corba_object(new_location_type_tie);
}

DB_Header * Location_Type_Rep_I::create_db_header() {

  return new DB_Header( "Location_Type Repository", "1.0", "\
#       <file>::=       <next_item_id>\n\
#                       <number_of_items>\n\
#                       <data_records>\n\
#\n\
#       <data_records>::=  | <data_record> <data_records>\n\
#\n\
#       <data_record>::=        <repository_item_id>\n\
#                               <location_type_name>\n\
#                               <prop_descriptions>\n\
#\n\
#       <prop_descriptions>::=  \"<\" <seq_maximum> <seq_length>\n\
#                                   <prop_descr_buffer> \">\"\n\
#\n\
#       <prop_descr_buffer>::=  | <property_descr> <prop_descr_buffer>\n\
#\n\
#       <property_descr>::= \"<\" <location_type_name> <type_code> \">\"\n\
#\n\
#       <location_type_name>::=   <string>\n\
#\n\
#       <type_code>::=     <string>\n\
#\n\
#       <string>::=    \"<\" <string_length> <string_val> \">\"\n\
#\n\
#\n");
    }

void Location_Type_Rep_I::construct_data_object(
    Domain_Repository::Static_Data_Observer::Static_Data_Object& 
      data_object_p,
    Location_Type_ptr item_p) {


  // construct union to store reference of added entity
  data_object_p.location_type_obj(
    Domain_Repository::Location_Type::_duplicate(item_p));
}

void Location_Type_Rep_I::create_corba_objects() {

#ifdef ORBIX
#ifdef FilePersistency
  // create a TIE for the object
  my_rep_manager = 
    new TIE_Domain_Repository_Management_Location_Type_Rep_Manager(
      Location_Type_Rep_I) (this);
  my_rep = 
    new TIE_Domain_Repository_Location_Type_Rep(Location_Type_Rep_I) (this);
#endif /* FilePersistency */
#elif defined OMNIORB
  my_rep_manager =
    new TIE_CLASS(Domain_Repository_Management,Location_Type_Rep_Manager)<Location_Type_Rep_I,0>(this);
  CONNECT_IMPL( Domain_Repository_Management::Location_Type_Rep_Manager_var,my_rep_manager);
  my_rep =
    new TIE_CLASS(Domain_Repository,Location_Type_Rep)<Location_Type_Rep_I,0>(this);
  CONNECT_IMPL( Domain_Repository::Location_Type_Rep_var,my_rep);
#endif

  write_ior(my_rep_manager, 
    "./Location_Type_Rep_Manager_iiop.ref");
  write_ior(my_rep, 
    "./Location_Type_Rep_iiop.ref");

}

void Location_Type_Rep_I::destroy_corba_objects() {

  CORBA::release(my_rep_manager);
  CORBA::release(my_rep);
}


