//
//  File:         meta_equipment_i.h
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 08.04.98
//
// General Description:
//   Header file for the implementation class of the Meta_Equipment interface
//

#ifndef meta_equipment_i_h
#define meta_equipment_i_h

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <errno.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "domain_rep_management.hh"
#include "abng_guarded.hh"
#include "abng_string.h"
#include "compatibility.hh"

class Meta_Equipment_I
{
 private:
  // state of the IDL Meta_Equipment object
  Domain_Repository::Meta_Equipment::Description my_descr;

  // reference of CORBA object encapsulated this C++ object
  Domain_Repository::Meta_Equipment_ptr my_corba_object;

  // state lock
  ABng_Guarded &my_guard;

 public:
  // ctor
   Meta_Equipment_I (
	const Domain_Repository::Meta_Equipment::Description& descr_init
   );

#ifdef FilePersistency
   Meta_Equipment_I (
	istream &iostr
   );
#endif

  virtual 
   ~Meta_Equipment_I ();

   operator==(	
	const Meta_Equipment_I&
   );

  // methods enabling access to the state

  void
   set_descr (
	const Domain_Repository::Meta_Equipment::Description& descr_update
   );
 
  const Domain_Repository::Meta_Equipment::Description&
   get_descr () const;

  void
   set_corba_object (
	const Domain_Repository::Meta_Equipment_ptr corba_object_p
   );

  const Domain_Repository::Meta_Equipment_ptr
   get_corba_object ();

  void 
   set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   );

  //
  // IDL operations of the Static_Data_Observed_E interface
  //                       ______________________

  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
                
  virtual void 
   attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs  
	ENV_DECL(env)
   )
		throw (CORBA::SystemException);

  virtual void 
   detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer);

  //
  // IDL operations of the Meta_Equipment interface
  //                       ______________
 
  virtual Domain_Repository::Meta_Equipment::Description* 
   descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
 
  virtual Domain_Repository::Meta_Equipment::Value_Description* 
   value_descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual Domain_Repository::Repository_Item_Id 
   item_id (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   type (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   vendor (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   model (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual Domain_Repository::Meta_Equipment::Dimensions
   my_dimensions (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual Domain_Repository::Meta_Equipment::Control_Info* 
   my_control_info (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual Domain_Repository::Property_Descriptions*
    specific_property_description_list ( 
	ENV_DECL1(env)
    ) 
    throw (CORBA::SystemException);

  virtual Domain_Repository::Meta_Equipment::Image* 
   my_icon (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

#ifdef ObjectStore
  //
  // method for ObjectStore
  //

  static os_typespec* get_os_typespec();

#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void
   save (
	ostream &iostr
   );

  virtual void
   restore (
	istream &iostr
   );
#endif
};   

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_Domain_Repository_Meta_Equipment(Meta_Equipment_I)
#else
DEF_TIE_Domain_Repository_Meta_Equipment(Meta_Equipment_I)
#endif
#endif


// implementation of this class need method "instance" of class Meta_Equipment_Rep_I
#include "meta_equipment_rep_i.h"

#endif /* meta_equipment_i_h */
