// $Id: sched_main.cc,v 1.6 1998/07/29 13:49:00 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/scheduler/sched_main.cc,v $
//
// $Log: sched_main.cc,v $
// Revision 1.6  1998/07/29 13:49:00  mencnar
// "Server_Status" implementation
//
// Revision 1.5  1998/06/18 10:21:56  mencnar
// moved to CVS
//
// Revision 1.4  1997/11/04 13:19:01  mencnar
// Sighting timeout from command line
//
// Revision 1.3  1997/10/31 14:25:11  mencnar
// Added comments
// Fixed memory leak in request queue
//
// Revision 1.2  1997/10/24 08:00:53  mencnar
// User identification using Orbix principal
//
// Revision 1.1  1997/10/23 09:17:33  mencnar
// Initial revision
//
//

#include "sched_main.hh"
#include <iostream.h>
#include "sched_debug.hh"
#include "sched_stat.hh"
#include "sched_repos.hh"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef ORBIX
DEF_TIE_ABng_Poller_AB_Observer(SchedulerImpl)
#elif OMNIORB
  // DEF_TIE__tie_AB_Observer
#endif
 


ABng_Core::Time Sched_Barrier::sighting_timeout;

int operator == ( const ABng_Poller::AB_Poller::Badge_Id &first,
		  const ABng_Poller::AB_Poller::Badge_Id &second) {
  if (first.length() != second.length()) return 0;
  int size= first.length();
  for (int i=0; i<size; i++)
    if (first[i] != second[i]) return 0;
  return 1;
}

int operator == ( const ABng_Core_System::Sensor_Id &first,
		  const ABng_Core_System::Sensor_Id &second) {
  return (first.sensor_num == second.sensor_num &&
	  first.network_num == second.network_num &&
	  (!strcmp( (const char *) first.group_name, (const char *) second.group_name)));
}

ABng_Core::Time Time_Stamp2Time( const Time_Stamp &ts) {
  ABng_Core::Time ret;
  ret.seconds= ts.sec;
  ret.microseconds= ts.usec;
  return ret;
}

//-------------------------------------------------------Sched_Request_Queue
//
Scheduler_Requests* Sched_Request_Queue::list_user_requests( RWCString user_name) {
  auto_lock local_lock( *this);

  int allsize= entries();
  int size= 0;

  for (int i=0; i<allsize; i++) {
    if (user_name == (const char *)((*this)[i]->user_name)) size++;
  }

  Scheduler_Requests *reqlist= new Scheduler_Requests(size);
  reqlist->length( size);

  for (int j=0; j<size; j++)
    if (user_name == (const char *)((*this)[j]->user_name)) 
      (*this)[j]->describe_request(  (*reqlist)[j]);
  return reqlist;
}

Scheduler_Requests* Sched_Request_Queue::list_all_requests() {
  auto_lock local_lock( *this);

  int size= entries();
  
  Scheduler_Requests *reqlist= new Scheduler_Requests(size);
  reqlist->length( size);

  for (int i=0; i<size; i++)
    (*this)[i]->describe_request( (*reqlist)[i]);
#ifdef DEBUG
  cout << dec << size << " requests in queue" << endl;
  cout << "Subrequest counter " << sched_stat.get_subrequest_counter() << endl
       << "Sent commands counter " << sched_stat.get_command_sent_counter() << endl;
  cout << "Reply counter " << sched_stat.get_reply_counter() << endl;
  cout << "Sighting errors counter " << sched_stat.get_sighting_errors_counter() << endl;
  cout << "Reply errors counter " << sched_stat.get_reply_errors_counter() << endl;
#endif
  return reqlist;
}


void Sched_Request_Queue::remove_request( CORBA::Long request_id) {
  auto_lock local_lock( *this);

  int size= entries();

  for (int i=0; i<size; i++)
    if ((*this)[i]->req_id == request_id) {
      Sched_Complex_Request *creq= (*this)[i];
      creq->cancel();
      removeAt(i);
      delete creq;
      return;
    }
  throw ABng_Core::Unknown_Request_Id();
}

Sched_Request_Queue::~Sched_Request_Queue() {
#ifdef DEBUG
  cerr << "~Sched_Request_Queue" << endl;
#endif
  auto_lock local_lock( *this);

  while (!isEmpty()) {
    first()->cancel();
    delete removeFirst();
  }
}

void Sched_Request_Queue::static_worker( void *arg) {
  Clean_Params &clpar= *(Clean_Params*)arg;

  try {
    clpar.queue.remove_request( clpar.request_id);
  }
  catch (...) {}
  delete &clpar;
}

void Sched_Request_Queue::clean( Sched_Complex_Request &req) {
  Clean_Params *clpar= new Clean_Params( *this, req.req_id);
  omni_thread *cleaner= new omni_thread( static_worker, clpar);
  cleaner->start();
}

#ifdef ORBIX
//-------------------------------------------------------Sched_Thread_Filter
//
class Sched_Thread_Filter : public CORBA::ThreadFilter {
  virtual int inRequestPreMarshal(CORBA::Request &, CORBA::Environment&);
};

static void startPerRequestThread( void *vp) {
  CORBA::Orbix.continueThreadDispatch( *(CORBA::Request *) vp);
}

int Sched_Thread_Filter::inRequestPreMarshal( CORBA::Request &creq,
					      CORBA::Environment &env) {
  omni_thread *thr;
  thr= new omni_thread( startPerRequestThread, (void*) &creq);
  thr->start();
  return -1;
}

// Sched_Thread_Filter my_thread_filter;  

#endif // ORBIX

//-------------------------------------------------------Sched_Location_List
//
//_________________________________Sched_Location_List::~Sched_Location_List
Sched_Location_List::~Sched_Location_List() {
#ifdef DEBUG
  cerr << "~Sched_Location_List" << endl;
#endif
  barrierid2barrier.applyToKeyAndValue( destroy_element, NULL);
}

void Sched_Location_List::destroy_element( Sched_Barrier_Id *,
					   Sched_Barrier * &bar,
					   void *) {
  delete bar;
}

//________________________Sched_Location_List::update_location_list_____
void Sched_Location_List::update_location_list(
	const Sched_Barrier_Id &bar) {

  auto_lock local_lock( *this);

  Sched_Barrier_Id badge_id( bar.badge_id);
  Sched_Barrier_Id newpos( bar.network_id, bar.station_id, bar.group_name);

  _get_barrier( badge_id).sighting_arrived( newpos);
}

//______________________________________Sched_Location_List::_get_barrier_____
Sched_Barrier & Sched_Location_List::_get_barrier( const Sched_Barrier_Id &loc) {

  Sched_Barrier *ret= barrierid2barrier.findValue( & loc);
  if (!ret) {
#ifdef DEBUG
    cout << "Creating new barrier for " << loc.badge_id << endl;
#endif
    ret= new Sched_Barrier( *this, loc);
    barrierid2barrier[ & ret->barrier_id]= ret;
  }
  return *ret;
}

//_____________________________________Sched_Location_List::get_barrier______
Sched_Barrier & Sched_Location_List::get_barrier( const Sched_Barrier_Id &loc) {
  auto_lock local_lock( *this);

  return _get_barrier( loc);
}

//_________________________________Sched_Location_List::handle_reply_____
void Sched_Location_List::handle_reply( Sched_Reply *reply) {
  auto_lock local_lock( *this);

  Sched_Barrier *schbarrier= barrierid2barrier.findValue( &(reply->min_barrier_id()));
  if (!schbarrier) {
    delete reply;
    return;
  } 
  schbarrier->reply_arrived( reply);
}

//-------------------------------------------------------------SchedulerImpl
//

SchedulerImpl::SchedulerImpl
          ( const char *object_name) :
#ifdef ORBIX
	ABng_Core_System::SchedulerBOAImpl( object_name),
#endif
	sb_repository( *this),
        my_req_id( 0) { 

#ifdef ORBIX
  poller_observer_obj= new TIE_ABng_Poller_AB_Observer(SchedulerImpl) (this);
  CONNECT_IMPL(ABng_Poller::AB_Observer_var,poller_observer_obj);
#else
  poller_observer_obj= new TIE_CLASS(ABng_Poller,AB_Observer)<SchedulerImpl,0>(this);
  CONNECT_IMPL(ABng_Poller::AB_Observer_var,poller_observer_obj);
#endif
  CONNECT_IMPL(ABng_Core_System::Scheduler_var,this);

  Bootstrap::synch_time( 20000); // 20 milliseconds should be enough
  Bootstrap::need_synch_time( 20000);

  ps_dev= new ProcessControlImpl( "scheduler", this);
}

void SchedulerImpl::detach( Poller_Item *pit, void *arg) {
  try {
    if (! CORBA::is_nil( pit->poller_ref.operator->())) 
      pit->poller_ref->detach_observer( ((SchedulerImpl*) arg)->poller_observer_obj);
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Caught exception " << ex
	 << "  -- detach_observer" << endl;    
  }
}

SchedulerImpl::~SchedulerImpl() {
  {
    auto_lock local_lock( *this);

    pollers.apply( detach, this);
  }
  delete ps_dev;
}

void SchedulerImpl::reference_resolved( const ABng_Server_Manager::Name &server_name,
					CORBA::Object_var server_ref,
					ABng_Server_Manager::uid callback_id) {
  try {
    if ( server_name >= Bootstrap::CTX_ABng_Core_System_Pollers()) {
#ifdef DEBUG
      cerr << "Poller resolved..narrow" << endl;
      //      cerr << server_ref->_object_to_string() << endl;
      if (CORBA::is_nil(server_ref)) cerr << "server is nil" << endl;
#endif
      ABng_Server_Manager::Name sname= 
	server_name - Bootstrap::CTX_ABng_Core_System_Pollers();
      if ( sname.length() != 1) return;

      auto_lock local_lock( *this);
      ABng_Poller::AB_Poller_var poller_ref;
#ifdef OMNIORB
      try {
	cerr << "Oli narrow" << endl;
	poller_ref= ABng_Poller::Oli_AB_Poller::_narrow( server_ref);
      }
      catch (...) {}
      if (CORBA::is_nil( poller_ref.operator->()))
#endif
	{
	  cerr << "AB_Poller narrow" << endl;
	  poller_ref= ABng_Poller::AB_Poller::_narrow( server_ref);
	}
      if ( CORBA::is_nil( poller_ref.operator->())) return;

#ifdef DEBUG
      cerr << "Narrow done" << endl;
#endif
      Poller_Item tmpitem( RWCString( (const char *) sname[ 0]));

      if (!pollers.contains( &tmpitem))
	pollers.insert( new Poller_Item( tmpitem));

      pollers.find( &tmpitem)->poller_ref= poller_ref;

      //      poller_ref->detach_observer( poller_observer_obj);
      // prevent multiple attached observers to the same poller
      //

#ifdef DEBUG
      cerr << "attach observer" << endl;
#endif
      poller_ref->attach_observer( poller_observer_obj);
#ifdef DEBUG
      cerr << "attach observer succeeded" << endl;
#endif
    }
    else if ( server_name == Bootstrap::SRV_Sensor_Repository()) {
      sb_repository.set_sensor_rep( 
          ABng_Core_Management::Sensor_Rep::_narrow( server_ref));
    }
    else if ( server_name == Bootstrap::SRV_Badge_Repository()) {
      sb_repository.set_badge_rep(
	  ABng_Core_Management::Badge_Rep::_narrow( server_ref));
    }
  }
  catch ( CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << ex << endl;
#endif
  }
}

ABng_Poller::AB_Poller_var SchedulerImpl::poller( const Sched_Barrier_Id &loc) {
  auto_lock local_lock( *this);
  Poller_Item *pit;
  Poller_Item tmpitem( RWCString( (const char *) loc.group_name));

  pit= pollers.find( &tmpitem);
  if (!pit) throw Domain_Repository::Destination_Unreachable("poller");
  return pit->poller_ref;
}
  
ABng_Core_System::Badge_Ids SchedulerImpl::get_all_badge_ids() {
  return sb_repository.get_all_badge_ids();
}

ABng_Core_System::Sensor_Ids SchedulerImpl::get_all_sensor_ids() {
  return sb_repository.get_all_sensor_ids();
}

CORBA::Long SchedulerImpl::create_request_id() {
  auto_lock local_lock( *this);

  return my_req_id++;
}

MY_PRINCIPAL_VAR SchedulerImpl::get_principal( ENV_DEF1( env)) {
#ifdef ORBIX
  if (env.m_request) 
       return CORBA::Principal_var( CORBA::string_dupl(env.m_request->principal()));
  else return CORBA::Principal_var( CORBA::Orbix.get_principal());
#elif defined OMNIORB
  MY_PRINCIPAL_VAR ret;
  return ret;
#endif
}

void SchedulerImpl::receive (
	const char * network_group_name, 
	const ABng_Poller::AB_Poller::Message& msg
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::receive called" << endl;
#endif

  Sched_Reply *reply;

  if (msg.length()>=3) {
    if (msg[1] & 0x20)
         reply= new Sched_Reply_Sensor_unknown( msg, network_group_name);
    else reply= new Sched_Reply_Badge_unknown( msg, network_group_name);
  
    location_rep.handle_reply( reply);
  }
}


void SchedulerImpl::sighting (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	ABng_Poller::AB_Poller::Sighting_Code code, 
	const ABng_Poller::AB_Poller::Field_Id& field, 
	ABng_Poller::AB_Poller::Byte click_num,
	ABng_Poller::AB_Poller::Byte attributes, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << hex;
  cout << "sighting called ( " << network_group_name
       << ", net: " << (int) network << ", sta: " << (int) station 
       << ", " << badge << ")" << endl;
#endif
  Sched_Barrier_Id pos( badge, network, station, network_group_name);

  try {
    sb_repository.check_valid( (ABng_Core_System::Sensor_Id) pos);
  }
  catch (...) {
    return;
  }

  location_rep.update_location_list( pos);
}

void SchedulerImpl::challenge_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	const ABng_Poller::AB_Poller::Octets& random, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::challenge_reply called" << endl;
#endif

  Sched_Reply_challenge *reply=
    new Sched_Reply_challenge( Time_Stamp2Time( ts),
			       Sched_Barrier_Id( badge, 
						 (Network_Id) network, 
						 (Station_Id) station,
						 network_group_name),
			       seq,
			       random);
  location_rep.handle_reply( reply);
}

void SchedulerImpl::sound_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	const Tones& sound, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::sound_reply called" << endl;  
#endif

  Sched_Reply_sound *reply=
    new Sched_Reply_sound( Time_Stamp2Time( ts),
			   Sched_Barrier_Id( badge, 
					     (Network_Id) network, 
					     (Station_Id) station,
					     network_group_name),
			   seq,
			   sound);
  location_rep.handle_reply( reply);
}

void SchedulerImpl::set_state_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	ABng_Poller::AB_Poller::Byte status, 
	const Tones& sound, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::set_state_reply called" << endl;
#endif

  Sched_Reply_set_state *reply=
    new Sched_Reply_set_state( Time_Stamp2Time( ts),
			       Sched_Barrier_Id( badge,
						 (Network_Id) network,
						 (Station_Id) station,
						 network_group_name),
			       seq,
			       sound,
			       status);
  location_rep.handle_reply( reply);
}

void SchedulerImpl::get_state_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	ABng_Poller::AB_Poller::Byte major_version, 
	ABng_Poller::AB_Poller::Byte minor_version, 
	ABng_Poller::AB_Poller::Byte status, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::get_state_reply called" << endl;
#endif

  Sched_Reply_get_state *reply=
    new Sched_Reply_get_state( Time_Stamp2Time( ts),
			       Sched_Barrier_Id( badge,
						 (Network_Id) network,
						 (Station_Id) station,
						 network_group_name),
			       seq,
			       major_version,
			       minor_version,
			       status);
  location_rep.handle_reply( reply);
}

void SchedulerImpl::set_home_reply (
	const char * network_group_name,
	Network_Id network,
	Station_Id station,
	const Badge_Id& badge,
	const ABng_Poller::AB_Poller::Address& address,
	Seq_Number seq,
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::set_home_reply called" << endl;
#endif

  Sched_Reply_set_home *reply=
    new Sched_Reply_set_home( Time_Stamp2Time( ts),
			      Sched_Barrier_Id( badge,
						(Network_Id) network,
						(Station_Id) station,
						network_group_name),
			      seq,
			      address);
  location_rep.handle_reply( reply);
}

void SchedulerImpl::get_home_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	const ABng_Poller::AB_Poller::Address& address, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::get_home_reply called" << endl;
#endif

  Sched_Reply_get_home *reply=
    new Sched_Reply_get_home( Time_Stamp2Time( ts),
			      Sched_Barrier_Id( badge,
						(Network_Id) network,
						(Station_Id) station,
						network_group_name),
			      seq,
			      address);
  location_rep.handle_reply( reply);
}


void SchedulerImpl::get_station_state_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	ABng_Poller::AB_Poller::Byte major_version, 
	ABng_Poller::AB_Poller::Byte minor_version, 
	ABng_Poller::AB_Poller::Byte status1, 
	ABng_Poller::AB_Poller::Byte status2, 
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::get_station_state_reply called" << endl;
  cout << "\tsta: " << (int) station << " net: " << (int) network
       << " group: " << network_group_name << endl;
#endif

  Sched_Reply_get_station_state *reply=
    new Sched_Reply_get_station_state( Time_Stamp2Time( ts),
				       Sched_Barrier_Id( (Network_Id) network,
							 (Station_Id) station,
							 network_group_name),
				       major_version,
				       minor_version,
				       status1,
				       status2);
  location_rep.handle_reply( reply);
}

void SchedulerImpl::null_message (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Time_Stamp& ts
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::null_message called" << endl;
#endif
}

Scheduler_Requests* SchedulerImpl::list_user_requests (
	const char * user_ids
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::list_user_request" << endl;
#endif

  return request_queue.list_user_requests( user_ids);
}

Scheduler_Requests* SchedulerImpl::list_all_requests (
	ENV_DEF1(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::list_all_requests" << endl;
#endif

  return request_queue.list_all_requests();
}

void SchedulerImpl::remove_request (
	CORBA::Long request_id
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::remove_request" << endl;
#endif
  request_queue.remove_request( request_id);
}

CORBA::Long SchedulerImpl::issue_sensor_command (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Octets& cmd, 
	CORBA::Boolean has_reply,
	CORBA::Octet reply_code,
	const ABng_Core::Command_Context& ctx, 	
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::issue_sensor_command" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif


  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));

  request_queue.append( scr= new Sched_Sensor_Request_issue( *this,
							    ctx,
							    caller,
							    get_principal(USE_ENV1(env)),
							    sensors,
							    cmd,
							    has_reply,
							    reply_code));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::issue_badge_command (
	const ABng_Core_System::Sensor_Ids& sensors,
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core_System::Octets& cmd, 
	CORBA::Boolean has_reply,
	CORBA::Octet reply_code,
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::issue_badge_command" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));

  request_queue.append( scr= new Sched_Badge_Request_issue( *this,
							    ctx,
							    caller,
							    get_principal(USE_ENV1(env)),
							    badges,
							    sensors,
							    cmd,
							    has_reply,
							    reply_code));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::challenge (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core_System::Octets& random, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::challenge called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));

  request_queue.append( scr= new Sched_Request_challenge( *this,
							  ctx,
							  caller,
							  get_principal(USE_ENV1(env)),
							  badges,
							  sensors,
							  random));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::sound (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core_System::Octets& sound, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cerr << "SchedulerImpl::sound called" << endl;
  for (int i=0; i<badges.length(); i++)
    cerr << (const ABng_Poller::AB_Poller::Octets&) badges[i] << endl;
  for (int j=0; j<sensors.length(); j++)
    cerr << sensors[j] << endl;
  cerr << ctx << endl;
#ifdef ORBIX
  cerr << "reply_to refCount " << reply_to->_refCount() << endl;
  cerr << "reply_to is_nil " << (int) CORBA::is_nil( reply_to) << endl;
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);
  
  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));


  request_queue.append( scr= new Sched_Request_sound( *this,
						      ctx,
						      caller,
						      get_principal(USE_ENV1(env)),
						      badges,
						      sensors,
						      sound));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::set_state (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	CORBA::Octet status, 
	const ABng_Core_System::Octets& sound, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::set_state called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif
  
  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));

  request_queue.append( scr= new Sched_Request_set_state( *this,
							  ctx,
							  caller,
							  get_principal(USE_ENV1(env)),
							  badges,
							  sensors,
							  status,
							  sound));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::get_state (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::get_state called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));

  request_queue.append( scr= new Sched_Request_get_state( *this,
							  ctx,
							  caller,
							  get_principal(USE_ENV1(env)),
							  badges,
							  sensors));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::set_home (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core_System::Octets& address, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::set_home called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));

  request_queue.append( scr= new Sched_Request_set_home( *this,
							 ctx,
							 caller,
							 get_principal(USE_ENV1(env)),
							 badges,
							 sensors,
							 address));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::get_home (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::get_home called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));

  request_queue.append( scr= new Sched_Request_get_home( *this,
							 ctx,
							 caller,
							 get_principal(USE_ENV1(env)),
							 badges,
							 sensors));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::reset_buffer (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::reset_buffer called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));
  
  request_queue.append( scr= new Sched_Request_reset_buffer( *this,
							     ctx,
							     caller,
							     get_principal(USE_ENV1(env)),
							     sensors));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::reset_fifos (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::reset_fifos called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));
  
  request_queue.append( scr= new Sched_Request_reset_fifos( *this,
							    ctx,
							    caller,
							    get_principal(USE_ENV1(env)),
							    sensors));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::get_station_state (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::get_station_state called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));

  request_queue.append( scr= new Sched_Request_get_station_state( *this,
								  ctx,
								  caller,
								  get_principal(USE_ENV1(env)),
								  sensors));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

CORBA::Long SchedulerImpl::set_output (
	const ABng_Core_System::Sensor_Ids& sensors, 
	CORBA::Octet output_value, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::set_output called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));
  
  request_queue.append( scr= new Sched_Request_set_output( *this,
							   ctx,
							   caller,
							   get_principal(USE_ENV1(env)),
							   sensors,
							   output_value));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;

}

CORBA::Long SchedulerImpl::set_persistence (
	const ABng_Core_System::Sensor_Ids& sensors,
	CORBA::Boolean mode, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)) {
#ifdef DEBUG
  cout << "SchedulerImpl::set_persistence called" << endl;
#endif
#ifdef ORBIX
  if (CORBA::is_nil(reply_to)) reply_to= NULL;
#endif

  Sched_Guarded:auto_lock local_lock( request_queue);

  Sched_Complex_Request *scr;
  ABng_Core_System::Scheduler_Caller_var caller(
     ABng_Core_System::Scheduler_Caller::_duplicate( reply_to));
  
  request_queue.append( scr= new Sched_Request_set_persistence( *this,
								ctx,
								caller,
								get_principal(USE_ENV1(env)),
								sensors,
								mode));
  scr->start();
  CORBA::Long ret= scr->req_id;
  return ret;
}

void initialize_sighting_timeout( int seconds=9) {
  Sched_Barrier::sighting_timeout.seconds= seconds;
  Sched_Barrier::sighting_timeout.microseconds= 0;
}

void usage( const char *prog_name) {
  cout << "Usage: " << prog_name << endl
       << "\t\t[-g <sighting_timeout>]" << endl;
#ifdef ORBIX
  cout << "\t\t[-t <implementation_timeout>]" << endl;
#endif
  exit(1);
}
  

int main( int argc, char **argv){
  const char *prog_name= *argv;

  CORBA::ORB_ptr orb_ptr= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  CORBA::BOA_ptr boa_ptr= orb_ptr->BOA_init( argc, argv, BOA_IDENTIFIER);

#ifdef ORBIX
  orb_ptr->setDiagnostics( 0);
  int implementation_timeout= CORBA::ORB::INFINITE_TIMEOUT;
#endif
  
  char c;
  while ((c = getopt(argc, argv, "g:t:h")) != EOF)
    switch (c) {
    case 'g':
      initialize_sighting_timeout( atoi( optarg));;
      break;
#ifdef ORBIX
    case 't':
      implementation_timeout= atoi( optarg);
      break;
#endif
    default:
      usage( *argv);
      break;
    }

  if (argc<1 || argc>4) usage( prog_name);

  try {
    Bootstrap::init( orb_ptr);

    SchedulerImpl the_scheduler("my_scheduler");

    try {
      Bootstrap::register_server_in_NS( Bootstrap::SRV_Scheduler(), 
					&the_scheduler);
      the_scheduler.register_callback( Bootstrap::SRV_Badge_Repository());
      the_scheduler.register_callback( Bootstrap::SRV_Sensor_Repository());
      the_scheduler.register_context_callback( Bootstrap::CTX_ABng_Core_System_Pollers());
    }
    catch ( CORBA::Exception &ex) {
      cerr << "Error while contacting server manager" << ex << endl;
      return 1;
    }

    //
    // Enter the event loop
    //
#ifdef DEBUG
    cerr << "Enter event loop" << endl;
#endif
    try {
#ifdef ORBIX
      boa_ptr->processEvents( implementation_timeout);
#elif defined OMNIORB
      boa_ptr->impl_is_ready();
#else
#error Not implemented
#endif
    }
    catch ( CORBA::SystemException &ex) {
      cerr << "Caught system exception " << ex
	   << " -- impl_is_ready" << endl;
      return 1;
    }
  }
  catch (CORBA::Exception &ex) {
    cerr << "Caught exception " << ex
         << " -- creating scheduler" << endl;
    return 1;
  }
#ifdef DEBUG
  cerr << "Implementation timed out, exiting" << endl;
#endif
  return 0;
}

