// $Id: sched_repos.cc,v 1.4 1998/07/29 13:47:07 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/scheduler/sched_repos.cc,v $
//
// $Log: sched_repos.cc,v $
// Revision 1.4  1998/07/29 13:47:07  mencnar
// exceptions Unknown_Badge(Sensor)_Id not thrown now
//
// Revision 1.3  1998/06/18 10:21:56  mencnar
// moved to CVS
//
// Revision 1.2  1997/10/23 14:52:01  mencnar
// Added comments
//
// Revision 1.1  1997/10/23 09:22:12  mencnar
// Initial revision
//
//

// Temporary log:
// Changes in IDLs
//

#include <iostream.h>
#include "sched_main.hh"
#include "sched_debug.hh"
#include "compatibility.hh"


Sched_SB_Repositories::Sched_SB_Repositories(
  SchedulerImpl &_impl) : 
   impl( _impl),
   sensors_descr( NULL), badges_descr( NULL) {

  CONNECT_IMPL( ABng_Core_System::System_Static_Data_Observer_var,this);
}


Sched_SB_Repositories::~Sched_SB_Repositories() {
  if (sensors_descr) delete sensors_descr;
  if (badges_descr) delete badges_descr;
}

//___________________________________________________________________________
// Creates sequence of all badge ids found in badge repository
//
ABng_Core_System::Badge_Ids Sched_SB_Repositories::get_all_badge_ids() {
  auto_lock local_lock( *this);

  update_badges();
  int size;
  ABng_Core_System::Badge_Ids ids;

  if (badges_descr == NULL) size= 0;
  else size= badges_descr->length();
  ids.length( size);
  
  for (int i=0; i<size; i++)
    ids[i]= (*badges_descr)[i].id;
  
  return ids;
}

void Sched_SB_Repositories::update_badges() {
  ABng_Core_System::Badge::Value_Descriptions *new_badges_descr= NULL;


  ABng_Core_System::Badge::Pattern pattern;
  pattern.is_any= 1;

  if (badges_descr == NULL) {
    try {
      if (! CORBA::is_nil( badge_rep_ref.operator->())) 
	new_badges_descr= badge_rep_ref->find_values( pattern);
    }  
    catch ( CORBA::COMM_FAILURE &) {
      reference_mutex.lock();
      badge_rep_ref= ABng_Core_Management::Badge_Rep::_nil();
      reference_mutex.unlock();
    }
    catch (...) {}
    if (new_badges_descr) {
      if (badges_descr) delete badges_descr;
      badges_descr= new_badges_descr;
    }
  }
}

void Sched_SB_Repositories::set_badge_rep( ABng_Core_Management::Badge_Rep_var ref) {
  reference_mutex.lock();

  badge_rep_ref= ref;

  try {
    
      badge_rep_ref->attach_add_system_static_data_observer( this);
#ifdef DEBUG
      cerr << "Attach succeeded" << endl;
#endif;
      
      ABng_Core_System::Badge::Pattern pattern;
      pattern.is_any= 1;
      ABng_Core_System::Badge::Value_Descriptions_var val_desc=
	badge_rep_ref->find_values( pattern);
      int size= val_desc->length();
      Domain_Repository::Repository_Item_Ids ids( size);
      ids.length( size);

      for (int i=0; i<size; i++)
	ids[i]= val_desc[i].item_id;
      badge_rep_ref->attach_system_static_data_observer( this, ids);
  
  }
  catch ( CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << "set_badge_rep failed" << ex << endl;
#endif
    badge_rep_ref= ABng_Core_Management::Badge_Rep::_nil();
  }
  reference_mutex.unlock();
  {
    delete badges_descr;
    badges_descr= 0;
  }
}

void Sched_SB_Repositories::notify(
	       Domain_Repository::Repository_Item_Id item_id, 
	       const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object &sd,
               Domain_Repository::Change_Type change
	       ENV_DEF(env)) {
  auto_lock local_lock( *this);

  try {
    if ( change == Domain_Repository::OBJECT_ADDED) {
#ifdef DEBUG
      cerr << "OBJECT_ADDED (sensor/badge)" << endl;
#endif
      switch (sd._d()) {
      case ABng_Core_System::System_Static_Data_Observer::SENSOR_SD:
	sd.sensor_obj()->attach_system_static_data_observer( this);
      case ABng_Core_System::System_Static_Data_Observer::BADGE_SD:
	sd.badge_obj()->attach_system_static_data_observer( this);
      }
    }
    else 
#ifdef DEBUG
      cerr << "OBJECT_REMOVED/UPDATED (sensor/badge)" << endl;
#endif
  }
  catch (...) {}

  if (badges_descr) {
    delete badges_descr;
    badges_descr= 0;
  }
  if (sensors_descr) {
    delete sensors_descr;
    sensors_descr= 0;
  }    
}

void Sched_SB_Repositories::abort(
	       ENV_DEF1(env)) {
  auto_lock local_lock( *this);

  if (badges_descr) {
    delete badges_descr;
    badges_descr= 0;
  }
  if (sensors_descr) {
    delete sensors_descr;
    sensors_descr= 0;
  }   
}



//____________________________________________________________________________
// Creates sequence of all sensor ids found in badge repository
//
ABng_Core_System::Sensor_Ids Sched_SB_Repositories::get_all_sensor_ids() {
  auto_lock local_lock( *this);

  update_sensors();

  int size;
  ABng_Core_System::Sensor_Ids ids;

  if (sensors_descr == NULL) size= 0;
  else size= sensors_descr->length();
  ids.length( size);
  
  for (int i=0; i<size; i++)
    ids[i]= (*sensors_descr)[i].id;
  
  return ids;
}

void Sched_SB_Repositories::update_sensors() {
  ABng_Core_System::Sensor::Value_Descriptions *new_sensors_descr= NULL;

  ABng_Core_System::Sensor::Pattern pattern;
  pattern.is_any= 1;

  if (sensors_descr == NULL) {
    try {
      if (! CORBA::is_nil( sensor_rep_ref.operator->()))
	new_sensors_descr= sensor_rep_ref->find_values( pattern);
    }  
    catch ( CORBA::COMM_FAILURE &) {
      reference_mutex.lock();
      sensor_rep_ref= ABng_Core_Management::Sensor_Rep::_nil();
      reference_mutex.unlock();
    }
    catch (...) {}

    if (new_sensors_descr) {
      if (sensors_descr) delete sensors_descr;
      sensors_descr= new_sensors_descr;
    }
  }  
}

void Sched_SB_Repositories::set_sensor_rep( ABng_Core_Management::Sensor_Rep_var ref) {
  reference_mutex.lock();

  sensor_rep_ref= ref;

  try {
      sensor_rep_ref->attach_add_system_static_data_observer( this);
      
      ABng_Core_System::Sensor::Pattern pattern;
      pattern.is_any= 1;
      ABng_Core_System::Sensor::Value_Descriptions_var val_desc=
	sensor_rep_ref->find_values( pattern);
      int size= val_desc->length();
      Domain_Repository::Repository_Item_Ids ids( size);
      ids.length( size);

      for (int i=0; i<size; i++)
	ids[i]= val_desc[i].item_id;
      sensor_rep_ref->attach_system_static_data_observer( this, ids);
  }
  catch (...) {
    sensor_rep_ref= ABng_Core_Management::Sensor_Rep::_nil();
  }
  reference_mutex.unlock();
  {
    auto_lock local_lock( *this);
    delete sensors_descr;
    sensors_descr= 0;
  }
}


//___________________________________________________________________________
// Check if sepcified badge id is known and valid
// May throw exception Badge_Id_Unknown
//
void Sched_SB_Repositories::check_valid( ABng_Core_System::Badge_Id &bid) {
  return;
  /*
  auto_lock local_lock( *this);
  
  if (! badges_descr) update_badges();
  if (! badges_descr) {
#ifdef DEBUG
    cerr << "Badges description list is empty" << endl;
    cerr << "Unknown badge id " << (Badge_Id &) bid << endl;
#endif
    throw ABng_Core_System::Unknown_Badge_Id( bid);
  }
  int size= badges_descr->length();
  for (int i=0; i<size; i++)
    if ((*badges_descr)[i].id == bid) return;
#ifdef DEBUG
  cerr << "Unknown badge id " << bid << endl;
#endif
  throw ABng_Core_System::Unknown_Badge_Id( bid);
  */
}


//__________________________________________________________________________
// Check if specified sensor id is known and valid
// May throw exception Sensor_Id_Unknown
//
void Sched_SB_Repositories::check_valid( ABng_Core_System::Sensor_Id &sid) {
  return;
  /*
  auto_lock local_lock( *this);
  if (! sensors_descr) update_sensors();
  if (! sensors_descr) throw ABng_Core_System::Unknown_Sensor_Id( sid);
  int size= sensors_descr->length();
  for (int i=0; i<size; i++)
    if ((*sensors_descr)[i].id == sid) return;
  throw ABng_Core_System::Unknown_Sensor_Id( sid);
  */
}








