// $Id: sched_request.hh,v 1.4 1998/06/18 10:21:56 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/scheduler/sched_request.hh,v $
//
// $Log: sched_request.hh,v $
// Revision 1.4  1998/06/18 10:21:56  mencnar
// moved to CVS
//
// Revision 1.3  1997/11/03 09:29:54  mencnar
// Memory leaks fixes ( added some destructors)
//
// Revision 1.2  1997/10/24 08:02:05  mencnar
// User identification using Orbix principal
//
// Revision 1.1  1997/10/23 09:21:42  mencnar
// Initial revision
//
//

//----------------------------------------------------Sched_Complex_Request
//
class Sched_Complex_Request : public ABng_Guarded {
public:
  RWTPtrDlist<Sched_Request> subreq;
  SchedulerImpl &schimpl;
  ABng_Core::Command_Context context;
  Scheduler_Caller_var reply_to;
  int reply_to_is_valid;
  CORBA::Long req_id;
  ABng_Core::Time request_time;
  MY_PRINCIPAL_VAR principal;
  CORBA::String_var user_name;
  int subreq_counter; // when reaches 0, the request is terminated;


  Sched_Complex_Request( SchedulerImpl &a_schimpl,
			 const ABng_Core::Command_Context &a_context,
			 const Scheduler_Caller_var &a_caller,
			 const MY_PRINCIPAL_VAR _principal);

  virtual void send_command( Sched_Request &req)= 0;
  virtual int hasReply()= 0;
  virtual void describe_request( Scheduler_Request &)= 0;
  virtual int accepts_reply( Sched_Request &, Sched_Reply &)= 0;
  virtual ~Sched_Complex_Request();
  // safe destructor

  ABng_Poller::AB_Poller_var poller( const Sched_Barrier_Id &loc);

  void start();
  // start all associated threads

  void cancel();
  // cancel all subrequests and join their threads 

  void subrequest_completed();
};

//------------------------------------------------------Sched_Badge_Request
//
class Sched_Badge_Request : public Sched_Complex_Request {
public:
  ABng_Core_System::Badge_Ids badges;
  ABng_Core_System::Sensor_Ids sensors;

  Sched_Badge_Request( SchedulerImpl &a_schimpl,
		       const ABng_Core::Command_Context &a_context,
		       const Scheduler_Caller_var &caller,
		       const MY_PRINCIPAL_VAR _principal,
		       const ABng_Core_System::Badge_Ids &_badges,
		       const ABng_Core_System::Sensor_Ids &_sensors);
  int hasReply() { return 1; }
  virtual void describe_request( Scheduler_Request &);  
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};


//------------------------------------------------Sched_Badge_Request_issue
//
class Sched_Badge_Request_issue : public Sched_Badge_Request {
public:
  ABng_Poller::AB_Poller::Octets data;
  CORBA::Boolean r_has_reply;
  CORBA::Octet reply_code;

  Sched_Badge_Request_issue( SchedulerImpl &a_schimpl,
		       const ABng_Core::Command_Context &a_context,
		       const Scheduler_Caller_var &caller,
		       const MY_PRINCIPAL_VAR _principal,
		       const ABng_Core_System::Badge_Ids &_badges,
		       const ABng_Core_System::Sensor_Ids &_sensors,
		       const ABng_Poller::AB_Poller::Octets &_data,
		       CORBA::Boolean _has_reply,
		       CORBA::Octet _reply_code) :
    Sched_Badge_Request( a_schimpl, a_context, caller, _principal, _badges, _sensors),
    data( _data), r_has_reply( _has_reply), reply_code( _reply_code) {}

  virtual void describe_request( Scheduler_Request &);  
  virtual void send_command( Sched_Request &req);
  virtual int hasReply() { return r_has_reply; }
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};

//--------------------------------------------------Sched_Request_challenge
class Sched_Request_challenge : public Sched_Badge_Request {
public:
  ABng_Poller::AB_Poller::Octets random;

  Sched_Request_challenge( SchedulerImpl &a_schimpl,
			   const ABng_Core::Command_Context &a_context,
			   const Scheduler_Caller_var &caller,
			   const MY_PRINCIPAL_VAR _principal,
			   const ABng_Core_System::Badge_Ids &_badges,
			   const ABng_Core_System::Sensor_Ids &_sensors,
			   const ABng_Poller::AB_Poller::Octets &_random) :
    Sched_Badge_Request( a_schimpl, a_context, caller, _principal, _badges, _sensors),
    random( _random) {}   
	       

  void send_command( Sched_Request &req);
  virtual void describe_request( Scheduler_Request &);  
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};

//------------------------------------------------------Sched_Request_sound
//
class Sched_Request_sound : public Sched_Badge_Request {
public:
  ABng_Poller::AB_Poller::Octets sound;

  Sched_Request_sound( SchedulerImpl &a_schimpl,
		       const ABng_Core::Command_Context &a_context,
		       const Scheduler_Caller_var &caller,
		       const MY_PRINCIPAL_VAR _principal,
		       const ABng_Core_System::Badge_Ids &_badges,
		       const ABng_Core_System::Sensor_Ids &_sensors,
		       const ABng_Poller::AB_Poller::Octets &_sound) : 
    Sched_Badge_Request( a_schimpl, a_context, caller, _principal, _badges, _sensors),
    sound( _sound) {}

  void send_command( Sched_Request &req);
  void describe_request( Scheduler_Request &dreq);
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};

//--------------------------------------------------Sched_Request_set_state
//
class Sched_Request_set_state : public Sched_Badge_Request {
public:
  CORBA::Octet status;
  ABng_Poller::AB_Poller::Octets sound;

  Sched_Request_set_state( SchedulerImpl &a_schimpl,
			   const ABng_Core::Command_Context &a_context,
			   const Scheduler_Caller_var &caller,
			   const MY_PRINCIPAL_VAR _principal,
			   const ABng_Core_System::Badge_Ids &_badges,
			   const ABng_Core_System::Sensor_Ids &_sensors,
			   CORBA::Octet _status,
			   const ABng_Poller::AB_Poller::Octets &_sound) : 
    Sched_Badge_Request( a_schimpl, a_context, caller, _principal, _badges, _sensors),
    status( _status), sound( _sound) {}

  void send_command( Sched_Request &req);
  virtual void describe_request( Scheduler_Request &);  
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};


//--------------------------------------------------Sched_Request_get_state
//
class Sched_Request_get_state : public Sched_Badge_Request {
public:
  Sched_Request_get_state( SchedulerImpl &a_schimpl,
			   const ABng_Core::Command_Context &a_context,
			   const Scheduler_Caller_var &caller,
			   const MY_PRINCIPAL_VAR _principal,
			   const ABng_Core_System::Badge_Ids &_badges,
			   const ABng_Core_System::Sensor_Ids &_sensors) :
    Sched_Badge_Request( a_schimpl, a_context, caller, _principal, _badges, _sensors) {}

  void send_command( Sched_Request &req);
  virtual void describe_request( Scheduler_Request &);  
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};

//---------------------------------------------------Sched_Request_set_home
//
class Sched_Request_set_home : public Sched_Badge_Request {
public:
  ABng_Poller::AB_Poller::Octets address;

  Sched_Request_set_home( SchedulerImpl &a_schimpl,
			  const ABng_Core::Command_Context &a_context,
			  const Scheduler_Caller_var &caller,
			  const MY_PRINCIPAL_VAR _principal,
			  const ABng_Core_System::Badge_Ids &_badges,
			  const ABng_Core_System::Sensor_Ids &_sensors,
			  const ABng_Poller::AB_Poller::Octets &_address) :
    Sched_Badge_Request( a_schimpl, a_context, caller, _principal, _badges, _sensors),
    address( _address) {}

  void send_command( Sched_Request &req);
  virtual void describe_request( Scheduler_Request &);  
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};

//---------------------------------------------------Sched_Request_get_home
class Sched_Request_get_home : public Sched_Badge_Request {
public:
  Sched_Request_get_home( SchedulerImpl &a_schimpl,
			  const ABng_Core::Command_Context &a_context,
			  const Scheduler_Caller_var &caller,
			  const MY_PRINCIPAL_VAR _principal,
			  const ABng_Core_System::Badge_Ids &_badges,
			  const ABng_Core_System::Sensor_Ids &_sensors) :
    Sched_Badge_Request( a_schimpl, a_context, caller, _principal, _badges, _sensors) {}

  void send_command( Sched_Request &req);
  virtual void describe_request( Scheduler_Request &);  
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};


//-----------------------------------------------------Sched_Sensor_Request
//
class Sched_Sensor_Request : public Sched_Complex_Request {
public:
  ABng_Core_System::Sensor_Ids sensors;

  Sched_Sensor_Request( SchedulerImpl &a_schimpl,
			const ABng_Core::Command_Context &a_context,
			const Scheduler_Caller_var &caller,
			const MY_PRINCIPAL_VAR _principal,
			const ABng_Core_System::Sensor_Ids &_sensors);

  virtual void describe_request( Scheduler_Request &);  
  virtual int accepts_reply( Sched_Request &, Sched_Reply &) { return 0; }
};


//------------------------------------------------Sched_Sensor_Request_issue
//
class Sched_Sensor_Request_issue : public Sched_Sensor_Request {
public:
  ABng_Poller::AB_Poller::Octets data;
  CORBA::Boolean r_has_reply;
  CORBA::Octet reply_code;

  Sched_Sensor_Request_issue( SchedulerImpl &a_schimpl,
			      const ABng_Core::Command_Context &a_context,
			      const Scheduler_Caller_var &caller,
			      const MY_PRINCIPAL_VAR _principal,
			      const ABng_Core_System::Sensor_Ids &_sensors,
			      const ABng_Poller::AB_Poller::Octets &_data,
			      CORBA::Boolean _has_reply,
			      CORBA::Octet _reply_code) :
    Sched_Sensor_Request( a_schimpl, a_context, caller, _principal, _sensors),
    data( _data),
    r_has_reply( _has_reply), reply_code( _reply_code) {}

  virtual void describe_request( Scheduler_Request &);  
  virtual void send_command( Sched_Request &req);
  virtual int hasReply() { return r_has_reply; }
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};

//------------------------------------------------Sched_Request_reset_buffer
//
class Sched_Request_reset_buffer : public Sched_Sensor_Request {
public:
  Sched_Request_reset_buffer( SchedulerImpl &a_schimpl,
			      const ABng_Core::Command_Context &a_context,
			      const Scheduler_Caller_var &caller,
			      const MY_PRINCIPAL_VAR _principal,
			      const ABng_Core_System::Sensor_Ids &_sensors) :
    Sched_Sensor_Request( a_schimpl, a_context, caller, _principal, _sensors) {}

  virtual void describe_request( Scheduler_Request &);  
  virtual void send_command( Sched_Request &req);
  virtual int hasReply() { return 0; }
};

//------------------------------------------------Sched_Request_reset_fifos
//
class Sched_Request_reset_fifos : public Sched_Sensor_Request {
public:
  Sched_Request_reset_fifos( SchedulerImpl &a_schimpl,
			     const ABng_Core::Command_Context &a_context,
			     const Scheduler_Caller_var &caller,
			     const MY_PRINCIPAL_VAR _principal,
			     const ABng_Core_System::Sensor_Ids &_sensors) :
    Sched_Sensor_Request( a_schimpl, a_context, caller, _principal, _sensors) {}

  virtual void describe_request( Scheduler_Request &);  
  virtual void send_command( Sched_Request &req);
  virtual int hasReply() { return 0; }
};

//-------------------------------------------Sched_Request_get_station_state
//
class Sched_Request_get_station_state : public Sched_Sensor_Request {
public:

  Sched_Request_get_station_state( SchedulerImpl &a_schimpl,
				   const ABng_Core::Command_Context &a_context,
				   const Scheduler_Caller_var &caller,
				   const MY_PRINCIPAL_VAR _principal,
				   const ABng_Core_System::Sensor_Ids &_sensors) :
    Sched_Sensor_Request( a_schimpl, a_context, caller, _principal, _sensors) {}

  virtual void describe_request( Scheduler_Request &);  
  virtual void send_command( Sched_Request &req);
  virtual int hasReply() { return 1; }
  virtual int accepts_reply( Sched_Request &, Sched_Reply &);
};


//--------------------------------------------------Sched_Request_set_output
//
class Sched_Request_set_output : public Sched_Sensor_Request {
public:
  CORBA::Octet output_value;

  Sched_Request_set_output( SchedulerImpl &a_schimpl,
			    const ABng_Core::Command_Context &a_context,
			    const Scheduler_Caller_var &caller,
			    const MY_PRINCIPAL_VAR _principal,
			    const ABng_Core_System::Sensor_Ids &_sensors,
			    CORBA::Octet ov) :
    Sched_Sensor_Request( a_schimpl, a_context, caller, _principal, _sensors),
    output_value( ov) {}

  virtual void describe_request( Scheduler_Request &);  
  virtual void send_command( Sched_Request &req);
  virtual int hasReply() { return 0; }
};

//--------------------------------------------Sched_Request_set_persistence
//
class Sched_Request_set_persistence : public Sched_Sensor_Request {
public:
  CORBA::Boolean mode;

  Sched_Request_set_persistence( SchedulerImpl &a_schimpl,
				 const ABng_Core::Command_Context &a_context,
				 const Scheduler_Caller_var &caller,
				 const MY_PRINCIPAL_VAR _principal,
				 const ABng_Core_System::Sensor_Ids &_sensors,
				 CORBA::Boolean md) :
    Sched_Sensor_Request( a_schimpl, a_context, caller, _principal, _sensors),
    mode( md) {}

  virtual void describe_request( Scheduler_Request &);  
  virtual void send_command( Sched_Request &req);
  virtual int hasReply() { return 0; }
};


//------------------------------------------------------------Sched_Request
//
class Sched_Request  {
  
  ABng_Core::Time last_sighting_time;


  int to_nearest_sensor;
  // non zero, if the request is directed to a nearest sensor
  //


  // list of sensors the request is directed to
  RWTValDlist<Sched_Barrier_Id> barrier_ids;

  Sched_Reply *my_reply;
  int cancel_flag;
  omni_mutex &guard;
  omni_condition cond_sleep;
  omni_condition cond_sighting_arrived;
  omni_thread *my_thread;
  Seq_Number seq_number;

  static void* static_worker( void *arg);
  void *worker();

public:
  Sched_Complex_Request &complex_request;
  Sched_Barrier &barrier;

  Sched_Barrier_Id last_pos; 
  // set to last sighting position after acquiring sighting
  //  

  Sched_Request(Sched_Complex_Request &a_complex_request,    
		Sched_Barrier &a_barrier,
		const RWTValDlist<Sched_Barrier_Id> &a_bar_loc,
		int t_n_s) :
    complex_request( a_complex_request), barrier( a_barrier), my_reply( NULL),
    cond_sighting_arrived( & a_barrier.mutex()),
    cond_sleep( & a_barrier.mutex()),
    cancel_flag( 0), guard( a_barrier.mutex()), 
    barrier_ids( a_bar_loc), to_nearest_sensor( t_n_s),
    my_thread( new omni_thread(static_worker, (void*) this)) {}

  ~Sched_Request() { if (my_reply) delete my_reply; }


  void start() {
    my_thread->start();
  }

  int operator == ( const Sched_Request &req) { return this == &req; }

  static int compare_time( 
	const ABng_Core::Time &t1,
	const ABng_Core::Time &t2);

  Sched_Barrier_Id & last_seen() { return last_pos; }
  Sched_Barrier & get_barrier() { return barrier; }
  void cancel();
  int cancelled() const				{ return cancel_flag; }
  void reply( Sched_Reply &x_reply);
  void join( void **x) 				{ my_thread->join(x); }

  Seq_Number get_seq_number() const             { return seq_number; }
  
  friend class Sched_Barrier;
};
