// $Id: msp_i.cc,v 1.4 1998/11/30 14:25:20 mencnar Exp $	
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/msp_i.cc,v $
//
// $Log: msp_i.cc,v $
// Revision 1.4  1998/11/30 14:25:20  mencnar
// Inserted RCS/CVS header
//
//

#include <string.h>
#include <iostream.h>
#include "msp_i.h"
#include "abng_core_management.hh"
#include "event_log_i.hh"
#include "omnithread.h"

#ifdef ORBIX
DEF_TIE_ABng_Core_Internals_Main_Sighting_Processor(MSP_I)
DEF_TIE_ABng_Poller_AB_Observer(MSP_I)
#endif


static unsigned hash_string(const RWCString& str) {

  unsigned res = str.hash();
  //cout << "hash_string: " << dec << res << " " << res % RWDEFAULT_CAPACITY << endl;
  return res;
}

void VSP_Entry::add_sensors(
  const ABng_Core_System::Sensor_Ids& sensor_id_list) {

  int len = sensor_id_list.length();
  for(int i = 0; i < len; i++) {
    Comparable_Sensor_Id csid(sensor_id_list[i]);
    my_sensor_list.insert(csid); // copy
#ifdef DEBUG
    cerr << "Insert sensor #" << i << endl;
    cerr << "sensor " << (int) csid.sensor_num 
	 << " network " << (int) csid.network_num
         << " group " << csid.group_name << endl;
#endif
  }
}

void VSP_Entry::remove_sensors(
  const ABng_Core_System::Sensor_Ids& sensor_id_list) {

  int len = sensor_id_list.length();
  if(len) {
    for(int i = 0; i < len; i++) {
      Comparable_Sensor_Id csid(sensor_id_list[i]);
      my_sensor_list.remove(csid); // copy
    }
  }
  else{
    my_sensor_list.clear();
  }
};

int VSP_Entry::contains_sensor(const ABng_Core_System::Sensor_Id& sid) {
  
  Comparable_Sensor_Id csid(sid);
  return my_sensor_list.contains(csid); // copy
};

ABng_Core_System::Sensor_Ids *VSP_Entry::get_sensors() {
  
  unsigned num = (unsigned) my_sensor_list.entries();
  ABng_Core_System::Sensor_Ids *res = 
    new ABng_Core_System::Sensor_Ids(num);
    Sensor_Id_List_Iterator iterator(my_sensor_list);
    int i = 0;
    while(iterator()) {
      (*res)[i++] = (ABng_Core_System::Sensor_Id) 
	iterator.key(); // copy
    }
    res->length(i);
    return res;
};

void VSP_Entry::add_badges(
  const ABng_Core_System::Badge_Ids& badge_id_list) {
  
  int len = badge_id_list.length();
  for(int i = 0; i < len; i++) {
    Comparable_Badge_Id cbid(badge_id_list[i]);
    my_badge_list.insert(cbid); // copy
#ifdef DEBUG
    cerr << "add badge (id) ";
    for (int x=0; x<badge_id_list[i].length(); x++)
      cerr << hex << (unsigned) (badge_id_list[i][x]) << ' ';
    cerr << endl;
#endif
  }
}

void VSP_Entry::remove_badges(
  const ABng_Core_System::Badge_Ids& badge_id_list) {
  
  int len = badge_id_list.length();
  if(len) {
    for(int i = 0; i < len; i++) {
      Comparable_Badge_Id cbid(badge_id_list[i]);
      my_badge_list.remove(cbid); // copy
    }
  }
  else {
    my_badge_list.clear();
  }
}

int VSP_Entry::contains_badge(const ABng_Core_System::Badge_Id& bid) {
  
  Comparable_Badge_Id cbid(bid);
  return my_badge_list.contains(cbid);// copy
};

int VSP_Entry::is_duplicated(const Comparable_Badge_Id& cbid,
			     TimeSeqNumber seq_num) {

  // A sequence number equal to 0 is not a real sequence number. 
  // It means that the message has not got a number.
  if(seq_num.seq_number == 0) {
    return 0;
  }

  TimeSeqNumber last_seq;
  // Retrieve a sequence number of the last message sent by 
  // the badge
  if(badge_ids_to_seq_numbers.findValue(cbid, last_seq )) {// copy
    // cout << "Last num: " << dec << (unsigned) last_num << endl;
    // cout << "Current num: " << dec <<
    //    (unsigned) seq_num << endl;
    
    ABng_Server_Manager::Time dst;
    dst.seconds= seq_num.sighting_time.seconds-last_seq.sighting_time.seconds;
    dst.microseconds= seq_num.sighting_time.microseconds-
      last_seq.sighting_time.microseconds;
    while (dst.microseconds<0) {
      dst.microseconds+= 1000000;
      dst.seconds--;
    }

    if(seq_num.seq_number == last_seq.seq_number &&
       dst.seconds == 0) {
      cerr << "Sighting duplicated, time period " << dst.microseconds << " us" << endl;
      // The message has been duplicated. Discard it. 
      //    cout << "The same seq nums - discard sighting" << endl;
      return 1;
    } 
    // The message has not been duplicated.
    // Replace the old sequence number with the new one.
    badge_ids_to_seq_numbers.remove(cbid);// copy
    badge_ids_to_seq_numbers.insertKeyAndValue(cbid, seq_num);// copy
    return 0;
  }
  // The MSP has not received a message from this badge yet.
  // Remember the sequence number  
  // cout << "Insert seq num" << endl;
  badge_ids_to_seq_numbers.insertKeyAndValue(cbid, seq_num);// copy
  return 0;
}


ABng_Core_System::Badge_Ids *VSP_Entry::get_badges() {
  
  unsigned num = (unsigned) my_badge_list.entries();
  ABng_Core_System::Badge_Ids *res = 
    new ABng_Core_System::Badge_Ids(num);
    Badge_Id_List_Iterator iterator(my_badge_list);
    int i = 0;
    while(iterator()) {
      (*res)[i++] = (ABng_Core_System::Badge_Id) 
	iterator.key();// copy
    }
    res->length(i);
    return res;
};


MSP_I::MSP_I() : 
  Basic_AB_Observer_I(),
  // Basic_I(),
  Bootstrap_listener(),
  names_to_vsps(hash_string, hash_table_size) {
    
#ifdef ORBIX
  my_msp_obj = 
    new TIE_ABng_Core_Internals_Main_Sighting_Processor(MSP_I) (this);
  my_poller_observer_obj =
    new TIE_ABng_Poller_AB_Observer(MSP_I) (this);
#elif defined OMNIORB
  my_msp_obj =
    new TIE_CLASS(ABng_Core_Internals,Main_Sighting_Processor)<MSP_I,0>(this);
  CONNECT_IMPL( ABng_Core_Internals::Main_Sighting_Processor_var,my_msp_obj);
  my_poller_observer_obj =
    new TIE_CLASS(ABng_Poller,AB_Observer)<MSP_I,0>(this);
  CONNECT_IMPL( ABng_Poller::AB_Observer_var,my_poller_observer_obj);

#endif

  Bootstrap::register_server_in_NS(
    Bootstrap::SRV_Main_Sighting_Processor(), my_msp_obj);
  register_context_callback(Bootstrap::CTX_ABng_Core_System_Pollers());
  ps_dev= new ProcessControlImpl( "main_sighting_processor", this);
  init_event_log("msp");
}

MSP_I::~MSP_I() {
  
  names_to_vsps.clearAndDestroy();
  delete ps_dev;
}

void MSP_I::sighting(
  const char *network_group_name,
  ABng_Poller::AB_Poller::Network_Id network, 
  ABng_Poller::AB_Poller::Station_Id station, 
  const ABng_Poller::AB_Poller::Badge_Id& badge,
  ABng_Poller::AB_Poller::Sighting_Code code,
  const ABng_Poller::AB_Poller::Field_Id& field,
  ABng_Poller::AB_Poller::Byte click_num,
  ABng_Poller::AB_Poller::Byte attributes,
  ABng_Poller::AB_Poller::Seq_Number seq,
  const ABng_Poller::AB_Poller::Time_Stamp& ts
  ENV_DEF(env) ) {
  

//  struct rusage ru;
//  struct timeval td;
//  start_timer( &ru, &td);
   
  unsigned i;
 
  LOGSIGHTING( network_group_name, ts);
  // #ifdef DEBUG
  cout << "Received: ";
  cout << "[" << network_group_name << ",";
  cout << hex << (unsigned) network<< ",";
  cout << hex << (unsigned) station << "] ";
  cout << "[";
  for (i = 0; i < badge.length(); i++) {
    cout << hex << (unsigned) badge[i] << " ";
  }
  cout << "] ";
  cout << hex << (unsigned) seq << endl;
  // #endif
    
  Comparable_Badge_Id cbid(badge);
  
  // Store data received from a poller in an Internal_Sighting_Description
  
  ABng_Core_Internals::Internal_Sighting_Description isd;
  
  isd.my_sensor.sensor_num = (ABng_Core_System::Sensor_Number) station;
  isd.my_sensor.network_num = (ABng_Core_System::Network_Number) network;
  isd.my_sensor.group_name = CORBA::string_dup( network_group_name);
  isd.my_badge.length(badge.length());
  for(i = 0; i < badge.length(); i++) {
    isd.my_badge[i] = badge[i];
  }
  isd.message_type = (unsigned short) code;
  isd.attributes = (unsigned short) click_num;
  isd.sequence_number = (ABng_Core::Seq_Number) seq;
  isd.update_time.seconds = ts.sec;
  isd.update_time.microseconds = ts.usec;

  TimeSeqNumber tsn( Bootstrap::gettimeofday(), (ABng_Core::Seq_Number) seq);
  
  Name_VSP_Dictionary_Iterator iterator(names_to_vsps);
  VSP_Entry *vsp_entry;
  RWCString *str;
  while(str = iterator()) {
    vsp_entry = iterator.value();

    if(!(vsp_entry->contains_sensor(isd.my_sensor) &&
	 vsp_entry->contains_badge(isd.my_badge))) {
      // The current vsp does not contain the sensor-badge pair
      // contained in the message - discard the message. 
      continue;
    }
    
    // The current vsp contains both the sensor-badge pair.
    if(!vsp_entry->wants_duplicates()) {
      // Filter out duplicated messages. 
      if(vsp_entry->is_duplicated(cbid, tsn)) {
        // The message is duplicated and the vsp does not want it.
        continue;
      }
    }
    // Now we are ready to send the sighting message to the VSP.
    try {
#ifdef DEBUG
      cout << "Sending to " << *str << endl;
#endif
      ABng_Core_Internals::View_Sighting_Processor_var vsp = 
	vsp_entry->get_vsp();

      /*      {
	omni_mutex mtx;
	omni_condition cnd( &mtx);
	for (int x=0; x<2000; x++) { */
     	  vsp->process_sighting(isd);
	  /*
	  mtx.lock();
	  unsigned long sec,nsec;
	  omni_thread::get_time(    &sec, &nsec, 
				    0, 100000);
 
	  cnd.timed_wait( sec, nsec);
	  mtx.unlock();
	}
      }
      */
    }
    catch (CORBA::SystemException &sysEx) {
      cout << "Cannot contact a vsp for view " << *str << endl;
      cout << "Removing vsp entry" << endl;
      try {
	remove_vsp(*str);
      }
      catch(...) {
      }
    }
    catch(...) {
      // ...
    }
  }

//  stop_timer( &ru, &td);
//  show_timer( &ru, &td);
} 

ABng_Core_Internals::Main_Sighting_Processor::
  View_Sighting_Processors *MSP_I::view_sp_list (
  ENV_DEF1(env) ) 
  throw (CORBA::SystemException) {

  int len = names_to_vsps.entries();
  ABng_Core_Internals::Main_Sighting_Processor::
    View_Sighting_Processors *res = 
      new ABng_Core_Internals::Main_Sighting_Processor::
        View_Sighting_Processors(len);
  res->length(len);

  Name_VSP_Dictionary_Iterator iterator(names_to_vsps);
  for(int i = 0; i < len && iterator(); i++) {
    (*res)[i] = ABng_Core_Internals::View_Sighting_Processor::_duplicate(
      iterator.value()->get_vsp());
  }
  return res;
}

void MSP_I::add_vsp (const char *view_name,
    ABng_Core_Internals::View_Sighting_Processor_ptr vsp, 
    const ABng_Core_System::Badge_Ids& badge_id_list, 
    const ABng_Core_System::Sensor_Ids& sensor_id_list, 
    CORBA::Boolean wants_duplicates
    ENV_DEF(env) ) 
    throw (CORBA::SystemException,
	Domain_Repository::Unknown_Object_Ref,
	Domain_Repository::Duplicate_Data_Record) {

#ifdef DEBUG
  cerr << "MSP_I::add_vsp " << view_name << endl;
#endif

  RWCString *str = new RWCString(view_name); //copy
  if (names_to_vsps.findValue(str)) {
    delete str;
    throw Domain_Repository::Duplicate_Data_Record();
  }
  VSP_Entry *vsp_entry_ptr = 
    new VSP_Entry( ABng_Core_Internals::View_Sighting_Processor::_duplicate( vsp), 
		   wants_duplicates);

  vsp_entry_ptr->add_badges(badge_id_list);
  vsp_entry_ptr->add_sensors(sensor_id_list);
  names_to_vsps.insertKeyAndValue(str, vsp_entry_ptr);
  // print_contents();
};

void MSP_I::find_vsp (
    const char *view_name, 
    ABng_Core_Internals::View_Sighting_Processor_ptr& vsp,
    ABng_Core_System::Badge_Ids*& badge_id_list, 
    ABng_Core_System::Sensor_Ids*& sensor_id_list, 
    CORBA::Boolean& wants_duplicates
    ENV_DEF(env) ) 
    throw (CORBA::SystemException,
	ABng_Core::Unknown_View_Name) {

  VSP_Entry *vsp_entry_ptr;
  RWCString *str = new RWCString(view_name); //copy
  if(!(vsp_entry_ptr = names_to_vsps.findValue(str))) {
    delete str;
    throw ABng_Core::Unknown_View_Name();
  }
  delete str;
  vsp = vsp_entry_ptr->get_vsp();
  badge_id_list = vsp_entry_ptr->get_badges();
  sensor_id_list = vsp_entry_ptr->get_sensors();
  wants_duplicates = vsp_entry_ptr->wants_duplicates();
}

void MSP_I::remove_vsp (
  const char *view_name
  ENV_DEF(env) ) 
  throw (CORBA::SystemException, 
	 ABng_Core::Unknown_View_Name) {

#ifdef DEBUG
  cerr << "MSP_I::remove_vsp " << view_name << endl;
#endif

  VSP_Entry *vsp_entry = 0;
  RWCString *str = new RWCString(view_name); //copy
  RWCString *found;
  if(!(found = names_to_vsps.remove(str))) {
    delete str;
    throw ABng_Core::Unknown_View_Name();
  }
  delete str;
  delete found;
}

void  MSP_I::add_badges (
  const char *view_name,
  const ABng_Core_System::Badge_Ids& badge_id_list
  ENV_DEF(env) ) 
  throw (CORBA::SystemException, 
	 ABng_Core::Unknown_View_Name) {

#ifdef DEBUG
  cerr << "MSP_I::add_badges " << badge_id_list.length() 
       << ' ' << view_name
       << endl;
#endif

  VSP_Entry *vsp_entry_ptr;
  RWCString *str = new RWCString(view_name);
  if(!(vsp_entry_ptr = names_to_vsps.findValue(str))) {
    delete str;
    throw ABng_Core::Unknown_View_Name();
  }
  delete str;
  vsp_entry_ptr -> add_badges(badge_id_list);
}

void  MSP_I::remove_badges (
  const char *view_name,
  const ABng_Core_System::Badge_Ids& badge_id_list
  ENV_DEF(env) ) 
  throw (CORBA::SystemException, 
	 ABng_Core::Unknown_View_Name, 
	 ABng_Core_System::Unknown_Badge_Id) {

#ifdef DEBUG
  cerr << "MSP_I::remove_badges " << badge_id_list.length() << endl;
#endif

  VSP_Entry *vsp_entry_ptr;
  RWCString *str = new RWCString(view_name);
  if(!(vsp_entry_ptr = names_to_vsps.findValue(str))) {
    delete str;
    throw ABng_Core::Unknown_View_Name();
  }
  delete str;
  vsp_entry_ptr->remove_badges(badge_id_list);
}

void  MSP_I::add_sensors(const char * view_name,
  const ABng_Core_System::Sensor_Ids& sensor_id_list
  ENV_DEF(env) ) 
  throw (CORBA::SystemException, 
	 ABng_Core::Unknown_View_Name) {

#ifdef DEBUG
  cerr << "MSP_I::add_sensors" << endl;
#endif

  VSP_Entry *vsp_entry_ptr;
  RWCString *str = new RWCString(view_name);
  if(!(vsp_entry_ptr = names_to_vsps.findValue(str))) {
    delete str;
    throw ABng_Core::Unknown_View_Name();
  }
  delete str;
  vsp_entry_ptr -> add_sensors(sensor_id_list);
#ifdef DEBUG
  cerr << "MSP_I::add_sensor succeeded" << endl;
#endif
}

void  MSP_I::remove_sensors (
  const char *view_name,
  const ABng_Core_System::Sensor_Ids& sensor_id_list
  ENV_DEF(env) ) 
  throw (CORBA::SystemException, 
	 ABng_Core::Unknown_View_Name,
	 ABng_Core_System::Unknown_Sensor_Id) {

#ifdef DEBUG
  cerr << "MSP_I::remove_sensors " << sensor_id_list.length() << endl;
#endif

  VSP_Entry *vsp_entry_ptr;
  RWCString *str = new RWCString(view_name);
  if(!(vsp_entry_ptr = names_to_vsps.findValue(str))) {
    delete str;
    throw ABng_Core::Unknown_View_Name();
  }
  delete str;
  vsp_entry_ptr->remove_sensors(sensor_id_list);
}

void MSP_I::reference_resolved(
  const ABng_Server_Manager::Name &server_name, 
  CORBA::Object_var server_ref,
  ABng_Server_Manager::uid callback_id) {

  cout << "Reference_resolved: " << server_name;
  cout << endl;

  if ( server_name >= Bootstrap::CTX_ABng_Core_System_Pollers()) {
    try {
      ABng_Poller::AB_Poller_var pol = 
	ABng_Poller::AB_Poller::_narrow(server_ref);
      pol->detach_observer(my_poller_observer_obj);
      pol->attach_observer(my_poller_observer_obj);
    }
    catch (...) {
      cerr << "Connect to a poller failed" << endl;
      //      invalidate_reference(server_name, directory_name);
    }
    cout << "Connected to poller " << server_name << endl;
    return;
  }
}

void MSP_I::print_contents() {

    cout << "===================================" << endl;
    cout << "             MSP contents" << endl;
    cout << "===================================" << endl;

    Name_VSP_Dictionary_Iterator iterator(names_to_vsps);
    VSP_Entry *vsp_entry;
    RWCString *str;
    int i = 0;
    while(str = iterator()) {
        vsp_entry = iterator.value();
        cout << "vsp " << *str << "{" << endl;
        Sensor_Id_List_Iterator sensor_set_iterator(vsp_entry->my_sensor_list);
        ABng_Core_System::Sensor_Id sid;
        cout << "\tSensors: {" << endl;
        while(sensor_set_iterator()) {
            sid = sensor_set_iterator.key();
            cout << "\t\t{";
            cout << sid.group_name << ",";
            cout << hex << sid.network_num << ",";
            cout << hex << sid.sensor_num << "}" << endl;
        }
        cout << "\t}" << endl;
        Badge_Id_List_Iterator badge_set_iterator(vsp_entry->my_badge_list);
        ABng_Core_System::Badge_Id bid;
        cout << "\tBadges: {" << endl;
        while(badge_set_iterator()) {
            bid = badge_set_iterator.key();
            cout << "\t\t[";
            for (i = 0; i < bid.length(); i++) {
              cout << hex << (unsigned) bid[i] << " ";
            }
            cout << "]" << endl;
        } 
        cout << "\t}" << endl;
        VSP_Entry::Badge_Id_Seq_Number_Dictionary_Iterator
          badge_ids_to_seq_numbers_iter(vsp_entry->badge_ids_to_seq_numbers);
        cout << "\tSequence numbers: {" << endl;
        while(badge_ids_to_seq_numbers_iter()) {
          cout << "\t\t{" << endl;
          ABng_Core_System::Badge_Id bid = badge_ids_to_seq_numbers_iter.key();
          cout << "[";
          for (i = 0; i < bid.length(); i++) {
            cout << bid[i] << " ";
          }
          cout << "], ";
          cout << badge_ids_to_seq_numbers_iter.value().seq_number << "}" << endl;
        }    
    }
    cout << "\t}" << endl;
    cout << "}" << endl;
    cout << "===================================" << endl;
}


void MSP_I::test() {
}




