// $Id: view_i.cc,v 1.2 1998/07/29 14:57:11 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/view_i.cc,v $
//
// $Log: view_i.cc,v $
// Revision 1.2  1998/07/29 14:57:11  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimization
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:41:47  jasz
// Initial revision
//
//
// File:          view_i.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  10.01.98
//
// General Description:
//   Definition of the View_I class implementing the View ABng component.

#ifdef ORBIX
#define EXCEPTIONS
#endif

#include <rw/rwdate.h>
#include <rw/rwtime.h>
#include "view_i.h"
#include "const.h"
#include "db_utils.hh"
#include "event_log_i.hh"
#include <rw/tvdlist.h>
#include <sys/resource.h>

extern "C"  int getrusage(int who, struct rusage *rusage);

#ifdef ORBIX
DEF_TIE_ABng_Core_View(View_I)
DEF_TIE_ABng_Core_Management_View_Manager(View_I)
DEF_TIE_ABng_Core_Internals_View_Sighting_Processor(View_I)
DEF_TIE_ABng_Core_System_System_Static_Data_Observer(View_I)
DEF_TIE_ABng_Core_System_Scheduler_Caller(View_I)
#endif

View_I *View_I::inst = 0;

// Constants for quality attribute updates.
const unsigned View_I::MIN_QUALITY = 0;
const unsigned View_I::MAX_QUALITY = 100;
const unsigned View_I::QUALITY_STEP = 20;


View_I *View_I::instance() {
  
  if(!inst) {
    cerr << "Cannot get an instance of the View implementation" << endl;
  }
  return inst;
} 

// The hot start constructor. Passing a view name is necessary as it
// determines the directory when the databases for this view are placed. 

View_I::  View_I(
#ifdef ObjectStore
  os_database *view_db_p,
  os_database *location_rep_db_p,
  os_database *locatable_rep_db_p,
#elif defined FilePersistency
  const char *view_db_p,
  const char *location_rep_db_p,
  const char *locatable_rep_db_p,
#endif
  const char *view_name_p,
  long sightings_dump_period_p
  ) :
  Bootstrap_listener(),
  ABng_Guarded(),
  repos_cond( & repos_guard.mutex()),
  last_repos_resolved( 0),
  my_wants_duplicates((CORBA::Boolean) 0),
  my_location_rep(location_rep_db_p, 0),
  my_locatable_rep(locatable_rep_db_p, view_name_p, sightings_dump_period_p, 
		   0, 0),
  my_view_obj(ABng_Core::View::_nil()),
  my_view_manager_obj(ABng_Core_Management::View_Manager::_nil()),
  my_vsp_obj(ABng_Core_Internals::View_Sighting_Processor::_nil()),
  my_scheduler(ABng_Core_System::Scheduler::_nil()),
  my_scheduler_caller_obj(ABng_Core_System::Scheduler_Caller::_nil()),
  my_msp(ABng_Core_Internals::Main_Sighting_Processor::_nil()) {

  if(inst) {
    cerr << "View cannot be instantiated twice." << endl; 
    return;
  }
  inst = this;
  if(!view_name_p || strlen(view_name_p) == 0) {
    cerr << "Empty view_name" << endl;
    exit(1);
  }
  my_view_name = CORBA::string_dup(view_name_p);
  
  if(!view_db_p) {
    cerr << "Empty view database name" << endl;
    exit(1);
  }
  my_view_db = new char[strlen(view_db_p) + 1];
  strcpy(my_view_db, view_db_p);
  init_event_log( view_name_p);  
}

// The cold start constructor.
View_I::  View_I(
#ifdef ObjectStore
  os_database *view_db_p,
  os_database *location_rep_db_p,
  os_database *locatable_rep_db_p,
#elif defined FilePersistency
  const char *view_db_p,
  const char *location_rep_db_p,
  const char *locatable_rep_db_p,
#endif
  const char *view_name_p,
  const char *location_type_name_p,
  int wants_duplicates_p,
  long sightings_dump_period_p) :
  Bootstrap_listener(),
  ABng_Guarded(),
  repos_cond( & repos_guard.mutex()),
  last_repos_resolved( 0),
  my_wants_duplicates((CORBA::Boolean) wants_duplicates_p),
  my_location_rep(location_rep_db_p, view_name_p, location_type_name_p, 500000),
  my_locatable_rep(locatable_rep_db_p, view_name_p, sightings_dump_period_p, 500000),
  my_view_obj(ABng_Core::View::_nil()),
  my_view_manager_obj(ABng_Core_Management::View_Manager::_nil()),
  my_vsp_obj(ABng_Core_Internals::View_Sighting_Processor::_nil()),
  my_scheduler(ABng_Core_System::Scheduler::_nil()),
  my_scheduler_caller_obj(ABng_Core_System::Scheduler_Caller::_nil()),
  my_msp(ABng_Core_Internals::Main_Sighting_Processor::_nil()) {

  if(inst) {
    cerr << "View cannot be instantiated twice." << endl; 
    return;
  }
  inst = this;

  if(!view_name_p || strlen(view_name_p) == 0) {
    cerr << "Empty view_name" << endl;
    exit(1);
  }
  my_view_name = CORBA::string_dup(view_name_p);  
  
  if(!view_db_p) {
    cerr << "Empty view database name" << endl;
    exit(1);
  }
  my_view_db = new char[strlen(view_db_p) + 1];
  strcpy(my_view_db, view_db_p);
}

View_I::~View_I() {

  delete [] my_view_db;
  destroy_corba_objects();
  delete ps_dev;
}

// Operations of the Scheduler_Caller interface (implementation interface)

void View_I::challenge_reply (const ABng_Core_System::Sensor_Id& sensor, 
			      const ABng_Core_System::Badge_Id& badge, 
			      const ABng_Core_System::Octets& random, 
			      const ABng_Core_System::Time& ts,
			      CORBA::Long request_id,
			      CORBA::Long repeat_num
			      ENV_DEF( IT_env) ) {

  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->challenge_reply( my_location_rep.sensor_id2location( sensor),
			     my_locatable_rep.badge_id2locatable( badge),
			     random,
			     ts,
			     request_id,
			     repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::sound_reply (const ABng_Core_System::Sensor_Id& sensor, 
			  const ABng_Core_System::Badge_Id& badge, 
			  const ABng_Core_System::Octets& sound, 
			  const ABng_Core_System::Time& ts, 
			  CORBA::Long request_id, 
			  CORBA::Long repeat_num
			  ENV_DEF( IT_env) ) {

  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->sound_reply( my_location_rep.sensor_id2location( sensor),
			 my_locatable_rep.badge_id2locatable( badge),
			 sound,
			 ts,
			 request_id,
			 repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::set_state_reply (const ABng_Core_System::Sensor_Id& sensor, 
			      const ABng_Core_System::Badge_Id& badge, 
			      CORBA::Octet status, 
			      const ABng_Core_System::Octets& sound, 
			      const ABng_Core_System::Time& ts, 
			      CORBA::Long request_id, 
			      CORBA::Long repeat_num
			      ENV_DEF( IT_env) ) {
  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->set_state_reply( my_location_rep.sensor_id2location( sensor),
			     my_locatable_rep.badge_id2locatable( badge),
			     status,
			     sound,
			     ts,
			     request_id,
			     repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::get_state_reply (const ABng_Core_System::Sensor_Id& sensor, 
			      const ABng_Core_System::Badge_Id& badge, 
			      CORBA::Octet major_version, 
			      CORBA::Octet minor_version, 
			      CORBA::Octet status, 
			      const ABng_Core_System::Time& ts, 
			      CORBA::Long request_id, 
			      CORBA::Long repeat_num
			      ENV_DEF( IT_env) ) {

  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->get_state_reply( my_location_rep.sensor_id2location( sensor),
			     my_locatable_rep.badge_id2locatable( badge),
			     major_version,
			     minor_version,
			     status,
			     ts,
			     request_id,
			     repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::set_home_reply (const ABng_Core_System::Sensor_Id& sensor, 
			     const ABng_Core_System::Badge_Id& badge, 
			     const ABng_Core_System::Octets& address, 
			     const ABng_Core_System::Time& ts, 
			     CORBA::Long request_id, 
			     CORBA::Long repeat_num
			     ENV_DEF( IT_env) ) {

  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->set_home_reply( my_location_rep.sensor_id2location( sensor),
			    my_locatable_rep.badge_id2locatable( badge),
			    address,
			    ts,
			    request_id,
			    repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::get_home_reply (const ABng_Core_System::Sensor_Id& sensor, 
			     const ABng_Core_System::Badge_Id& badge, 
			     const ABng_Core_System::Octets& address, 
			     const ABng_Core_System::Time& ts, 
			     CORBA::Long request_id, 
			     CORBA::Long repeat_num
			     ENV_DEF( IT_env) ) {
  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->get_home_reply( my_location_rep.sensor_id2location( sensor),
			    my_locatable_rep.badge_id2locatable( badge),
			    address,
			    ts,
			    request_id,
			    repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::get_station_state_reply (const ABng_Core_System::Sensor_Id& sensor,
				      CORBA::Octet major_version, 
				      CORBA::Octet minor_version, 
				      CORBA::Octet status1, 
				      CORBA::Octet status2, 
				      const ABng_Core_System::Time& ts, 
				      CORBA::Long request_id, 
				      CORBA::Long repeat_num
				      ENV_DEF( IT_env) ) {

  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->get_station_state_reply( my_location_rep.sensor_id2location( sensor),
				     major_version,
				     minor_version,
				     status1,
				     status2,
				     ts,
				     request_id,
				     repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::sensor_command_reply( const ABng_Core_System::Sensor_Id& sensor,
				   const ABng_Core_System::Octets &reply_body,
				   const ABng_Core_System::Time &ts,
				   CORBA::Long request_id,
				   CORBA::Long repeat_num
				   ENV_DEF( IT_env) ) {

  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->location_command_reply( 
			       my_location_rep.sensor_id2location( sensor),
			       reply_body,
			       ts,
			       request_id,
			       repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::badge_command_reply( const ABng_Core_System::Sensor_Id& sensor,
				  const ABng_Core_System::Badge_Id& badge,
				  const ABng_Core_System::Octets &reply_body,
				  const ABng_Core_System::Time &ts,
				  CORBA::Long request_id,
				  CORBA::Long repeat_num
				  ENV_DEF( IT_env) ) {

  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->locatable_command_reply( 
			   my_location_rep.sensor_id2location( sensor),
			   my_locatable_rep.badge_id2locatable( badge),
			   reply_body,
			   ts,
			   request_id,
			   repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::null_message (const ABng_Core_System::Sensor_Id& sensor, 
			   const ABng_Core_System::Time& ts, 
			   CORBA::Long request_id,
			   CORBA::Long repeat_num
			   ENV_DEF( IT_env) ) {
}

void View_I::error_occured (const ABng_Core_System::Sensor_Id& sensor,
			    const ABng_Core_System::Badge_Id& badge,
			    ABng_Core::Command_Error_Code error_code,
			    CORBA::Long request_id,
			    CORBA::Long repeat_num,
			    CORBA::Long retry_num
			    ENV_DEF( IT_env) ) {
  // badge paramater for commands not concerning badges ?
  //
  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    ABng_Core::Location_var location;
    ABng_Core::Locatable_var locatable;

    try {
      location= my_location_rep.sensor_id2location( sensor);
    }
    catch (ABng_Core::Invalid_Argument &ex1) {}
    try {
      locatable= my_locatable_rep.badge_id2locatable( badge);
    }
    catch (ABng_Core::Invalid_Argument &ex2) {}

    caller->error_occured( location,
			   locatable,
			   error_code,
			   request_id,
			   repeat_num,
			   retry_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::request_processed (const ABng_Core_System::Sensor_Id& sensor,
				const ABng_Core_System::Badge_Id& badge,
				const ABng_Core_System::Time& ts, 
				CORBA::Long request_id,
				CORBA::Long repeat_num
				ENV_DEF( IT_env) ) {
  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    ABng_Core::Location_var location;
    ABng_Core::Locatable_var locatable;

    try {
      location= my_location_rep.sensor_id2location( sensor);
    }
    catch (ABng_Core::Invalid_Argument &ex1) {}
    try {
      locatable= my_locatable_rep.badge_id2locatable( badge);
    }
    catch (ABng_Core::Invalid_Argument &ex2) {}

    caller->request_processed( location,
			       locatable,
			       ts,
			       request_id,
			       repeat_num);
  }
  catch (ABng_Core::Invalid_Argument &ex) {}
  catch (...) {
    request_mapper.remove_request( request_id);
  }
}

void View_I::request_completed( CORBA::Long request_id
				ENV_DEF( IT_env) ) {
  ABng_Core::Caller_var caller= request_mapper.get_caller( request_id);
  if ( CORBA::is_nil( (ABng_Core::Caller_ptr) caller)) return;
  try {
    caller->request_completed( request_id);
  }
  catch (...) {}
  request_mapper.remove_request( request_id);  
}

// Operations of the Request_Queue interface.



ABng_Core::Request_Queue::Requests* View_I::map_scheduler_requests(
    ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests *sreqs) {

  ABng_Core::Request_Queue::Requests *ret=
    new ABng_Core::Request_Queue::Requests( sreqs->length());
  for ( int i=0; i<sreqs->length(); i++) {
    (*ret)[i].uid=             (*sreqs)[i].uid;
    (*ret)[i].request_id=      (*sreqs)[i].request_id;
    (*ret)[i].location_list=
      my_location_rep.sensor_ids2locations( & (*sreqs)[i].sensors); 
    (*ret)[i].locatable_list=
      my_locatable_rep.badge_ids2locatables( & (*sreqs)[i].badges);
    (*ret)[i].arguments=       (*sreqs)[i].arguments;
    (*ret)[i].command_name=    (*sreqs)[i].command_name;
    (*ret)[i].command_ctx=     (*sreqs)[i].command_ctx;
    (*ret)[i].request_time=    (*sreqs)[i].request_time;
  }
  return ret;
}

void View_I::map_scheduler_exception() {
  try {
    throw;
  }
  catch (ABng_Core::Invalid_Request_Time&) {}
  catch (ABng_Core::Invalid_Argument&) {}
  catch (Domain_Repository::Destination_Unreachable&) {}
  catch (ABng_Core_System::Invalid_Sensor_Id&) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
  catch (ABng_Core_System::Unknown_Sensor_Id&) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
  catch (ABng_Core_System::Invalid_Badge_Id&) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
  catch (ABng_Core_System::Unknown_Badge_Id&) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
  catch (CORBA::COMM_FAILURE&) {
    throw Domain_Repository::Destination_Unreachable("scheduler");
  }
  return;
}

ABng_Core_System::Scheduler_var View_I::get_scheduler_ref() {
  if (CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) my_scheduler))
    throw Domain_Repository::Destination_Unreachable("scheduler");
  else return my_scheduler;
}

ABng_Core::Request_Queue::Requests* View_I::list_user_requests (
    const char * user_id
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests_var reqs=
        get_scheduler_ref()->list_user_requests( user_id);

      return map_scheduler_requests( reqs.operator->());
  }
  catch (CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

ABng_Core::Request_Queue::Requests* View_I::list_all_requests (
    ENV_DEF1( IT_env) ) {

  try {
    ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests_var reqs=
       get_scheduler_ref()->list_all_requests();
    return map_scheduler_requests( reqs.operator->());
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

void View_I::remove_request (CORBA::Long request_id
    ENV_DEF( IT_env) ) {

  try {
    get_scheduler_ref()->remove_request( request_id);
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

// Operations of the Commander interface.

CORBA::Long View_I::issue_location_command (
    const ABng_Core::Locations& location_list, 
    const ABng_Core_System::Octets& cmd,
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->issue_sensor_command(
	     sids,
	     cmd,
	     has_reply,
	     reply_code,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::issue_locatable_command (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core_System::Octets& cmd, 
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Badge_Ids bids=
      my_locatable_rep.locatables2badge_ids( &locatable_list);
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->issue_badge_command(
	     sids,
	     bids,
	     cmd,
	     has_reply,
	     reply_code,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::challenge (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core_System::Octets& random, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Badge_Ids bids=
      my_locatable_rep.locatables2badge_ids( &locatable_list);
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->challenge(
	     sids,
	     bids,
	     random,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::sound (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core_System::Octets& _sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to  
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Badge_Ids bids=
      my_locatable_rep.locatables2badge_ids( &locatable_list);
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->sound(
	     sids,
	     bids,
	     _sound,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::set_state (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    CORBA::Octet status, 
    const ABng_Core_System::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to  
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Badge_Ids bids=
      my_locatable_rep.locatables2badge_ids( &locatable_list);
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->set_state(
	     sids,
	     bids,
	     status,
	     sound,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::get_state (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to  
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Badge_Ids bids=
      my_locatable_rep.locatables2badge_ids( &locatable_list);
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->get_state(
	     sids,
	     bids,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

  CORBA::Long View_I::set_home (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core_System::Octets& address, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to  
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Badge_Ids bids=
      my_locatable_rep.locatables2badge_ids( &locatable_list);
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->set_home(
	     sids,
	     bids,
	     address,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

  CORBA::Long View_I::get_home (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to  
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Badge_Ids bids=
      my_locatable_rep.locatables2badge_ids( &locatable_list);
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->get_home(
	     sids,
	     bids,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::reset_buffer (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to  
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->reset_buffer(
	     sids,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::reset_fifos (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to  
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->reset_fifos(
	     sids,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::get_station_state (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to  
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->get_station_state(
	     sids,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::set_output (
    const ABng_Core::Locations& location_list, 
    CORBA::Octet output_value, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->set_output(
	     sids,
	     output_value,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

CORBA::Long View_I::set_persistence (
    const ABng_Core::Locations& location_list, 
    CORBA::Boolean mode, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  try {
    ABng_Core_System::Sensor_Ids sids= 
      my_location_rep.locations2sensor_ids( &location_list);

    CORBA::Long req_id= 
      get_scheduler_ref()->set_persistence(
	     sids,
	     mode,
	     ctx,
	     my_scheduler_caller_obj);
    Request_Record rrec( req_id);
			 
    request_mapper.add_request( rrec, 
				ABng_Core::Caller::_duplicate( reply_to));    
    return req_id;
  }
  catch ( CORBA::Exception &ex) {
    map_scheduler_exception();
    throw;
  }
}

// Operations of View:

Domain_Repository::Location_Type::Location_Type_Name View_I::type (
    ENV_DEF1( IT_env) ) {

  return my_locatable_rep.type();
}

ABng_Core::Sighting_Descriptions* View_I::get_sightings (
    const ABng_Core_System::Time& since
    ENV_DEF(env)) {

  return my_locatable_rep.get_sightings(since);
}

ABng_Core_System::Sensor_Ids* View_I::sensor_id_list (
    ENV_DEF1(env)) {

  return my_location_rep.get_sensor_list();
}

ABng_Core_System::Badge_Ids* View_I::badge_id_list (
  ENV_DEF1(env))  {

  return my_locatable_rep.get_badge_list();
}

void View_I::process_sighting (
  const ABng_Core_Internals::Internal_Sighting_Description& isd
  ENV_DEF(env)) {

  LOGTIMESTAMP( isd.update_time);
  //#ifdef DEBUG
#if 0
  {
    static struct timeval last_time= {0,0};
    static struct rusage last_usage;
    struct timeval tv;
    struct rusage ru;
    gettimeofday(&tv,NULL);
    getrusage( RUSAGE_SELF, &ru);
    
    if (!last_time.tv_sec) {
      last_time= tv;
      last_usage= ru;
    } else {
      long dur= (tv.tv_sec-last_time.tv_sec)*1000000+(last_time.tv_usec-tv.tv_usec);
      if (dur>3000000) {
	double usg= 
	  ((last_usage.ru_utime.tv_sec-ru.ru_utime.tv_sec)+
	   (last_usage.ru_stime.tv_sec-ru.ru_stime.tv_sec))*1000000+
	  (last_usage.ru_utime.tv_usec-ru.ru_utime.tv_usec)+
	  (last_usage.ru_stime.tv_usec-ru.ru_stime.tv_usec);
	cerr << usg/dur << endl;
	
	last_time= tv;
	last_usage= ru;
      }
    }
  }
  //  cout << "[" << isd.my_sensor.group_name << ",";
  
  //  cout << isd.my_sensor.network_num << ",";
  //  cout << isd.my_sensor.sensor_num << "]" << "   ";
  //  cout << "[";
  //  for (int i = 0; i < isd.my_badge.length(); i++) {
  //    cout << hex << (unsigned) isd.my_badge[i] << " ";
  //  }
  //  cout << "]" << endl;
#endif
  
  Location_I *old_location = 0;
  Location_I *location = my_location_rep.resolve_sensor(isd.my_sensor);
  Locatable_I *locatable = my_locatable_rep.resolve_badge(isd.my_badge);
  if(!locatable) {
    // It's not your badge, do nothing, discard the sighting
    LOGTIMESTAMP( isd.update_time);
    return;
  }
  if(!location) {
    // This should not happen as long as the MSP has correct dictionaries.
    // Might be case c) (see ABng document, point 6)
    // The locatable might have gone out from the view, do nothing, 
    // discard the sighting
    LOGTIMESTAMP( isd.update_time);
    return;
  }
  // Get the old location of the locatable.
  old_location = locatable->get_location();
  if(old_location && location != old_location) {
    // Case a) (see ABng document, point 6)
    old_location->process_sighting(locatable, Location_I::CAME_OUT);
    location->process_sighting(locatable, Location_I::CAME_IN);
  }
  else if(!old_location) {
    // Case b)
    location->process_sighting(locatable, Location_I::CAME_IN);
  }
  // Case a), b) and d)
  // Set the current sighting
  locatable->process_sighting(isd, location);
  // Notify dynamic data observers
  my_location_rep.dynamic_notify(location, old_location, locatable);
  my_locatable_rep.dynamic_notify(locatable);
  if (!CORBA::is_nil( channel_observer)) {
    try {
      channel_observer->receive_sighting( locatable->local_get_sighting());
    }
    catch (CORBA::Exception &ex) {
      cerr << "channel_observer failed " << ex << endl;
      channel_observer= NULL;
    }
  }
  LOGTIMESTAMP( isd.update_time);
}

void View_I::create_corba_objects() {

#ifdef ORBIX
  my_view_obj = 
    new TIE_ABng_Core_View(View_I) (this);
  my_view_manager_obj = 
    new TIE_ABng_Core_Management_View_Manager(View_I) (this);
  my_vsp_obj = 
    new TIE_ABng_Core_Internals_View_Sighting_Processor(View_I) (this);
  my_ssdo_obj = 
    new TIE_ABng_Core_System_System_Static_Data_Observer(View_I) (this);
  my_scheduler_caller_obj =
    new TIE_ABng_Core_System_Scheduler_Caller(View_I) (this);
#elif defined OMNIORB
  my_view_obj =
    new TIE_CLASS(ABng_Core,View)<View_I,0>(this);
  CONNECT_IMPL( ABng_Core::View_var,my_view_obj);
  my_view_manager_obj =
    new TIE_CLASS(ABng_Core_Management,View_Manager)<View_I,0>(this);
  CONNECT_IMPL( ABng_Core_Management::View_Manager_var,my_view_manager_obj);
  my_vsp_obj =
    new TIE_CLASS(ABng_Core_Internals,View_Sighting_Processor)<View_I,0>(this);
  CONNECT_IMPL( ABng_Core_Internals::View_Sighting_Processor_var,my_vsp_obj);
  my_ssdo_obj =
    new TIE_CLASS(ABng_Core_System,System_Static_Data_Observer)<View_I,0>(this);
  CONNECT_IMPL( ABng_Core_System::System_Static_Data_Observer_var,my_ssdo_obj);
  my_scheduler_caller_obj =
    new TIE_CLASS(ABng_Core_System,Scheduler_Caller)<View_I,0>(this);
  CONNECT_IMPL( ABng_Core_System::Scheduler_Caller_var,my_scheduler_caller_obj);

#endif
}

void View_I::destroy_corba_objects() {

  CORBA::release(my_view_obj);
  CORBA::release(my_view_manager_obj);
  CORBA::release(my_vsp_obj);
  CORBA::release(my_ssdo_obj);
  CORBA::release(my_scheduler_caller_obj);
}

void View_I::notify(
  Domain_Repository::Repository_Item_Id id,
  const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
  Domain_Repository::Change_Type change
  ENV_DEF(env)) {
#ifdef DEBUG
  cerr << "<<notify" << endl;
#endif    
  my_location_rep.notify(id, sd, change USE_ENV(env));
  my_locatable_rep.notify(id, sd, change USE_ENV(env));
#ifdef DEBUG
  cerr << ">>notify" << endl;
#endif
}

void View_I::register_my_servers( void *args) {
  View_I &view= *((View_I*) args);

  {
    ABng_S_Guarded::auto_lock local_lock(view.repos_guard);
    
    while ( CORBA::is_nil( (ABng_Core_Management::Badge_Holder_Rep_ptr) 
			   view.my_locatable_rep.get_bh_rep()) ||
	    CORBA::is_nil( (ABng_Core_Management::ABng_Location_Description_Rep_ptr)
			   view.my_location_rep.get_abng_ld_rep())) {

      unsigned long sec,nsec;
      int wait_ret;
     
      omni_thread::get_time( 	&sec, &nsec, 
				60, 0);
#ifdef OMNIORB
      wait_ret= view.repos_cond.timedwait( sec, nsec);
#else     
      wait_ret= view.repos_cond.timed_wait( sec, nsec);
#endif
      omni_thread::get_time( 	&sec, &nsec, 
				0, 0);
      if ( sec-view.last_repos_resolved > 60) // no repository resolved in last minute
	break;  // give it up, register your references
    }
  }
#ifdef DEBUG
  cerr << "Registering view references" << endl;
#endif
  view.my_locatable_rep.load_observers();
  view.my_locatable_rep.load_dynamic_observers();
  view.my_location_rep.load_observers();
  view.my_location_rep.load_dynamic_observers();

  Bootstrap::register_server_in_NS(
    Bootstrap::CTX_ABng_Core_Public_Views() + view.my_view_name, view.my_view_obj);
  Bootstrap::register_server_in_NS(
    Bootstrap::CTX_ABng_Core_System_Vsps() + view.my_view_name, view.my_vsp_obj);
  Bootstrap::register_server_in_NS(
    Bootstrap::CTX_ABng_Core_Management_View_Managers() + view.my_view_name, 
    view.my_view_manager_obj);
}

void View_I::init(int cold_start) {

  //   synch_time( 200);

#ifdef FilePersistency
  ifstream my_rep(my_view_db);
  if (!my_rep.rdbuf()->is_open()) {
    if(cold_start) {
      ofstream create_rep(my_view_db);
      save();
      if(!create_rep) {
	cerr << "Cannot write the database file:" << endl;
	cerr << "\t" << my_view_db << endl;
	exit(1);
      }
      my_rep.open(my_view_db, ios::in );
      DB_Header db_header;
      if (!(my_rep >> db_header)) {
	cerr << "Cannot read the created database file:" << endl;
	cerr << "\t" << my_view_db << endl;
	cerr << "It has improper access rigts or has been corrupted." << endl;
	exit(1);
      }
    }
    else {
      cerr << "Cannot open the database file:" << endl;
      cerr << "\t\"" << my_view_db << endl;
      cerr << "Use the cold start option to create the database."<< endl;
      exit(1);
    }
  }
  else {
    if(cold_start) {
      cerr << "Cannot perform the cold start because the database file:" << endl;
      cerr <<"\t" <<  my_view_db << endl;
      cerr << "already exists. Remove the file or start the server without the cold start" << endl;
      cerr << "option." << endl;
      exit(1);
    }
    else {
      restore(my_rep);
      if (!my_rep) {
	cerr << "Cannot read the database file:" << endl;
	cerr << "\t" << my_view_db << endl;
	cerr << "It has improper access rigts or has been corrupted." << endl;
	exit(1);
      }
    }
  }
#endif
  my_location_rep.init(cold_start);
  my_location_rep.set_burst_delay( r_burst_delay_time);
  my_locatable_rep.init(cold_start);
  my_locatable_rep.set_burst_delay( r_burst_delay_time);
  create_corba_objects();
  (new omni_thread( register_my_servers, this))->start();

  register_callback(Bootstrap::SRV_Main_Sighting_Processor());
  register_callback(Bootstrap::SRV_Scheduler());
  register_callback(Bootstrap::SRV_ABng_Location_Description_Repository());
  register_callback(Bootstrap::SRV_Badge_Holder_Repository());
  register_callback(Bootstrap::SRV_Sensor_Repository());
  register_callback(Bootstrap::SRV_Badge_Repository());

  RWCString server_name= "view_";
  server_name= server_name+my_view_name;
  ps_dev= new ProcessControlImpl( (const char*) server_name, this);
}

CORBA::ULong View_I::burst_delay_time( ENV_DEF1(env)) {
  return r_burst_delay_time;
}

void View_I::burst_delay_time( CORBA::ULong new_burst_time
			       ENV_DEF(env)) {
  r_burst_delay_time= new_burst_time;
  save();
  my_location_rep.set_burst_delay( r_burst_delay_time);
  my_locatable_rep.set_burst_delay( r_burst_delay_time);
}

CosTypedEventChannelAdmin::TypedEventChannel_ptr View_I::sighting_channel( ENV_DEF1(env)) {

  return CosTypedEventChannelAdmin::TypedEventChannel::_duplicate( channel);
}

void View_I::sighting_channel( CosTypedEventChannelAdmin::TypedEventChannel_ptr ch
			       ENV_DEF(env)) {
  
#ifdef ORBIX
  if (CORBA::is_nil(ch)) ch= NULL;
#endif

  if (!CORBA::is_nil(ppushcons)) {
    try {
      ppushcons->disconnect_push_consumer();
    }
    catch (...) {}
  }
  channel= CosTypedEventChannelAdmin::TypedEventChannel::_duplicate( ch);
  if (!CORBA::is_nil(channel)) {
    try {
      CosTypedEventChannelAdmin::TypedSupplierAdmin_var sa=
	channel->for_suppliers();
      if (CORBA::is_nil(sa)) {
	cerr << "TypeSupplierAdmin is nil" << endl;
	return;
      }
      ppushcons= sa->obtain_typed_push_consumer("");
      ppushcons->connect_push_supplier( NULL); 
      // it's allowed to pass a null reference as a push supplier
    }
    catch (CORBA::Exception &ex) {
      cerr << "Could not obtain proxy push consumer" << endl;
      cerr << ex << endl;
      return;
    }
    CORBA::Object_var objconsumer;
    try {
      objconsumer= ppushcons->get_typed_consumer();
      cerr << (const char *) 
	(Bootstrap::base_impl->orb->object_to_string( objconsumer))
	   << endl;
      channel_observer= ABng_Core::Dynamic_Data_Observer::_narrow( objconsumer);
    }
    catch (...) {
      cerr << "Unable to get a typed consumer" << endl;
      return;
    }
  }
  else channel_observer= NULL;
}

ABng_Core_System::System_Static_Data_Observer_ptr View_I::get_ssdo_ptr() {
  return my_ssdo_obj;
}


DB_Header * View_I::create_db_header() {

  return new DB_Header( "View", "1.0", "\
#       <file>::=       <view_name>\n\
#                       <wants_duplicates_flag>\n\
#\n", 1);
}

void View_I::save() {

  ofstream iostr(my_view_db);
  save(iostr);
}

void View_I::save (ostream &iostr) {

  DB_Header *db_header = create_db_header();
  iostr << *db_header;
  delete db_header;

  iostr << my_view_name << endl;
  iostr << (unsigned) my_wants_duplicates << endl;
  iostr << r_burst_delay_time << endl;
}
  

void View_I::restore (istream &iostr) {

  DB_Header db_header;
  unsigned x;
  if (!(iostr >> db_header)) return;
  iostr >> my_view_name;
  iostr >> (unsigned) x;
  my_wants_duplicates = (CORBA::Boolean)x;
  iostr >> r_burst_delay_time;
}

void View_I::resolve_abng_ld_rep(
  ABng_Core_Management::ABng_Location_Description_Rep_var abng_ld_rep_p) {

  my_location_rep.resolve_abng_ld_rep(abng_ld_rep_p);
}

void View_I::resolve_bh_rep(
  ABng_Core_Management::Badge_Holder_Rep_var bh_rep_p) {

#ifdef DEBUG
#ifdef ORBIX
  cerr << "bh_rep refcount(1) " << bh_rep_p->_refCount() << endl;
#endif
#endif
  my_locatable_rep.resolve_bh_rep(bh_rep_p);
#ifdef DEBUG
#ifdef ORBIX
  cerr << "bh_rep refcount(1) " << bh_rep_p->_refCount() << endl;
#endif
#endif 
}

void View_I::resolve_sensor_rep(
  ABng_Core_Management::Sensor_Rep_var sensor_rep_p) {

  my_location_rep.resolve_sensor_rep(sensor_rep_p);
}

void View_I::resolve_badge_rep(
  ABng_Core_Management::Badge_Rep_var badge_rep_p) {

  my_locatable_rep.resolve_badge_rep(badge_rep_p);
}

void View_I::resolve_msp(
  ABng_Core_Internals::Main_Sighting_Processor_var msp_p) {

  my_msp = msp_p;
  my_location_rep.resolve_msp(my_msp);
  my_locatable_rep.resolve_msp(my_msp);
  ABng_Core_System::Sensor_Ids_var sids = my_location_rep.get_sensor_list();
  ABng_Core_System::Badge_Ids_var bids = my_locatable_rep.get_badge_list();
  try {
    my_msp->remove_vsp( my_view_name);
  }
  catch ( ABng_Core::Unknown_View_Name &) {}
  my_msp->add_vsp(my_view_name, 
    ABng_Core_Internals::View_Sighting_Processor::_duplicate(my_vsp_obj), 
    *(bids.operator->()), *(sids.operator->()), my_wants_duplicates);
}

void View_I::resolve_scheduler(
  ABng_Core_System::Scheduler_var scheduler_p) {

  my_scheduler= scheduler_p;
}

void View_I::reference_resolved(
  const ABng_Server_Manager::Name &server_name, 
  CORBA::Object_var server_ref,
  ABng_Server_Manager::uid callback_id) {

  ABng_S_Guarded::auto_lock local_lock( repos_guard);

#ifdef DEBUG
  cerr << "<<reference_resolved" << endl;
#endif
  if (server_name == Bootstrap::SRV_Main_Sighting_Processor()) {
    ABng_Core_Internals::Main_Sighting_Processor_var msp;
    try {
      msp = 
        ABng_Core_Internals::Main_Sighting_Processor::_narrow(server_ref);
    }
    catch(...) {
      return;
    }
#ifdef DEBUG
    cout << "Connected to the MSP " << server_name << endl;
#endif
    resolve_msp(msp);
  }
  if (server_name == Bootstrap::SRV_Scheduler()) {
    ABng_Core_System::Scheduler_var sch;
    try {
      sch= ABng_Core_System::Scheduler::_narrow( server_ref);
      if ( CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch)) return;
    }
    catch (...) {
      return;
    }
#ifdef DEBUG
    cout << "Got scheduler reference" << endl;
#endif
    resolve_scheduler( sch);
  }
  if (server_name == Bootstrap::SRV_ABng_Location_Description_Repository()) {
    ABng_Core_Management::ABng_Location_Description_Rep_var abng_ld_rep;
    try {
      abng_ld_rep = 
        ABng_Core_Management::ABng_Location_Description_Rep::_narrow(
          server_ref);
    }
    catch(...) {
      return;
    }
#ifdef DEBUG
    cout << "Connected to the ABng_Location_Description_Rep " << server_name << endl;
#endif
    resolve_abng_ld_rep(abng_ld_rep);
    omni_thread::get_time( &last_repos_resolved, &last_repos_resolved_us, 
			   0, 0);
    repos_cond.signal();
  }
  if (server_name == Bootstrap::SRV_Badge_Holder_Repository()) {
    ABng_Core_Management::Badge_Holder_Rep_var bh_rep;
    try {
      bh_rep = 
        ABng_Core_Management::Badge_Holder_Rep::_narrow(
          server_ref);
    }
    catch(...) {
      return;
    }
#ifdef DEBUG
    cout << "Connected to the Badge_Holder_Rep " << server_name << endl;
#endif
    resolve_bh_rep(bh_rep);
    omni_thread::get_time( &last_repos_resolved, &last_repos_resolved_us, 
			   0, 0);
    repos_cond.signal();
  }

  if (server_name == Bootstrap::SRV_Sensor_Repository()) {
    ABng_Core_Management::Sensor_Rep_var sensor_rep;
    try {
      sensor_rep = 
        ABng_Core_Management::Sensor_Rep::_narrow(server_ref);
    }
    catch(...) {
      return;
    }
#ifdef DEBUG    
    cout << "Connected to the Sensor_Rep " << server_name << endl;
#endif
    resolve_sensor_rep(sensor_rep);
  }

  if (server_name == Bootstrap::SRV_Badge_Repository()) {
    ABng_Core_Management::Badge_Rep_var badge_rep;
    try {
      badge_rep = 
        ABng_Core_Management::Badge_Rep::_narrow(server_ref);
    }
    catch(...) {
      return;
    }
#ifdef DEBUG    
    cout << "Connected to the Badge_Rep " << server_name << endl;
#endif
    resolve_badge_rep(badge_rep);
  }
#ifdef DEBUG
  cerr << ">>reference_resolved" << endl;
#endif
}






