// $Id: user_i.cc,v 1.3 1998/11/30 14:55:11 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/user_rep/user_i.cc,v $
//
// $Log$
//

//
//  File:         user_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 28.02.98
//
// General Description:
//   Implementation file for the User_I class
//

#define EXCEPTIONS
#include "user_i.h"
#include "db_utils.hh"

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

//
// Implementation of constructor, destructor and operators
//                   _____________________________________

  User_I::User_I (
	const Domain_Repository::User::Description& descr_init
  ) : 
	my_guard(*(User_Rep_I::instance()))
{
      	// init my_descr
	my_descr.item_id = descr_init.item_id;
	my_descr.uid = ABng_String::copy_string(descr_init.uid);
	my_descr.forename = ABng_String::copy_string(descr_init.forename);
	my_descr.surname = ABng_String::copy_string(descr_init.surname);
	my_descr.position = ABng_String::copy_string(descr_init.position);
	my_descr.email = ABng_String::copy_string(descr_init.email);
	my_descr.phone = ABng_String::copy_string(descr_init.phone);

	// init my_corba_object
	my_corba_object = (Domain_Repository::User_ptr)NULL;
}

#ifdef FilePersistency
   User_I::User_I (
	istream &iostr
   ) :
	my_guard(*(User_Rep_I::instance()))
{
	// read description from the file
	restore(iostr);

	// init my_corba_object
	my_corba_object = (Domain_Repository::User_ptr)NULL;
}
#endif


  User_I::~User_I (
  )
{
#ifdef DEBUG
  cerr << "~User_I" << endl;
#endif
  //	ABng_String::free_string(my_descr.uid);
  //	ABng_String::free_string(my_descr.forename);
  //	ABng_String::free_string(my_descr.surname);
  //	ABng_String::free_string(my_descr.position);
  //	ABng_String::free_string(my_descr.email);
  //	ABng_String::free_string(my_descr.phone);
}

  User_I::operator== (
	const User_I& user_p
  )
{
  return !strcmp( my_descr.uid, user_p.my_descr.uid);
}

//
// Implementation of methods enabling access to the state
//

void
  User_I::set_descr (
	const Domain_Repository::User::Description& descr_update
  )
{       
	// update my_descr
	/// NO !!!! my_descr.item_id = _descr_update.item_id;
#ifdef DEBUG
  cerr << "User::set_descr " << my_descr.uid << endl;
#endif
  //  	ABng_String::free_string(my_descr.uid);
	my_descr.uid = ABng_String::copy_string(descr_update.uid);

	//	ABng_String::free_string(my_descr.forename);
	my_descr.forename = ABng_String::copy_string(descr_update.forename);

	//	ABng_String::free_string(my_descr.surname);
	my_descr.surname = ABng_String::copy_string(descr_update.surname);

	//	ABng_String::free_string(my_descr.position);
	my_descr.position = ABng_String::copy_string(descr_update.position);

	//	ABng_String::free_string(my_descr.email);
	my_descr.email = ABng_String::copy_string(descr_update.email);

	//	ABng_String::free_string(my_descr.phone);
	my_descr.phone = ABng_String::copy_string(descr_update.phone);
}

const Domain_Repository::User::Description&
   User_I::get_descr (
   ) const
{
	return my_descr;  
}

void
   User_I::set_corba_object (
	const Domain_Repository::User_ptr corba_object_p
   )
{
	my_corba_object = corba_object_p;
}

const Domain_Repository::User_ptr
   User_I::get_corba_object (
   ) 
{
	return my_corba_object;
}

void 
   User_I::set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   )
{
	my_descr.item_id = item_id_p;
}

//
// Implementation of IDL operations of the Static_Data_Observed_E interface
//                                         ______________________

Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   User_I::static_data_observer_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	return User_Rep_I::instance()->static_data_observer_list(
		my_descr.item_id USE_ENV(env));
}
                
void 
   User_I::attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs  
	ENV_DEF(env)
   )
		throw (CORBA::SystemException)
{
  	// initialize the sequence for this user item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this user item id into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
	User_Rep_I::instance()->attach_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

void 
   User_I::detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer)
{
	// initialize the sequence for this user item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this user item it into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
	User_Rep_I::instance()->detach_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

//
// Implementation of IDL operations of the User interface
//                                         ____
 
Domain_Repository::User::Description* 
  User_I::descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	// duplicate my_descr
	Domain_Repository::User::Description* my_descr_duplicated =
		new Domain_Repository::User::Description(my_descr);

	return my_descr_duplicated;
}

Domain_Repository::User::Value_Description* 
  User_I::value_descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	// duplicate my_descr
	Domain_Repository::User::Value_Description* my_value_descr_duplicated =
		new Domain_Repository::User::Description(my_descr);

	return my_value_descr_duplicated;
}
 
Domain_Repository::Repository_Item_Id 
   User_I::item_id (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	// no lock here: item_id is immutable

	return my_descr.item_id;
}

char* 
   User_I::uid (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.uid);
}

char* 
   User_I::forename (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.forename);
}

char* 
   User_I::surname (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.surname);
}

char* 
   User_I::position (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.position);
}

char* 
   User_I::email (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.email);
}

char* 
   User_I::phone (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.phone);
}

#ifdef FilePersistency
//
//		Store and restore state methods
//		_________________

void
   User_I::save (
	ostream &iostr
   )
{
  iostr << my_descr.item_id  << ' ';
  save_string(iostr, my_descr.uid); iostr << ' ';
  save_string(iostr, my_descr.forename); iostr << ' ';
  save_string(iostr, my_descr.surname); iostr << ' ';
  save_string(iostr, my_descr.position); iostr << ' ';
  save_string(iostr, my_descr.email); iostr << ' ';
  save_string(iostr, my_descr.phone); iostr << ' ';
  iostr << endl;
}

void
   User_I::restore (
	istream &iostr
   )
{
  char *tmp;
  iostr >> my_descr.item_id;
  restore_string( iostr, tmp);
  my_descr.uid= tmp;
  restore_string( iostr, tmp);
  my_descr.forename= tmp;
  restore_string( iostr, tmp);
  my_descr.surname= tmp;
  restore_string( iostr, tmp);
  my_descr.position= tmp;
  restore_string( iostr, tmp);
  my_descr.email= tmp;
  restore_string( iostr, tmp);
  my_descr.phone= tmp;
}
#endif /* FilePersistency */


