// $Id: user_rep_i.cc,v 1.3 1998/11/30 15:01:02 mencnar Exp $
// $Source: /home/iris/grant/abng/CVS/sources/servers/user_rep/user_rep_i.cc,v $
// $Log: user_rep_i.cc,v $
// Revision 1.3  1998/11/30 15:01:02  mencnar
// simplified hashing function
//
//

//
//  File:         user_rep_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 03.10.97
//
// General Description:
//   Implementation file for the User_Rep_I class
//

#define EXCEPTIONS

#include "user_rep_i.h"
#include "Naming.hh"
#include "db_utils.hh"

#include <rw/cstring.h>

//
// 		Constructor
//              ___________

   User_Rep_I::User_Rep_I (

#ifdef ObjectStore
    	os_database* db_p
#elif defined FilePersistency
    	const char *repository_name_p
#endif
    ) : Basic_Rep_I<User_Types, U_SDO_Types>(hash_user,
#ifdef ObjectStore
    		db_p
#elif defined FilePersistency
    		repository_name_p
#endif
        ) 
{
  	_instance = this;
  	create_corba_objects();
}

void 
   User_Rep_I::init (
	int cold_start_p
   ) 
{
	Basic_Rep_I<User_Types, U_SDO_Types>::init(cold_start_p);

	try {
	  ABng_Server_Manager::Name n1=
	    Bootstrap::SRV_User_Repository_Manager();
	  ABng_Server_Manager::Name n2=
	    Bootstrap::SRV_User_Repository();

#ifdef DEBUG
	  cerr << n1.length() << ' ' << n2.length() << endl;
#endif
	  Bootstrap::register_server_in_NS(n1, my_rep_manager);
	  Bootstrap::register_server_in_NS(n2, my_rep);
	  ps_dev= new ProcessControlImpl( "user_repository", this);
	}
	catch (CORBA::Exception &ex) {
		cerr << "Caught exception while contacting server manager" << ex << endl;
		exit( 1);
	}
}

void 
   User_Rep_I::init_corba_object ( 
	User_I *item_p
   ) 
{
	 // create the TIE (CORBA) object for the C++ object
#ifdef ORBIX
#ifdef ObjectStore
	ObjectStoreTIE_Domain_Repository_User(User_I)* new_user_tie =
		new(my_db, ObjectStoreTIE_Domain_Repository_User(User_I)::get_os_typespec())
		ObjectStoreTIE_Domain_Repository_User(User_I) (item_p);
#else
	TIE_Domain_Repository_User( User_I)* new_user_tie = 
		new TIE_Domain_Repository_User(User_I) (item_p);
#endif
#elif defined OMNIORB
  Domain_Repository::User_ptr new_user_tie=
    new TIE_CLASS(Domain_Repository,User)<User_I,0>(item_p);
  CONNECT_IMPL( Domain_Repository::User_var, new_user_tie);
	

#endif
	item_p->set_corba_object(new_user_tie);
}

void 
   User_Rep_I::create_corba_objects (
   ) 
{
#ifdef ORBIX
#ifdef FilePersistency
	my_rep_manager = new TIE_Domain_Repository_Management_User_Rep_Manager(User_Rep_I) (this);
	// create a TIE for the object
	my_rep = new TIE_Domain_Repository_User_Rep(User_Rep_I) (this);
	write_ior(my_rep_manager, "./User_Rep_iiop.ref");
#endif /* FilePersistency */
#elif defined OMNIORB
  my_rep =
    new TIE_CLASS(Domain_Repository,User_Rep)<User_Rep_I,0>(this);
  CONNECT_IMPL( Domain_Repository::User_Rep_var,my_rep);
  my_rep_manager =
    new TIE_CLASS(Domain_Repository_Management,User_Rep_Manager)<User_Rep_I,0>(this);
  CONNECT_IMPL( Domain_Repository_Management::User_Rep_Manager_var,my_rep_manager);
#endif
}

User_Rep_I* User_Rep_I::_instance = 0;

User_Rep_I* 
   User_Rep_I::instance(
   )
{
   	return _instance;
}

//
// 		Destructor
//		__________

   User_Rep_I::~User_Rep_I ()
{
  	destroy_corba_objects();
	delete ps_dev;
}

void 
   User_Rep_I::destroy_corba_objects (
   ) 
{ 
	CORBA::release(my_rep_manager);
	CORBA::release(my_rep);
}

//
// Private functions
//

void 
   User_Rep_I::reference_resolved ( 
	const ABng_Server_Manager::Name &name,
	CORBA::Object_var server_ref,
	ABng_Server_Manager::uid callback_id
   ) 
{
	auto_lock local_lock(*this);
}

DB_Header* 
   User_Rep_I::create_db_header (
   )
{
	return new DB_Header("User Repository", "1.0", "\
#       <data_record>::=  	<item_id> <uid> <forename> <surname>\n\
#				<position> <email> <phone>\n\
#\n\
#       <uid>::=		<string>\n\
#	<forename>::=		<string>\n\
#	<surname>::=		<string>\n\
#	<position>::=		<string>\n\
#	<email>::=		<string>\n\
#	<phone>::=		<string>\n\
#\n\
");
}

unsigned 
   User_Rep_I::hash_user (
	const User_I& user_i_p
   )
{
	Domain_Repository::User::Description user_descr = 
		user_i_p.get_descr();
	RWCString _uid(user_descr.uid);

	return _uid.hash();
}

void
   User_Rep_I::gather_user (
	User_I *current_user,
	void* apply_data
   )
{
	// project apply_data (void*) to Users*
	Domain_Repository::Users *collected_users =
		(Domain_Repository::Users*)apply_data;

	// increase the length of the container
	collected_users->length(collected_users->length() + 1);

	// put duplicated user (CORBA object) into the container
	(*collected_users)[collected_users->length() - 1] =
		Domain_Repository::User::_duplicate(current_user->get_corba_object());
}

void
   User_Rep_I::match_user(
	User_I *current_user,
	void* apply_data
   )
{
	const Domain_Repository::User::Pattern& data_record_pattern = 
		((struct MatchUserApplyDataStruct*)apply_data)->_data_record_pattern;

	Domain_Repository::User::Description current_user_descr;

	// check general patten matching rule
	if (data_record_pattern.is_any){
		// any user matches this pattern
		gather_user(current_user, 
			((struct MatchUserApplyDataStruct*)apply_data)->_found_users);
		return;
	}

	current_user_descr = current_user->get_descr();

	// check matching of item_id
	if (!data_record_pattern.item_id_pattern.is_any && 
		(data_record_pattern.item_id_pattern.value != current_user_descr.item_id))
		// this user item_id does not match pattern 
		return;
		
	// check matching of uid
	if (!data_record_pattern.uid_pattern.is_any)
		if(!ABng_String::match_string(data_record_pattern.uid_pattern.value, 
			current_user_descr.uid))
		return;

	// check matching of forename
	if (!data_record_pattern.forename_pattern.is_any)
		if(!ABng_String::match_string(data_record_pattern.forename_pattern.value, 
			current_user_descr.forename))
		return;

	// check matching of surname
	if (!data_record_pattern.surname_pattern.is_any)
		if(!ABng_String::match_string(data_record_pattern.surname_pattern.value, 
			current_user_descr.surname))
		return;

	// check matching of position
	if (!data_record_pattern.position_pattern.is_any)
		if(!ABng_String::match_string(data_record_pattern.position_pattern.value, 
			current_user_descr.position))
		return;

	// check matching of email
	if (!data_record_pattern.email_pattern.is_any)
		if(!ABng_String::match_string(data_record_pattern.email_pattern.value, 
			current_user_descr.email))
		return;

	// check matching of phone
	if (!data_record_pattern.phone_pattern.is_any)
		if(!ABng_String::match_string(data_record_pattern.phone_pattern.value, 
			current_user_descr.phone))
		return;

	// this user matches the pattern
	gather_user(current_user, 
		((struct MatchUserApplyDataStruct*)apply_data)->_found_users);
}

void 
   User_Rep_I::construct_data_object (
	Domain_Repository::Static_Data_Observer::Static_Data_Object& data_object_p,
	Domain_Repository::User_ptr item_p
   ) 
{
	// construct union to store reference of added entity
	data_object_p.user_obj(Domain_Repository::User::_duplicate(item_p));
}

void 
   User_Rep_I::collect_values ( 
	User_I *lt, 
	Domain_Repository::User::Value_Description& ld
   ) 
{
	ld = lt->get_descr();
}

//
// Implementation of IDL operations of the System_Static_Data_Observed_R interface
//                                         _____________________________

 Domain_Repository::Static_Data_Observed_R::Observer_Entries*
   User_Rep_I::static_data_observer_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{	
	return observer_list(USE_ENV1(env));
}

 Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   User_Rep_I::add_static_data_observer_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	return add_observer_list(USE_ENV1(env));
}
                        
void 
   User_Rep_I::attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DEF(env)
   )
		throw (CORBA::SystemException, 
                      Domain_Repository::Unknown_Repository_Item_Ids)
{
	attach_observer(obs, observed_id_list USE_ENV(env));
}

void 
   User_Rep_I::detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs, 
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids, 
                       Domain_Repository::Unknown_Observer,
		       Domain_Repository::Unmatched_Observer_Item_Ids)
{
	detach_observer(obs, observed_id_list USE_ENV(env));
}

void 
   User_Rep_I::attach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DEF(env)
   )
		throw (CORBA::SystemException)
{
	attach_add_observer(obs USE_ENV(env));
}

void 
   User_Rep_I::detach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer)
{
	detach_add_observer(obs USE_ENV(env));
}

// Suplementary operation

 Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   User_Rep_I::static_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException)
{
	return get_item_observers(item_id_p USE_ENV(env));
}

//
// Implementation of IDL operations of the User_Rep interface
//                                         __________

Domain_Repository::User_Rep*
   User_Rep_I::rep(
	ENV_DEF1(env)
   ) 
  	throw (CORBA::SystemException) 
{
    return Domain_Repository::User_Rep::_duplicate(my_rep);

}

Domain_Repository::Users* 
   User_Rep_I::user_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	return item_list(USE_ENV1(env));
}
                
 Domain_Repository::User_ptr 
   User_Rep_I::add (
	const Domain_Repository::User::Description& data_record
	ENV_DEF(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Duplicate_Data_Record)
{
	return Basic_Rep_I<User_Types, U_SDO_Types>::add(data_record USE_ENV(env));
}

void 
   User_Rep_I::remove (
	Domain_Repository::User_ptr object_to_remove 
	ENV_DEF(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Object_Ref)
{
	Basic_Rep_I<User_Types, U_SDO_Types>::remove(object_to_remove USE_ENV(env));
}

Domain_Repository::Users* 
   User_Rep_I::find (
	const Domain_Repository::User::Pattern& data_record_pattern
	ENV_DEF(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented)
{
  	auto_lock local_lock(*this);

  	return private_find(data_record_pattern USE_ENV(env));
}

Domain_Repository::Users* 
   User_Rep_I::private_find (
	const Domain_Repository::User::Pattern& data_record_pattern
	ENV_DEF(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented)
{
	// create a structure consisting of a pointer to the container
	//  and a reference of the pattern
	MatchUserApplyDataStruct apply_data = MatchUserApplyDataStruct(
		(Domain_Repository::Users*)new 
			Domain_Repository::Users(my_item_list.entries()),
			data_record_pattern);
#ifdef DEBUG
	cerr << "private_find...apply" << endl;
#endif
	// browse through hash table to collect users
	my_item_list.apply(match_user, (void*)&apply_data);

	// pointer to found_users was stored inside apply_data thus return it
	return apply_data._found_users;
}

 Domain_Repository::User::Value_Descriptions* 
   User_Rep_I::find_values (
	const Domain_Repository::User::Pattern& data_record_pattern
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented)
{
	return Basic_Rep_I<User_Types, U_SDO_Types>::find_values(data_record_pattern USE_ENV(env));
}

void 
   User_Rep_I::update (
	Domain_Repository::User_ptr object_to_update, 
	const Domain_Repository::User::Description& data_record_pattern
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
		       Domain_Repository::Unknown_Object_Ref,
		       Domain_Repository::Duplicate_Data_Record)
{
	Basic_Rep_I<User_Types, U_SDO_Types>::update(object_to_update, data_record_pattern USE_ENV(env));
}
