// -*- Mode: C++; -*-
//
//  File:         user_rep_i.h
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 29.01.98
//
// General Description:
//   Header file for User_Rep implementation class
//

#ifndef user_rep_i_h
#define user_rep_i_h

#include "user_i.h"
#include "basic_rep_i.h"
#include "bootstrap_n.hh"
#include "process_status.hh"

class U_SDO_Types {

public:
  	typedef Domain_Repository::Static_Data_Observed_R Observed;
  	typedef Domain_Repository::Static_Data_Observer Observer;
  	typedef Domain_Repository::Static_Data_Observer_ptr Observer_ptr;
  	typedef Domain_Repository::Static_Data_Observed_E::Static_Data_Observers
	Observers;
};

class User_Types {

public:
	typedef Domain_Repository::User Item;  
	typedef Domain_Repository::User_ptr Item_ptr;  
  	typedef Domain_Repository::Users Items;  
  	typedef User_I Item_I;  
  	typedef Domain_Repository::User::Description Initial_Data;
  	typedef Domain_Repository::User::Description Description;
  	typedef Domain_Repository::Static_Data_Observer::
		Static_Data_Object Data_Object;
};

class User_Rep_I : public Bootstrap_listener,
                   public Basic_Rep_I<User_Types, U_SDO_Types>,
		   public ServerStatusInterface   
{
 public:
  // ctor:
  User_Rep_I (
#ifdef ObjectStore
	os_database* db
#elif defined FilePersistency
    	const char *repository_name_p
#endif
  );

  virtual void 
   init (
	int cold_start_p = 0
   );

  // get instance of this class 
  //  - Singleton pattern (E. Gamm & al.,"Design Paterns", p. 127)
  static User_Rep_I* 
    instance ();

  // dctor:
  virtual 
     ~User_Rep_I ();

 private:
  // pointer to an unique istance of the class
  static User_Rep_I* _instance;

  // repository
  Domain_Repository_Management::User_Rep_Manager_ptr my_rep_manager;
  Domain_Repository::User_Rep_ptr my_rep;
  ProcessControlImpl *ps_dev;


  virtual void 
   init_corba_object (
	User_I *item_p
   );

  virtual void 
   create_corba_objects (
   );

  virtual void 
   destroy_corba_objects (
   );

  void 
   reference_resolved (
	const ABng_Server_Manager::Name &name,
	CORBA::Object_var server_ref,
	ABng_Server_Manager::uid callback_id
   );

  virtual DB_Header*
   create_db_header (
   );

  // calculate the hash value of the user
  static unsigned 
   hash_user (
	const User_I& user_i_p
   ); 

  // put the current user into the container
  static void
   gather_user (
	User_I* current_user,
	void* apply_data
   );

  // definition of structure containing parameters of match_user method
  struct MatchUserApplyDataStruct {
    Domain_Repository::Users* _found_users;
    const  Domain_Repository::User::Pattern& _data_record_pattern;

    // its constructor
    MatchUserApplyDataStruct (
	Domain_Repository::Users* _found_users_p,
   	const Domain_Repository::User::Pattern& _data_record_pattern_p
    ):
	 	_found_users(_found_users_p),
		_data_record_pattern(_data_record_pattern_p) {}
  };

  // if the current user matches the pattern puts it into the container
  static void
   match_user (
	User_I* current_user,
	void* apply_data
   );
 
  virtual void 
   construct_data_object (
    	Domain_Repository::Static_Data_Observer::Static_Data_Object& data_object_p,
    	Domain_Repository::User_ptr item_p
   );

  virtual void 
   collect_values (
	User_I *lt, 
    	Domain_Repository::User::Value_Description& ld
   );

 public:
  //
  // IDL operations of the Static_Data_Observed_R interface
  //                       ______________________

  virtual Domain_Repository::Static_Data_Observed_R::Observer_Entries*
   static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
             
  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   add_static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
           
  virtual void 
   attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids);

  virtual void 
   detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs, 
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids, 
                       Domain_Repository::Unknown_Observer,
		       Domain_Repository::Unmatched_Observer_Item_Ids);
  
  virtual void 
   attach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DECL(env)
   )
		throw (CORBA::SystemException);

  virtual void 
   detach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer);

  // Suplementary operation
  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   static_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException);

  //
  // IDL operations of the User_Rep interface
  //                       __________

  virtual Domain_Repository::User_Rep*
   rep (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual Domain_Repository::Users* 
   user_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
                
  virtual Domain_Repository::User_ptr 
   add (
	const  Domain_Repository::User::Description& data_record
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Duplicate_Data_Record);

  virtual void 
   remove (
	Domain_Repository::User_ptr object_to_remove
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Object_Ref);

  virtual Domain_Repository::Users* 
   find (
	const  Domain_Repository::User::Pattern& data_record_pattern
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);

  // no locking here 
  Domain_Repository::Users* 
   private_find (
	const Domain_Repository::User::Pattern& data_record_pattern
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);
  
  virtual Domain_Repository::User::Value_Descriptions* 
   find_values (
	const Domain_Repository::User::Pattern& data_record_pattern
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);

  virtual Domain_Repository::User::Query_Results*
   complex_query(
		 const Domain_Repository::User::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 Domain_Repository::Static_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return Basic_Rep_I<User_Types, U_SDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual void 
   update (
	Domain_Repository::User_ptr object_to_update, 
	const Domain_Repository::User::Description& data_record_pattern
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
		       Domain_Repository::Unknown_Object_Ref,
		       Domain_Repository::Duplicate_Data_Record);

#ifdef ObjectStore
  //
  // for ObjectStore
  //

  static os_typespec* get_os_typespec();
#endif
};


#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_Domain_Repository_User_Rep(User_Rep_I)
DEF_ObjectStoreTIE_Domain_Repository_Management_User_Rep_Manager(User_Rep_I)
#else
DEF_TIE_Domain_Repository_User_Rep(User_Rep_I)
DEF_TIE_Domain_Repository_Management_User_Rep_Manager(User_Rep_I)
#endif
#elif defined OMNIORB
// DEF_TIE__tie_User_Rep
// DEF_TIE__tie_User_Rep_Manager
#endif

#endif /* user_rep_i_h */
