//
//  File:         location_type_rep_observer_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 16.12.97
//
// General Description:
//   Implementation file for the Location_Type_Rep_Observer_I class
//

#include <CORBA.h>

#include "location_type_rep_observer_i.h"

extern CORBA::String_var str1;
CORBA::String_var str2;

Location_Type_Rep_Observer_I::Location_Type_Rep_Observer_I (
) {
  // init my_corba_object
  my_corba_object = (Domain_Repository::Static_Data_Observer_ptr)NULL;
}

Location_Type_Rep_Observer_I::~Location_Type_Rep_Observer_I ()
{
  CORBA::release(my_corba_object);
}

void 
 Location_Type_Rep_Observer_I::notify (
       Domain_Repository::Repository_Item_Id id,
       const Domain_Repository::Static_Data_Observer::Static_Data_Object& sd, 
       Domain_Repository::Change_Type change,
       CORBA::Environment &IT_env
)
		throw (CORBA::SystemException)
{
	int id_p;
	Location_Type_ptr location_type_p;

	switch(change) {
	    case Domain_Repository::OBJECT_ADDED:
	       cerr << "Location_Type_Rep_Observer: new location_type added" 
                    << endl;	
	       location_type_p= sd.location_type_obj();

	       // attempt to get the id of this location_type
	       try {
		   id_p = location_type_p->item_id();
	       } catch (CORBA::SystemException& se){
		   cerr << "Location_Type_Rep monitor error: ";
		   cerr << "exception when trying to get the id of the new location_type! ";
		   cerr << endl << &se << endl;
	       }
	       cerr << "Location_Type_Rep_Observer: new location_type id " 
                    << id_p << endl;

	       str2 = CORBA::Orbix.object_to_string(my_corba_object);
				      cout << strcmp(str1, str2) << endl;

	       // attempt to register observer in this location_type
	       try {
		    location_type_p->attach_static_data_observer(
                        my_corba_object
                    );
	       } catch (CORBA::SystemException& se){
		    cerr << "Location_Type_Rep monitor error: ";
		    cerr << "exception when trying to register observer for the new location_type! ";
		    cerr << endl << &se << endl;
	       }
	       cerr << "Location_Type_Rep monitor: observer for new location_type registered" << endl;
	       break;

	    case Domain_Repository::OBJECT_REMOVED:
	       cerr << "Location_Type_Rep_Observer: location_type removed" 
                    << endl;	
	       break;

	    case Domain_Repository::OBJECT_UPDATED:
	       cerr << "Location_Type_Rep_Observer: location_type updated" 
                    << endl;	
	       location_type_p= sd.location_type_obj();

	       // attempt to get the id of this location_type
	       try {
		   id_p = location_type_p->item_id();
	       } catch (CORBA::SystemException& se){
		   cerr << "Location_Type_Rep monitor error: ";
		   cerr << "exception when trying to get the id of the updated location_type! ";
		   cerr << endl << &se << endl;
	       }
	       cerr << "Location_Type_Rep_Observer: updated location_type id " << id_p << endl;
	       break;

	    default:
	       cerr << "Location_Type_Rep_Observer error: unknow change type !!!" << endl;	
	}
}

void 
   Location_Type_Rep_Observer_I::abort (
	CORBA::Environment &IT_env
   ) 
		throw (CORBA::SystemException)
{
	cerr << "Location_Type_Rep_Observer: abort called!!!" << endl 
             << endl;	
	cerr << "Location_Type_Rep_Observer: exiting!!" << endl;

	exit(0);	

}

