//
//  File:         sensor_rep_observer_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 16.12.97
//
// General Description:
//   Implementation file for the Sensor_Rep_Observer_I class
//

#include "sensor_rep_observer_i.h"

   Sensor_Rep_Observer_I::Sensor_Rep_Observer_I (
   )
{
	// init my_corba_object
	my_corba_object = (ABng_Core_System::System_Static_Data_Observer_ptr)NULL;
}

   Sensor_Rep_Observer_I::~Sensor_Rep_Observer_I ()
{
	CORBA::release(my_corba_object);
}

 void 
   Sensor_Rep_Observer_I::notify (
	Domain_Repository::Repository_Item_Id id,
	const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
	Domain_Repository::Change_Type change,
	CORBA::Environment &IT_env
   )
		throw (CORBA::SystemException)
{
	int id_p;
	Sensor_ptr sensor_p;

	switch(change) {
		case Domain_Repository::OBJECT_ADDED:
			cerr << "Sensor_Rep_Observer: new sensor added" << endl;	
			sensor_p= sd.sensor_obj();

			// attempt to get the id of this sensor
			try {
				id_p = sensor_p->item_id();
			} catch (CORBA::SystemException& se){
				cerr << "Sensor_Rep monitor error: ";
				cerr << "exception when trying to get the id of the new sensor! ";
				cerr << endl << &se << endl;
			}
			cerr << "Sensor_Rep_Observer: new sensor id " << id_p << endl;

			// attempt to register observer in this sensor
			try {
				sensor_p->attach_system_static_data_observer(my_corba_object);
			} catch (CORBA::SystemException& se){
				cerr << "Sensor_Rep monitor error: ";
				cerr << "exception when trying to register observer for the new sensor! ";
				cerr << endl << &se << endl;
			}
			cerr << "Sensor_Rep monitor: observer for new sensor registered" << endl;
			break;

		case Domain_Repository::OBJECT_REMOVED:
			cerr << "Sensor_Rep_Observer: sensor removed" << endl;	
			break;

		case Domain_Repository::OBJECT_UPDATED:
			cerr << "Sensor_Rep_Observer: sensor updated" << endl;	
			sensor_p= sd.sensor_obj();

			// attempt to get the id of this sensor
			try {
				id_p = sensor_p->item_id();
			} catch (CORBA::SystemException& se){
				cerr << "Sensor_Rep monitor error: ";
				cerr << "exception when trying to get the id of the updated sensor! ";
				cerr << endl << &se << endl;
			}
			cerr << "Sensor_Rep_Observer: updated sensor id " << id_p << endl;
			break;

		default:
			cerr << "Sensor_Rep_Observer error: unknow change type !!!" << endl;	
	}
}

 void 
   Sensor_Rep_Observer_I::abort (
	CORBA::Environment &IT_env
   ) 
		throw (CORBA::SystemException)
{
	cerr << "Sensor_Rep_Observer: abort called!!!" << endl << endl;	
	cerr << "Sensor_Rep_Observer: exiting!!" << endl;

	exit(0);	

}

