
#include <iostream.h>
#include <fstream.h>
#include "abng_server_manager.hh"
#include "compatibility.hh"
#include "bootstrap_n.hh"
#include <rw/cstring.h>
#include <omnithread.h>

class Test_Server : public Bootstrap_listener {
public:
  Test_Server() {}

  virtual void reference_resolved( const ABng_Server_Manager::Name &name,
                                   CORBA::Object_var server_ref,
				   ABng_Server_Manager::uid callback_id) {

    cerr << "callback_id " << callback_id << " " << name << " startup" << endl;
  }

  virtual void server_shutdown( const ABng_Server_Manager::Name &name,
				ABng_Server_Manager::uid callback_id) {
    cerr << "callback_id " << callback_id << " " << name << " shutdown" << endl;
  }
};


 
static void register_impl( void *arg) {
  CORBA::BOA_ptr boa_ptr= (CORBA::BOA_ptr) arg;
  try {
#ifdef ORBIX
    boa_ptr->processEvents( CORBA::ORB::INFINITE_TIMEOUT);
#elif defined OMNIORB
    boa_ptr->impl_is_ready();
#endif
  }
  catch ( CORBA::SystemException &ex) {
    cout << "Caught system exception " << ex
         << " -- impl_is_ready" << endl;
    return;
  }
}

int main( int argc, char **argv) {
  CORBA::ORB_ptr orb_ptr= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  CORBA::BOA_ptr boa_ptr= orb_ptr->BOA_init( argc, argv, BOA_IDENTIFIER);

  Bootstrap::init( orb_ptr);
  Test_Server test_server;
  //  CONNECT_IMPL(ABng_Server_Manager::Callback_var,&(test_server.callbackimpl))
  RWCString command;
  int leave= 0;
  ( new omni_thread( register_impl, boa_ptr))->start();
  ABng_Server_Manager::Callback::_duplicate( & (test_server.callbackimpl));

  do {
    cout << "? ";
    cin >> ws >> command;
    if (!cin) command= "";
    do {
      if ( command == "?" || command == "help") {
	cout << "help" << endl
	     << "quit" << endl
	     << "register_server complex_name" << endl
	     << "unregister_server complex_name" << endl
	     << "register_callback complex_name" << endl
	     << "register_context_callback complex_name" << endl
	     << "unregister_callback uid" << endl;
      }
      else if ( command == "quit") {
	leave= 1;
	break;
      }
      else if ( command == "register_server") {
	ABng_Server_Manager::Name name;
	cin >> name;
	if (!cin) break;
	cout << "server IOR file";
	char buf[4096];
	cin >> buf;
	ifstream ifs(buf);
	if (!ifs) {
	  cerr << "Could not open " << buf << endl;
	}
	else {
	  cerr << "Reading IOR" << endl;
	  ifs >> buf;
	}
	CORBA::Object_var obj;
	try {
	  obj= orb_ptr->string_to_object( buf);
	}
	catch (CORBA::Exception &ex) {
	  cerr << ex << endl;
	};
	if (CORBA::is_nil(obj))
	  break;

	try {
	  Bootstrap::register_server_in_NS( name, & (test_server.callbackimpl));
	}
	catch ( CORBA::Exception &ex) {
	  cout << "Caught " << ex << "#" << endl;
	  // break;
	}
	cout << "Server registered" << endl;
      }
      else if ( command == "unregister_server") { 
	ABng_Server_Manager::Name name;
	cin >> name;
	if (!cin) break;
	try {
	  Bootstrap::unregister_server_from_NS( name);
	}
	catch ( CORBA::Exception &ex) {
	  cout << "Caught " << ex << endl;
	  break;
	}
	cout << "Server unregistered" << endl;
      }
      else if ( command == "register_callback") {
	ABng_Server_Manager::Name name;
	ABng_Server_Manager::uid id;
	cin >> name;
	if (!cin) break;
	try {
	  id= test_server.register_callback( name);
	}
	catch ( CORBA::Exception &ex) {
	  cout << "Caught " << ex << endl;
	  break;
	}
	cout << "Callback registered, uid == " << id << endl;
      }
      else if ( command == "register_context_callback") {
	ABng_Server_Manager::Name name;
	ABng_Server_Manager::uid id;
	cin >> name;
	if (!cin) break;
	try {
	  id= test_server.register_context_callback( name);
	}
	catch ( CORBA::Exception &ex) {
	  cout << "Caught " << ex << endl;
	  break;
	}
	cout << "Callback registered, uid == " << id << endl;
      }
      else if ( command == "unregister_callback") {
	ABng_Server_Manager::uid id;
	cin >> id;
	if (!cin) break;
	try {
	  test_server.unregister_callback( id);
	}
	catch ( CORBA::Exception &ex) {
	  cout << "Caught " << ex << endl;
	  break;
	}
	cout << "Callback unregistered" << endl;
      }
      else cout << "Unknown command " << command << endl;
    } while (0);
    if (!cin) cout << "Syntax error" << endl;
    cin.clear( iostream::goodbit);
  } while (!leave);
}

