//
//  File:         user_rep_observer_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 16.12.97
//
// General Description:
//   Implementation file for the User_Rep_Observer_I class
//

#include "user_rep_observer_i.h"

   User_Rep_Observer_I::User_Rep_Observer_I (
   )
{
	// init my_corba_object
	my_corba_object = (Domain_Repository::Static_Data_Observer_ptr)NULL;
}

   User_Rep_Observer_I::~User_Rep_Observer_I ()
{
	CORBA::release(my_corba_object);
}

 void 
   User_Rep_Observer_I::notify (
	Domain_Repository::Repository_Item_Id id,
	const Domain_Repository::Static_Data_Observer::Static_Data_Object& sd, 
	Domain_Repository::Change_Type change,
	CORBA::Environment &IT_env
   )
		throw (CORBA::SystemException)
{
	int id_p;
	User_ptr user_p;

	cerr << "User_Rep_Observer: notify " << id << endl;

	switch(change) {
		case Domain_Repository::OBJECT_ADDED:
			cerr << "User_Rep_Observer: new user added" << endl;	
			user_p= sd.user_obj();

			// attempt to get the id of this user
			try {
				id_p = user_p->item_id();
			} catch (CORBA::SystemException& se){
				cerr << "User_Rep monitor error: ";
				cerr << "exception when trying to get the id of the new user! ";
				cerr << endl << &se << endl;
				break;
			}
			cerr << "User_Rep_Observer: new user id " << id_p << endl;

			// attempt to register observer in this user
			try {
				user_p->attach_static_data_observer(my_corba_object);
			} catch (CORBA::SystemException& se){
				cerr << "User_Rep monitor error: ";
				cerr << "exception when trying to register observer for the new user! ";
				cerr << endl << &se << endl;
			}
			cerr << "User_Rep monitor: observer for new user registered" << endl;
			break;

		case Domain_Repository::OBJECT_REMOVED:
			cerr << "User_Rep_Observer: user removed" << endl;	
			break;

		case Domain_Repository::OBJECT_UPDATED:
			cerr << "User_Rep_Observer: user updated" << endl;	
			user_p= sd.user_obj();

			// attempt to get the id of this user
			try {
				id_p = user_p->item_id();
			} catch (CORBA::SystemException& se){
				cerr << "User_Rep monitor error: ";
				cerr << "exception when trying to get the id of the updated user! ";
				cerr << endl << &se << endl;
				break;
			}
			cerr << "User_Rep_Observer: updated user id " << id_p << endl;
			break;

		default:
			cerr << "User_Rep_Observer error: unknow change type !!!" << endl;	
	}
}

 void 
   User_Rep_Observer_I::abort (
	CORBA::Environment &IT_env
   ) 
		throw (CORBA::SystemException)
{
	cerr << "User_Rep_Observer: abort called!!!" << endl << endl;	
	cerr << "User_Rep_Observer: exiting!!" << endl;

	exit(0);	

}

